package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import jpel.language.Environment;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeException;
import jpel.tree.NodeProcessor;
import jpel.tree.NodeReader;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderExtractor;

/**
 * Exibe as dependncias de uma rvore da linguagem funcional. Tipicamente
 * expande toda as dependncias de um DataHolder e filtra a listagem completa
 * de DataHolders j resolvida.
 */
public class DataHolderDeclaration implements DataHolderExtractor {

    public DataHolderDeclaration() {}

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName()};
    }

    public String getDescription() {
        return "Evironment extractor type";
    }

    public List extract(DataHolder holder) throws DataHolderException {
        List list = new LinkedList();
        try {
            NodeReader reader = NodeBuilder.lookupNodeReader(holder);
            Node node = reader.read(holder);
            NodeProcessor processor = NodeBuilder.lookupNodeProcessor(
                                      Environment.class.getName()
                                      + ".dependency");
            node = processor.process(node);
            extractAll(node, list, new Stack());
        }
        catch(ClassNotFoundException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
        catch(NodeException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
        return list;
    }

    private void extractAll(Node root, List result,
        Stack stack) throws DataHolderException {
        DataHolder holder;
        if(root.isRoot()) {
            holder = ((Include) root.getValue()).getHolder();
            result.add(holder);
            stack.push(holder);
        }
        else {
            holder = ((Include) root.getValue()).getHolder();
            DataHolder tmp = ((DataHolder) stack.peek()).resolve(holder);
            result.add(tmp);
            stack.push(tmp);
        }
        if(root.isFolder()) {
            Iterator iter = root.children();
            while(iter.hasNext()) {
                extractAll((Node) iter.next(), result, stack);
            }
        }
        stack.pop();
    }

    public static void main(String[] args) {
        try {
            DataHolderDeclaration dhd = new DataHolderDeclaration();
            List l = dhd.extract(new jpel.util.dataholder.DataHolderFile(
                     "examples/server.xml"));
            System.out.println("LISTA:" + l);
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }

    }
}