package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.File;
import java.net.URL;

import jpel.language.Abstraction;
import jpel.language.DeclarationException;
import jpel.language.DeclarationFunction;
import jpel.language.Environment;
import jpel.language.EnvironmentLoader;
import jpel.language.EnvironmentLoaderException;
import jpel.language.Expression;
import jpel.language.ExpressionBuilder;
import jpel.language.ExpressionId;
import jpel.language.ExpressionListImpl;
import jpel.language.ExpressionParser;
import jpel.language.ExpressionString;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeConversor;
import jpel.tree.NodeReader;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderFactory;

/**
 * Define um loader que carrega ambientes da linguagem funcional.
 */
public class EnvironmentLoaderNode implements EnvironmentLoader {

    private DataHolderFactory holderFactory;
    private ExpressionParser expressionParser;

    public EnvironmentLoaderNode() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        holderFactory = DataHolderBuilder.getDataHolderFactory();
        expressionParser = ExpressionBuilder.getExpressionParser();
    }

    public String[] getTypes() {
        return new String[] {
            String.class.getName(),
            File.class.getName(),
            URL.class.getName(),
            String[].class.getName()
        };
    }

    public String getDescription() {
        return
            "Build a environment based on Strings, Files, URLs, os String arrays.";
    }

    public Environment load(Object source) throws EnvironmentLoaderException {
        Environment result = null;
        if(source instanceof String || source instanceof File
            || source instanceof URL) {
            result = loadDefault(source);
        }
        else if(source instanceof String[]) {
            String[] args = (String[]) source;
            result = loadDefault(args[0]);
            try {
                for(int i = 0; i < args.length; i++) {
                    ExpressionId id = new ExpressionId("$" + i);
                    Expression in = new ExpressionString(args[i]);
                    /** @todo generalizar a passagem de parametros. */
                    /*
                                         try {
                        in = expressionParser.getExpression(args[i]);
                                         }
                                         catch(ExpressionParserException exc) {
                        in = new ExpressionString(args[i]);
                                         }
                     */
                    Abstraction body = new Abstraction(new
                                       ExpressionListImpl(), in);
                    DeclarationFunction dec = new DeclarationFunction(null, id,
                                              body);
                    dec.execute(result);
                }
            }
            catch(DeclarationException exc) {
                throw new EnvironmentLoaderException(exc.getMessage(), exc);
            }
        }
        else {
            throw new EnvironmentLoaderException(
                "You cannot load a configuration from a "
                + source.getClass().getName());
        }
        return result;
    }

    private Environment loadDefault(Object source) throws
        EnvironmentLoaderException {
        Environment result = null;
        DataHolder holder = null;
        try {
            holder = holderFactory.produce(source);
            NodeReader reader = NodeBuilder.lookupNodeReader(holder);
            Node node = reader.read(holder);
            NodeConversor conversor = NodeBuilder.lookupNodeConversor(
                                      Environment.class.getName());
            result = (Environment) conversor.convert(node);
        }
        catch(Exception exc) {
            throw new EnvironmentLoaderException(exc.getMessage(), exc);
        }
        finally {
            if(holder != null) {
                try {
                    holder.close();
                }
                catch(DataHolderException exc) {
                    throw new EnvironmentLoaderException(exc.getMessage(), exc);
                }
            }
        }
        return result;
    }
}