package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import jpel.language.Declaration;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationType;
import jpel.language.Environment;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeException;
import jpel.tree.NodeMatcher;
import jpel.tree.NodeProcessor;
import jpel.util.Debugger;

/**
 * Define o processo de busca dentro de uma estrutura que envolve declaraes.
 */

public class NodeMatcherEnvironment implements NodeMatcher {

    private NodeProcessor processor;

    private Pattern pattern;
    private boolean useName;
    private boolean useBody;
    private boolean useDescription;
    private boolean recursive;

    public NodeMatcherEnvironment() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        processor = new NodeProcessorAll();
        setUseName(true);
        setUseBody(false);
        setUseDescription(false);
        setRecursive(false);
    }

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName()};
    }

    public String getDescription() {
        return "Environment matcher";
    }

    public Node process(Node node) throws NodeException {
        return privateProcess(isRecursive() ? processor.process(node)
            : node.mirror());
    }

    public Node privateProcess(Node node) throws NodeException {
        Iterator iter = node.children();
        List list = new LinkedList();
        while(iter.hasNext()) {
            Node child = (Node) iter.next();
            list.add(child);
        }
        iter = list.iterator();
        while(iter.hasNext()) {
            Node child = (Node) iter.next();
            if(!match(child)) {
                if(child.isFolder()) {
                    privateProcess(child);
                    if(child.isLeaf()) {
                        child.remove();
                    }
                }
                else {
                    child.remove();
                }
            }
        }
        return node;
    }

    public void setPattern(String pattern) throws NodeException {
        try {
            this.pattern = Pattern.compile(pattern);
            Debugger.println("Editor", "search", "Pattern '" + pattern + "'");
        }
        catch(PatternSyntaxException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
    }

    public String getPattern() {
        return pattern.pattern();
    }

    public void setUseName(boolean useName) {
        this.useName = useName;
    }

    public boolean isUseName() {
        return useName;
    }

    public void setUseBody(boolean useBody) {
        this.useBody = useBody;
    }

    public boolean isUseBody() {
        return useBody;
    }

    public void setUseDescription(boolean useDescription) {
        this.useDescription = useDescription;
    }

    public boolean isUseDescription() {
        return useDescription;
    }

    public void setRecursive(boolean recursive) {
        this.recursive = recursive;
    }

    public boolean isRecursive() {
        return recursive;
    }

    private boolean match(Node node) {
        boolean result = false;
        Object obj = node.getValue();
        String str = null;
        Matcher matcher = null;
        if(obj instanceof Declaration) {
            Declaration dec = (Declaration) obj;
            DeclarationType decType = dec.getType();
            if(isUseName()) {
                str = dec.getName().getName();
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
            str = null;
            if(isUseBody()) {
                if(decType.isFunction() || decType.isNative()) {
                    str = String.valueOf(((DeclarationFunction) dec).
                          getAbstraction().getBody());
                }
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
            str = null;
            if(isUseDescription()) {
                str = dec.getDescription();
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
        }
        else if(obj instanceof Include) {
            Include dec = (Include) obj;
            if(isUseName()) {
                str = String.valueOf(dec.getHolder().getReference());
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
            if(isUseBody()) {
                str = String.valueOf(dec.getName());
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
            if(isUseDescription()) {
                str = dec.getDescription();
                if(str != null) {
                    matcher = pattern.matcher(str);
                    result = result || matcher.find();
                }
            }
        }
        return result;
    }
}