package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import jpel.language.Environment;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeException;
import jpel.tree.NodeProcessor;
import jpel.tree.NodeReader;
import jpel.util.Debugger;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderFactory;

/**
 * Realiza a expanso completa de uma arvore de declaraes.
 */
public class NodeProcessorAll implements NodeProcessor {

    protected DataHolderFactory holderFactory;
    protected Stack stack;

    public NodeProcessorAll() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        this(DataHolderBuilder.getDataHolderFactory());
    }

    public NodeProcessorAll(DataHolderFactory holderFactory) {
        this.holderFactory = holderFactory;
        stack = new Stack();
    }

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName() + ".all"};
    }

    public String getDescription() {
        return "Expand all environment";
    }

    public Node process(Node node) throws NodeException {
        stack.clear();
        return privateProcess(node.mirror());
    }

    private Node privateProcess(Node node) throws NodeException {
        try {
            if(node.isRoot()) {
                stack.push(node.getValue());
            }
            Object obj = node.getValue();
            if(!node.isRoot() && obj instanceof Include) {
                Include decBase = (Include) stack.peek();
                DataHolder base = decBase.getHolder();
                Include decHolder = (Include) obj;
                DataHolder holder = decHolder.getHolder();
                DataHolder resolved = base.resolve(holder);
                Include decResolved = new Include(resolved, decHolder.getName());
                if(!stack.contains(decResolved)) {
                    stack.push(decResolved);
                    NodeReader reader = NodeBuilder.lookupNodeReader(resolved);
                    Node newNode = reader.read(resolved);
                    Iterator sub = newNode.children();
                    while(sub.hasNext()) {
                        Node n = (Node) sub.next();
                        node.insert(n);
                        privateProcess(n);
                    }
                    stack.pop();
                }
                else {
                    Debugger.println("NPE", "process",
                        "Ciclyc dependency '" + node + "' ignored.");
                    node.remove();
                }
            }
            else {
                Iterator iter = node.children();
                List list = new LinkedList();
                while(iter.hasNext()) {
                    Node child = (Node) iter.next();
                    list.add(child);
                }
                iter = list.iterator();
                while(iter.hasNext()) {
                    Node child = (Node) iter.next();
                    privateProcess(child);
                }
            }
        }
        catch(ClassNotFoundException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
        catch(DataHolderException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
        return node;
    }
}