package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import jpel.language.Environment;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeException;
import jpel.util.dataholder.DataHolderFactory;

/**
 * Fornece a rvore de dependencia de recursos.
 */
public class NodeProcessorDependency extends NodeProcessorAll {

    public NodeProcessorDependency() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        super();
    }

    public NodeProcessorDependency(DataHolderFactory holderFactory) {
        super(holderFactory);
    }

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName() + ".dependency"};
    }

    public String getDescription() {
        return "Show the dependency tree";
    }

    public Node process(Node node) throws NodeException {
        Node all = super.process(node);
        return privateProcess(all);
    }

    private Node privateProcess(Node node) throws NodeException {
        Iterator iter = node.children();
        List list = new LinkedList();
        while(iter.hasNext()) {
            Node child = (Node) iter.next();
            list.add(child);
        }
        iter = list.iterator();
        while(iter.hasNext()) {
            Node child = (Node) iter.next();
            privateProcess(child);
        }
        if(node.isLeaf()) {
            if(!(node.getValue()instanceof Include)) {
                node.remove();
            }
        }
        return node;
    }
}