package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import jpel.language.Abstraction;
import jpel.language.Declaration;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationType;
import jpel.language.Environment;
import jpel.language.EnvironmentBuilder;
import jpel.language.EnvironmentFactory;
import jpel.language.EnvironmentFactoryException;
import jpel.language.ExpressionId;
import jpel.language.ExpressionList;
import jpel.language.ExpressionListImpl;
import jpel.language.Include;
import jpel.language.MapReplace;
import jpel.language.MapReplaceImpl;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeException;
import jpel.tree.NodeFactory;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderFactory;

/**
 * Remove as declaraes de import de uma rvore de declaraes.
 */
public class NodeProcessorEnvironment extends NodeProcessorAll {

    private NodeFactory nodeFactory;
    private EnvironmentFactory environmentFactory;

    public NodeProcessorEnvironment() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        this(DataHolderBuilder.getDataHolderFactory(),
            NodeBuilder.getNodeFactory(),
            EnvironmentBuilder.getEnvironmentFactory());
    }

    public NodeProcessorEnvironment(DataHolderFactory holderFactory,
        NodeFactory nodeFactory, EnvironmentFactory environmentFactory) {
        super(holderFactory);
        this.nodeFactory = nodeFactory;
        this.environmentFactory = environmentFactory;
    }

    public String[] getTypes() {
        return new String[] {
            Environment.class.getName()};
    }

    public String getDescription() {
        return "Process a node to the environment prototype";
    }

    public Node process(Node node) throws NodeException {
        Node all = super.process(node);
        return privateProcess(all);
    }

    private Node privateProcess(Node node) throws NodeException {
        Node result = nodeFactory.getNode(node.getValue());
        Stack stack = new Stack();
        stack.push(result);
        pilha(node, stack);
        return result;
    }

    private void pilha(Node node, Stack stack) throws NodeException {
        List list = new LinkedList();
        Iterator iter = node.children();
        while(iter.hasNext()) {
            list.add(iter.next());
        }
        iter = list.iterator();
        while(iter.hasNext()) {
            Node tmp = (Node) iter.next();
            if(tmp.isFolder()) {
                Object value = tmp.getValue();
                if(!(value instanceof Include)) {
                    Node module = nodeFactory.getNode(value, (Node) stack.peek());
                    stack.push(module);
                }
                else {
                    Include dec = (Include) value;
                    // se a inclusao tem nome ento deve-se: 1.criar um mdulo
                    // adicional e 2. modificar os Ids da sub-rvore para terem
                    // como prefixo o nome do mdulo.
                    if(dec.getName() != null) {
                        try {
                            DeclarationModule newModule = new DeclarationModule(null,
                                dec.getName(), environmentFactory.empty());
                            Node module = nodeFactory.getNode(newModule,
                                          (Node) stack.peek());
                            stack.push(module);
                        }
                        catch(EnvironmentFactoryException ex) {
                            throw new NodeException(ex.getMessage(), ex);
                        }
                    }
                }
                pilha(tmp, stack);
                if(!(value instanceof Include)) {
                    stack.pop();
                }
                else {
                    Include dec = (Include) value;
                    // modificando a sub-rvore.
                    if(dec.getName() != null) {
                        ExpressionId prefix = dec.getName();
                        Node module = (Node) stack.pop();
                        addPrefix(prefix, module);
                    }
                }
            }
            else {
                nodeFactory.getNode(tmp.getValue(), (Node) stack.peek());
            }
        }
    }

    /**
     * Adiciona um prefixo ao conjunto de declaraoes de uma sub-rvore. Apenas
     * os identificadores que nao fazem parte do grupo de operadores padro 
     * que pode ser substitudo.
     * @param prefix O prefixo que deve ser adicionado.
     * @param node A sub-rvore que deve ser modificada.
     * @throws NodeException Caso haja erro durante o processo de mundana.
     */
    private void addPrefix(ExpressionId prefix, Node node) throws NodeException {
        Declaration dec = (Declaration) node.getValue();
        DeclarationType type = dec.getType();
        if(type.isFunction()) { // muda o nome dos identificadores.
            DeclarationFunction lDec = (DeclarationFunction) dec;
            MapReplace map = buildMap(prefix, lDec);
            dec = dec.rebuild(map);
            node.setValue(dec);
        }
        else if(type.isNative()) {
        // nada para fazer, este tipo sempre  imutvel
        // o Id identifica uma classe e nao pode ser alterado independente
        // de onde for a declarao.
        }
        Iterator sub = node.children();
        while(sub.hasNext()) {
            addPrefix(prefix, (Node) sub.next());
        }
    }

    /**
     * Fornece o mapeamento que deve ser usado para substituir as declaraes.
     * Encontrando os Id que devem receber o prefixo dado pelo "as" e ignorando
     * os Id que fazem parte do grupo de operadores padro.
     * @param prefix O prefixo que deve ser adiocionado.
     * @param lDec A declaraao que deve ser substituida.
     * @return O mapeamento dos identificadores que devem receber o prefixo e
     * seu nome destino.
     */
    private MapReplace buildMap(ExpressionId prefix, DeclarationFunction lDec) {
        Abstraction abs = lDec.getAbstraction();
        MapReplace map = new MapReplaceImpl();
        ExpressionList free = new ExpressionListImpl();
        abs.freeVariable(free);
        Iterator iFree = free.iterator();
        while(iFree.hasNext()) {
            ExpressionId item = (ExpressionId) iFree.next();
            if(!environmentFactory.isLang(item)) {
                map.put(item,
                    new ExpressionId(prefix.getName() + ExpressionId.SEPARATOR
                    + item.getName()));
            }
        }
        return map;
    }
}