package jpel.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;

import jpel.language.Abstraction;
import jpel.language.Declaration;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationNative;
import jpel.language.DeclarationType;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeException;
import jpel.tree.NodeWriter;
import jpel.util.StringFormater;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderException;

/**
 * Realiza a escrita de uma rvore no formato JPEL.
 */
public class NodeWriterJpel implements NodeWriter {

    private String gap = "";
    private String blk = "  ";

    private String F_DESC = "#!";
    private String DESC = "#";
    private String NATIVE = "native";
    private String MODULE = "module";
    private String INCLUDE = "include";
    private String IMPORT = "import";

    public NodeWriterJpel() {
    }

    public String[] getTypes() {
        return new String[] {
            "jpel"};
    }

    public String getDescription() {
        return "JPEL resources";
    }

    public void write(Node node, DataHolder holder) throws NodeException,
        DataHolderException {
        try {
            gap = "";
            write(node, holder.getWriter());
        }
        catch(IOException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    private void write(Node node, Writer w) throws NodeException, IOException {
        if(node.isRoot()) {
            Include dec = (Include) node.getValue();
            DataHolder holder = dec.getHolder();
            if(dec.getDescription() != null) {
                String str = prepare(F_DESC, dec.getDescription());
                w.write(gap + str + "\n");
            }
            w.write("\n");
            Iterator i = node.children();
            int index = 0;
            while(i.hasNext()) {
                if(index > 0) {
                    write(gap, "", w);
                }
                write((Node) i.next(), w);
                index++;
            }
        }
        else {
            Object obj = node.getValue();
            if(obj instanceof Declaration) {
                Declaration declaration = (Declaration) obj;
                DeclarationType type = declaration.getType();
                if(type.isModule()) {
                    DeclarationModule dec = (DeclarationModule) obj;
                    if(dec.getDescription() != null) {
                        String str = prepare(DESC, dec.getDescription());
                        w.write(gap + str + "\n");
                    }
                    write(gap, MODULE + " " + dec.getName(), w);
                    write(gap, "{", w);
                    gap += blk;

                    Iterator i = node.children();
                    int index = 0;
                    while(i.hasNext()) {
                        if(index > 0) {
                            write(gap, "", w);
                        }
                        write((Node) i.next(), w);
                        index++;
                    }
                    gap = gap.substring(blk.length());

                    write(gap, "}", w);
                }
                else {
                    StringBuffer result = new StringBuffer();
                    if(type.isFunction()) {
                        DeclarationFunction dec = (DeclarationFunction) obj;
                        if(dec.getDescription() != null) {
                            String str = prepare(DESC, dec.getDescription());
                            w.write(gap + str + "\n");
                        }
                        Abstraction abs = dec.getAbstraction();

                        Expression body = abs.getBody();
                        String str = String.valueOf(body);
                        boolean hasLine = str.indexOf("\n") >= 0;

                        ExpressionList args = abs.getArguments();
                        if(args.length() > 0) {
                            write(gap, dec.getName() + " " + args.toString("(",
                                ",",
                                ")") + " = " + (!hasLine ? str : ""), w);
                        }
                        else {
                            write(gap,
                                dec.getName() + " = " + (!hasLine ? str : ""),
                                w);
                        }
                        if(hasLine) {
                            write(gap + blk,
                                StringFormater.toWrite(abs.getBody(), gap + blk),
                                w);
                        }
                    }
                    else if(type.isNative()) {
                        DeclarationNative dec = (DeclarationNative) obj;
                        if(dec.getDescription() != null) {
                            String str = prepare(DESC, dec.getDescription());
                            w.write(gap + str + "\n");
                        }
                        Abstraction abs = dec.getAbstraction();
                        write(gap,
                            NATIVE + " " + dec.getName() + " "
                            + abs.getArguments().toString("(", ",", ")")
                            + " = ", w);
                        write(gap + blk, abs.getBody(), w);
                    }
                }
            }
            else if(obj instanceof Include) {
                Include dec = (Include) obj;
                if(dec.getDescription() != null) {
                    String str = prepare(DESC, dec.getDescription());
                    w.write(gap + str + "\n");
                }
                DataHolder holder = dec.getHolder();
                Object reference = holder.getReference();
                write(gap,
                    INCLUDE + " \""
                    + String.valueOf(reference).replace('\\',
                    '/') + "\"" + (dec.getName() != null
                    ? " as " + dec.getName() : ""),
                    w);
            }
        }
        w.flush();
    }

    private String prepare(String prefix, String str) {
        str = prefix + str.replaceAll("\n", "\n" + prefix);
        str = StringFormater.toWrite(str, gap);
        return str;
    }

    private void write(String gap, Object obj, Writer w) throws IOException {
        w.write(gap + obj + "\n");
    }
}