package jpel.bridge.xml;

import java.io.IOException;
import java.io.InputStream;
import java.util.Stack;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import jpel.language.Abstraction;
import jpel.language.Declaration;
import jpel.language.DeclarationException;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationNative;
import jpel.language.EnvironmentBuilder;
import jpel.language.EnvironmentFactory;
import jpel.language.EnvironmentFactoryException;
import jpel.language.Expression;
import jpel.language.ExpressionBuilder;
import jpel.language.ExpressionId;
import jpel.language.ExpressionList;
import jpel.language.ExpressionListImpl;
import jpel.language.ExpressionParser;
import jpel.language.ExpressionParserException;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeException;
import jpel.tree.NodeFactory;
import jpel.tree.NodeReader;
import jpel.util.Debugger;
import jpel.util.StringFormater;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderFactory;
import jpel.util.dataholder.DataHolderFactoryException;

/**
 * Realiza a leitura de uma rvore escrita no formato XML.
 */
public class NodeReaderXML extends DefaultHandler implements NodeReader {

    private SAXParser saxParser;

    private DataHolderFactory holderFactory;
    private NodeFactory nodeFactory;
    private EnvironmentFactory environmentFactory;
    private ExpressionParser expressionParser;

    private Stack stack;
    private Node rootNode;
    private Include rootHolder;

    private Include decInclude;
    private DeclarationModule decModule;
    private DeclarationFunction decFunction;
    private DeclarationNative decNative;
    private Stack stackDeclarations;

    private NodeXMLConstants constants;

    private boolean onDescription;
    private StringBuffer description;

    private boolean onValue;
    private StringBuffer value;

    public NodeReaderXML() throws ParserConfigurationException, SAXException,
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        SAXParserFactory saxFactory = SAXParserFactory.newInstance();
        saxParser = saxFactory.newSAXParser();
        holderFactory = DataHolderBuilder.getDataHolderFactory();
        nodeFactory = NodeBuilder.getNodeFactory();
        environmentFactory = EnvironmentBuilder.getEnvironmentFactory();
        expressionParser = ExpressionBuilder.getExpressionParser();
        stack = new Stack();

        stackDeclarations = new Stack();
        constants = NodeXMLConstants.getInstance();

        onDescription = false;
        description = new StringBuffer();

        onValue = false;
        value = new StringBuffer();
    }

    public void setConstants(NodeXMLConstants constants) {
        this.constants = constants;
    }

    public NodeXMLConstants getConstants() {
        return constants;
    }

    public String[] getTypes() {
        return new String[] {
            "xml"};
    }

    public String getDescription() {
        return "XML resources";
    }

    public Node read(DataHolder holder) throws NodeException,
        DataHolderException {

        stack.clear();
        stackDeclarations.clear();

        description.setLength(0);
        value.setLength(0);

        try {
            rootHolder = new Include(holder, null);
            InputStream source = holder.getInputStream();
            saxParser.parse(source, this);
            source.close();
        }
        catch(IOException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
        catch(SAXException exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
        return rootNode;
    }

    public void startElement(String uri, String localName, String qName,
        Attributes attributes) throws SAXException {
        String strTag = qName;
        String attValue = null;
        try {
            //System.out.println( "START:<" + localName + "," + qName + ","+ attributes + ">" );
            if(strTag.equals(constants.CONFIGURATION)) {
                stackDeclarations.push(rootHolder);
                Node node = nodeFactory.getNode(rootHolder);
                stack.push(node);
            }
            else if(strTag.equals(constants.FUNCTION)) {
                attValue = attributes.getValue(constants.NAME);
                if(attValue == null) {
                    throw new SAXException(constants.FUNCTION
                        + " missing identifier. i.e " + constants.NAME
                        + "=\"HOME\"");
                }
                ExpressionId name = (ExpressionId) expressionParser.
                                    getExpression(attValue);
                attValue = attributes.getValue(constants.FORMAL);
                ExpressionList formals = null;
                if(attValue != null) {
                    formals = (ExpressionList) expressionParser.getExpression(
                              attValue);
                }
                else {
                    formals = new ExpressionListImpl();
                }
                Abstraction abs = new Abstraction(formals, null);
                decFunction = new DeclarationFunction(null, name, abs);
                stackDeclarations.push(decFunction);
            }
            else if(strTag.equals(constants.NATIVE)) {
                attValue = attributes.getValue(constants.NAME);
                if(attValue == null) {
                    throw new SAXException(constants.NATIVE
                        + " missing native identifier. i.e " + constants.NAME
                        + "=\"TIME\"");
                }
                ExpressionId name = (ExpressionId) expressionParser.
                                    getExpression(attValue);
                ExpressionList formals = null;
                attValue = attributes.getValue(constants.FORMAL);
                if(attValue != null) {
                    formals = (ExpressionList) expressionParser.getExpression(
                              attValue);
                }
                else {
                    formals = new ExpressionListImpl();
                }
                Abstraction abs = new Abstraction(formals, null);
                try {
                    decNative = new DeclarationNative(null, name, abs);
                }
                catch(DeclarationException ex) {
                    throw new SAXException(ex.getMessage(), ex);
                }
                stackDeclarations.push(decNative);
            }
            else if(strTag.equals(constants.MODULE)) {
                attValue = attributes.getValue(constants.NAME);
                if(attValue == null) {
                    throw new SAXException(constants.MODULE
                        + " missing module identifier. i.e " + constants.NAME
                        + "=\"MODULE\"");
                }
                ExpressionId name = (ExpressionId) expressionParser.
                                    getExpression(attValue);
                try {
                    decModule = new DeclarationModule(null, name,
                                environmentFactory.empty());
                }
                catch(EnvironmentFactoryException ex) {
                    throw new SAXException(ex.getMessage(), ex);
                }

                stackDeclarations.push(decModule);
                Node node = nodeFactory.getNode(decModule, (Node) stack.peek());
                stack.push(node);
            }
            else if(strTag.equals(constants.INCLUDE)) {
                attValue = attributes.getValue(constants.NAME);
                if(attValue == null) {
                    throw new SAXException(constants.INCLUDE
                        + " missing include reference. i.e " + constants.NAME
                        + "=\"file.xml\"");
                }
                DataHolder holder = holderFactory.produce(attValue);
                ExpressionId name = null;
                attValue = attributes.getValue(constants.AS);
                if(attValue != null) {
                    name = (ExpressionId) expressionParser.getExpression(
                           attValue);
                }
                decInclude = new Include(holder, name);
                stackDeclarations.push(decInclude);
                nodeFactory.getNode(decInclude, (Node) stack.peek());
            }
            else if(strTag.equals(constants.DESCRIPTION)) {
                onDescription = true;
                description.setLength(0);
            }
            else if(strTag.equals(constants.VALUE)) {
                onValue = true;
                value.setLength(0);
            }
            else {
                Debugger.println("NodeReaderXML", "startElement",
                    "Unknow tag:" + strTag);
            }
        }
        catch(NodeException exc) {
            throw new SAXException(exc.getMessage(), exc);
        }
        catch(DataHolderFactoryException exc) {
            throw new SAXException(exc.getMessage(), exc);
        }
        catch(ExpressionParserException exc) {
            throw new SAXException(exc.getMessage(), exc);
        }
    }

    public void characters(char[] ch, int start,
        int length) throws SAXException {
        if(onDescription) {
            description.append(new String(ch, start, length));
        }
        if(onValue) {
            value.append(new String(ch, start, length));
        }
    }

    public void endElement(String uri, String localName,
        String qName) throws SAXException {
        //System.out.println( "END:<" + localName + "," + qName + ">" );
        String strTag = qName;
        try {
            if(strTag.equals(constants.CONFIGURATION)) {
                rootNode = (Node) stack.pop();
                if(!stackDeclarations.isEmpty()
                    && stackDeclarations.peek() == rootHolder) {
                    stackDeclarations.pop();
                }
            }
            else if(strTag.equals(constants.FUNCTION)) {
                if(!stackDeclarations.isEmpty()
                    && stackDeclarations.peek() == decFunction) {
                    stackDeclarations.pop();
                }
                nodeFactory.getNode(decFunction, (Node) stack.peek());
                decFunction = null;
            }
            else if(strTag.equals(constants.NATIVE)) {
                if(!stackDeclarations.isEmpty()
                    && stackDeclarations.peek() == decNative) {
                    stackDeclarations.pop();
                }
                nodeFactory.getNode(decNative, (Node) stack.peek());
                decNative = null;
            }
            else if(strTag.equals(constants.MODULE)) {
                if(!stackDeclarations.isEmpty()
                    && stackDeclarations.peek() == decModule) {
                    stackDeclarations.pop();
                }
                decModule = null;
                stack.pop();
            }
            else if(strTag.equals(constants.INCLUDE)) {
                if(!stackDeclarations.isEmpty()
                    && stackDeclarations.peek() == decInclude) {
                    stackDeclarations.pop();
                }
                decInclude = null;
            }
            else if(strTag.equals(constants.DESCRIPTION)) {
                String desc = description.length() > 0 ? description.toString() : null;
                if(desc != null) {
                    desc = StringFormater.toRead(desc);
                }
                if(!stackDeclarations.isEmpty()) {
                    Object peek = stackDeclarations.peek();
                    if( peek instanceof Declaration ) {
                        ((Declaration) peek).setDescription(desc);
                    }
                    else {
                        ((Include) peek).setDescription(desc);
                    }
                }
            }
            else if(strTag.equals(constants.VALUE)) {
                String val = value.length() > 0 ? value.toString() : null;
                boolean empty = (val == null);
                if(!empty) {
                    val = StringFormater.toRead(val);
                }
                Declaration dec = (Declaration) stackDeclarations.peek();
                // function checking
                if(dec == decFunction) {
                    if(!empty) {
                        Expression body = (Expression) expressionParser.
                                          getExpression(val);
                        decFunction.getAbstraction().setBody(body);
                    }
                    else {
                        throw new SAXException(decFunction.getName()
                            + ": Missing value description.");
                    }
                }
                // native checking
                else if(dec == decNative) {
                    if(!empty) {
                        ExpressionId body = (ExpressionId) expressionParser.
                                            getExpression(val);
                        decNative.getAbstraction().setBody(body);
                    }
                    else {
                        throw new SAXException(decNative.getName()
                            + ": Missing value description.");
                    }
                }
            }
            else {
                Debugger.println("NodeReaderXML", "endElement",
                    "Unknow tag:" + strTag);
            }
            //System.out.println( "END:<" + localName + "," + qName + ">" );
        }
        catch(NodeException exc) {
            throw new SAXException(exc.getMessage(), exc);
        }
        catch(ExpressionParserException exc) {
            throw new SAXException(exc.getMessage(), exc);
        }
    }
}