package jpel.bridge.xml;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.Writer;
import java.util.Iterator;

import jpel.language.Abstraction;
import jpel.language.Declaration;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationNative;
import jpel.language.DeclarationType;
import jpel.language.ExpressionId;
import jpel.language.ExpressionList;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeException;
import jpel.tree.NodeWriter;
import jpel.util.StringFormater;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderException;

/**
 * Realiza a escrita de uma rvore escrita no formato XML.
 */
public class NodeWriterXML implements NodeWriter {

    private String gap = "";
    private String blk = "  ";

    private NodeXMLConstants constants;

    public NodeWriterXML() {
        constants = NodeXMLConstants.getInstance();
    }

    public void setConstants(NodeXMLConstants constants) {
        this.constants = constants;
    }

    public NodeXMLConstants getConstants() {
        return constants;
    }

    public String[] getTypes() {
        return new String[] {
            "xml"};
    }

    public String getDescription() {
        return "XML resources";
    }

    public void write(Node node, DataHolder holder) throws NodeException,
        DataHolderException {
        try {
            gap = "";
            write(node, holder.getWriter());
        }
        catch(IOException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    private void write(Node node, Writer w) throws NodeException, IOException {
        if(node.isRoot()) {
            write(gap, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>", w);
            write(gap, "<" + constants.CONFIGURATION + ">", w);
            gap += blk;
            Include dec = (Include) node.getValue();
            DataHolder holder = dec.getHolder();
            if(dec.getDescription() != null) {
                write(gap, "<" + constants.DESCRIPTION + "><![CDATA[", w);
                write(gap + blk, StringFormater.toWrite(dec.getDescription(),
                    gap + blk), w);
                write(gap, "]]></" + constants.DESCRIPTION + ">", w);
                w.write("\n");
            }
            Iterator i = node.children();
            int index = 0;
            while(i.hasNext()) {
                if(index > 0) {
                    write(gap, "", w);
                }
                write((Node) i.next(), w);
                index++;
            }
            gap = gap.substring(blk.length());
            write(gap, "</" + constants.CONFIGURATION + ">", w);
        }
        else {
            Object obj = node.getValue();
            if(obj instanceof Declaration) {
                Declaration declaration = (Declaration) obj;
                DeclarationType type = declaration.getType();
                if(type.isModule()) {
                    DeclarationModule dec = (DeclarationModule) obj;
                    write(gap,
                        "<" + constants.MODULE + " " + constants.NAME
                        + "=\"" + dec.getName() + "\"" + ">", w);

                    gap += blk;

                    if(dec.getDescription() != null) {
                        write(gap, "<" + constants.DESCRIPTION + "><![CDATA[",
                            w);
                        write(gap + blk,
                            StringFormater.toWrite(dec.getDescription(),
                            gap + blk), w);
                        write(gap, "]]></" + constants.DESCRIPTION + ">", w);
                    }

                    if(dec.getDescription() != null ) {
                        w.write("\n");
                    }

                    Iterator i = node.children();
                    int index = 0;
                    while(i.hasNext()) {
                        if(index > 0) {
                            write(gap, "", w);
                        }
                        write((Node) i.next(), w);
                        index++;
                    }
                    gap = gap.substring(blk.length());
                    write(gap, "</" + constants.MODULE + ">", w);
                }
                else {
                    if(type.isFunction()) {
                        DeclarationFunction dec = (DeclarationFunction) obj;
                        ExpressionId name = dec.getName();
                        Abstraction body = dec.getAbstraction();

                        ExpressionList args = body.getArguments();
                        write(gap,
                            "<" + constants.FUNCTION + " " + constants.NAME
                            + "=\"" + name + "\"" +
                            (args.length() <= 0 ? "" :
                            " " + constants.FORMAL
                            + "=\""
                            + args.toString("(", ",", ")")
                            + "\"") + ">", w);

                        if(dec.getDescription() != null) {
                            write(gap + blk,
                                "<" + constants.DESCRIPTION + "><![CDATA[", w);
                            write(gap + blk + blk,
                                StringFormater.toWrite(dec.getDescription(),
                                gap + blk + blk), w);
                            write(gap + blk,
                                "]]></" + constants.DESCRIPTION + ">", w);
                        }

                        write(gap + blk,
                            "<" + constants.VALUE + "><![CDATA[", w);
                        write(gap + blk + blk,
                            StringFormater.toWrite(((Abstraction)
                            body).getBody(), gap + blk + blk), w);
                        write(gap + blk, "]]></" + constants.VALUE + ">",
                            w);

                        write(gap, "</" + constants.FUNCTION + ">", w);
                    }
                    else if(type.isNative()) {
                        DeclarationNative dec = (DeclarationNative) obj;
                        ExpressionList args = dec.getAbstraction().getArguments();
                        write(gap,
                            "<" + constants.NATIVE + " " + constants.NAME
                            + "=\"" + dec.getName() + "\"" +
                            (args.length() <= 0 ? "" : " " + constants.FORMAL
                            + "=\"" + args.toString("(", ",", ")") + "\"")
                            + ">", w);
                        if(dec.getDescription() != null) {
                            write(gap + blk,
                                "<" + constants.DESCRIPTION + "><![CDATA[", w);
                            write(gap + blk + blk,
                                StringFormater.toWrite(dec.getDescription(),
                                gap + blk + blk), w);
                            write(gap + blk,
                                "]]></" + constants.DESCRIPTION + ">", w);
                        }
                        write(gap + blk, "<" + constants.VALUE + "><![CDATA[",
                            w);
                        write(gap + blk + blk, dec.getAbstraction().getBody(),
                            w);
                        write(gap + blk, "]]></" + constants.VALUE + ">", w);
                        write(gap, "</" + constants.NATIVE + ">", w);
                    }
                }
            }
            else if(obj instanceof Include) {
                Include dec = (Include) obj;
                DataHolder holder = dec.getHolder();
                Object reference = holder.getReference();
                write(gap,
                    "<" + constants.INCLUDE + " " + constants.NAME
                    + "=\"" + String.valueOf(reference).replace('\\',
                    '/') + "\"" + (dec.getName() != null
                    ? " as=\"" + dec.getName() + "\"" : "") + ">", w);
                if(dec.getDescription() != null) {
                    write(gap + blk,
                        "<" + constants.DESCRIPTION + "><![CDATA[", w);
                    write(gap + blk + blk,
                        StringFormater.toWrite(dec.getDescription(),
                        gap + blk + blk), w);
                    write(gap + blk,
                        "]]></" + constants.DESCRIPTION + ">", w);
                }
                write(gap, "</" + constants.INCLUDE + ">", w);
            }
        }
        w.flush();
    }

    private void write(String gap, Object obj, Writer w) throws IOException {
        w.write(gap + obj + "\n");
    }
}