package jpel.bridge.xml;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

import jpel.util.Debugger;

/**
 * XML readers constants. If you want change tags names use the JVM parameter
 * -Dxml.constants to indicate tag patterns.
 * <P>
 * Ex:
 * <br>CONFIGURATION :: config
 * <br>MODULE        :: group
 * <br>...
 */
public class NodeXMLConstants {

    public static final String JVM_PARAMETER = "xml.constants";
    private static final String COMMENT = "#";
    private static final String TYPE = "::";

    public String CONFIGURATION = "configuration";
    public String INCLUDE = "include";
    public String MODULE = "module";
    public String FUNCTION = "function";
    public String NATIVE = "native";
    public String NAME = "id";
    public String AS = "as";
    public String FORMAL = "args";
    public String DESCRIPTION = "description";
    public String VALUE = "value";
    public String CALL = "call";

    private NodeXMLConstants() {
        this(System.getProperty(JVM_PARAMETER));
    }

    private NodeXMLConstants(String file) {
        if(file != null) {
            try {
                FileReader fin = new FileReader(file);
                BufferedReader bin = new BufferedReader(fin);
                String input;
                while((input = bin.readLine()) != null) {
                    String str = input.trim();
                    if(str.startsWith(COMMENT)) {
                        continue;
                    }
                    int pos;
                    if((pos = str.indexOf(TYPE)) > 0) {
                        String field = str.substring(0, pos).trim();
                        String value = str.substring(pos
                                       + TYPE.length()).trim();
                        if("CONFIGURATION".equals(field)) {
                            CONFIGURATION = value;
                        }
                        else if("INCLUDE".equals(field)) {
                            INCLUDE = value;
                        }
                        else if("MODULE".equals(field)) {
                            MODULE = value;
                        }
                        else if("FUNCTION".equals(field)) {
                            FUNCTION = value;
                        }
                        else if("NATIVE".equals(field)) {
                            NATIVE = value;
                        }
                        else if("NAME".equals(field)) {
                            NAME = value;
                        }
                        else if("AS".equals(field)) {
                            AS = value;
                        }
                        else if("FORMAL".equals(field)) {
                            FORMAL = value;
                        }
                        else if("DESCRIPTION".equals(field)) {
                            DESCRIPTION = value;
                        }
                        else if("VALUE".equals(field)) {
                            VALUE = value;
                        }
                        else if("CALL".equals(field)) {
                            CALL = value;
                        }
                        else {
                            throw new IOException(field
                                + " is not a valie field.");
                        }
                    }
                }
                bin.close();
                fin.close();
                Debugger.println("XMLContants", "load",
                    "Using file '" + file + "'");
            }
            catch(IOException ex) {
                ex.printStackTrace();
            }
        }
        else {
            Debugger.println("XMLContants", "load", "Using default schema.");
        }
    }

    public static NodeXMLConstants getInstance() {
        return new NodeXMLConstants();
    }

    public static NodeXMLConstants getInstance(String file) {
        return new NodeXMLConstants(file);
    }
}