package jpel.gui.bridge;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jpel.bridge.NodeProcessorDependency;
import jpel.gui.language.JPaneEnvironment;
import jpel.gui.tree.JPaneNode;
import jpel.gui.tree.JTreeNode;
import jpel.gui.tree.JTreeNodeEditor;
import jpel.gui.tree.JTreeNodeRenderer;
import jpel.gui.tree.UtilsNode;
import jpel.gui.util.Commands;
import jpel.gui.util.DebuggerSinkTextArea;
import jpel.gui.util.Icons;
import jpel.gui.util.JMenuBarGroup;
import jpel.gui.util.JMenuGroup;
import jpel.gui.util.JToolBarGroup;
import jpel.gui.util.JTreeSmartEvent;
import jpel.gui.util.JTreeSmartListener;
import jpel.language.Environment;
import jpel.language.ExpressionBuilder;
import jpel.language.ExpressionParser;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeConversor;
import jpel.tree.NodeException;
import jpel.tree.NodeFactory;
import jpel.tree.NodeMatcher;
import jpel.tree.NodeProcessor;
import jpel.tree.NodeReader;
import jpel.tree.NodeWriter;
import jpel.util.Debugger;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBinaryStream;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderFactory;
import jpel.util.dataholder.DataHolderFactoryException;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

public class FrameMain extends JFrame {
    private static int serial = 0;
    private HashMap openedFiles = new HashMap();
    private NodeFilePanel tabNode;

    private JTreeNodeEditor nodeEditor = new NodeEditorLanguage();
    private JTreeNodeRenderer nodeRenderer = new NodeRendererLanguage();
    private NodeMatcherPanel matcherPanel;
    private JFileChooser jfc = UtilsNode.getNodeFileChooser();

    private NodeFactory nodeFactory;
    private DataHolderFactory holderFactory;
    private ExpressionParser expressionParser;

    private JPanel contentPane;
    private BorderLayout borderLayout1 = new BorderLayout();
    private JToolBarGroup jToolBarMain = new JToolBarGroup();
    private JPanel jPanelStatus = new JPanel();
    private JSplitPane jSplitPane1 = new JSplitPane();
    private JSplitPane jSplitPane2 = new JSplitPane();
    private JPanel jPanel1 = new JPanel();
    private JPanel jPanelBottom = new JPanel();
    private JPanel jPanelRight = new JPanel();
    private JSplitPane jSplitPane3 = new JSplitPane();
    private JPanel jPanelLeft = new JPanel();
    private BorderLayout borderLayout3 = new BorderLayout();
    private JPanel jPanelCenter = new JPanel();
    private JTabbedPane jTabbedPaneFiles = new JTabbedPane();
    private BorderLayout borderLayout4 = new BorderLayout();
    private JMenuBarGroup jMenuBarMain = new JMenuBarGroup();
    private JLabel jLabelStatus = new JLabel();
    private BorderLayout borderLayout5 = new BorderLayout();
    private JSplitPane jSplitPane4 = new JSplitPane();
    private BorderLayout borderLayout6 = new BorderLayout();
    private JPanel jPanelProcessed = new JPanel();
    private JPanel jPanelConverted = new JPanel();
    private JScrollPane jScrollPane1 = new JScrollPane();
    private BorderLayout borderLayout7 = new BorderLayout();
    private BorderLayout borderLayout8 = new BorderLayout();
    private JTreeNode jTreeNodeProcessed = new JTreeNode();
    private JPaneEnvironment jPanelEnvironment = new JPaneEnvironment();
    private JTabbedPane jTabbedPaneBottom = new JTabbedPane();
    private BorderLayout borderLayout2 = new BorderLayout();
    private JPanel jPanelLog = new JPanel();
    private JPanel jPanelSearch = new JPanel();
    private JPanel jPanelDependency = new JPanel();
    private JScrollPane jScrollPaneLog = new JScrollPane();
    private BorderLayout borderLayout9 = new BorderLayout();
    private JTextArea jTextAreaLog = new JTextArea();
    private JScrollPane jScrollPaneDependency = new JScrollPane();
    private BorderLayout borderLayout10 = new BorderLayout();
    private JTreeNode jTreeDependency = new JTreeNode();
    private JScrollPane jScrollPaneSearch = new JScrollPane();
    private JTreeNode jTreeSearch = new JTreeNode();
    private BorderLayout borderLayout11 = new BorderLayout();
    private Border border1;

    //Construct the frame
    public FrameMain() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
            jbInit();
            populateMenuAndToolBar();
            nodeFactory = NodeBuilder.getNodeFactory();
            holderFactory = DataHolderBuilder.getDataHolderFactory();
            expressionParser = ExpressionBuilder.getExpressionParser();
            matcherPanel = new NodeMatcherPanel();

            nodeEditor.setPreferredSize(new Dimension(300, 300));
            jTreeNodeProcessed.setNodeRenderer(nodeRenderer);
            jPanelEnvironment.setParser(expressionParser);

            jTreeDependency.setNodeRenderer(nodeRenderer);
            jTreeSearch.setNodeRenderer(nodeRenderer);
            JTreeNodeListener treeListener = new JTreeNodeListener(this);
            jTreeDependency.addJTreeSmartListener(treeListener);
            jTreeSearch.addJTreeSmartListener(treeListener);

            Debugger.setDebuggerSink(new DebuggerSinkTextArea(jTextAreaLog));
            Debugger.enableDebug();

            /*
                 PrintStream out = new PrintStream(new ByteArrayOutputStream() {
                    public void flush() {
                        jTextAreaLog.append( toString() );
                 jTextAreaLog.setCaretPosition( jTextAreaLog.getText().length() );
                        reset();
                    }
                },true
                         );
                         System.setErr( out );
                         System.setOut( out );
             */
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    //Component initialization
    private void jbInit() throws Exception {
        //setIconImage(Toolkit.getDefaultToolkit().createImage(FrameMain.class.getResource("[Your Icon]")));
        contentPane = (JPanel)this.getContentPane();
        border1 = BorderFactory.createLineBorder(SystemColor.desktop, 1);
        contentPane.setLayout(borderLayout1);
        this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        this.setJMenuBar(jMenuBarMain);
        this.setSize(new Dimension(516, 403));
        this.setTitle("Editor");
        jSplitPane2.setOrientation(JSplitPane.VERTICAL_SPLIT);
        jSplitPane2.setOneTouchExpandable(true);
        jSplitPane2.setResizeWeight(0.8);
        jSplitPane1.setOneTouchExpandable(true);
        jSplitPane1.setResizeWeight(0.8);
        jPanelBottom.setLayout(borderLayout2);
        jPanel1.setLayout(borderLayout3);
        jPanelStatus.setBorder(BorderFactory.createEtchedBorder(Color.white,
            new Color(142, 142, 142)));
        jPanelStatus.setLayout(borderLayout5);
        jSplitPane3.setOneTouchExpandable(true);
        jSplitPane3.setResizeWeight(0.2);
        jPanelCenter.setLayout(borderLayout4);
        jLabelStatus.setText("<status>");
        jMenuBarMain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jMenuBarMain_actionPerformed(e);
            }
        });
        jToolBarMain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jToolBarMain_actionPerformed(e);
            }
        });
        jTabbedPaneFiles.addChangeListener(new javax.swing.event.
            ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                jTabbedPaneFiles_stateChanged(e);
            }
        });
        jSplitPane4.setOrientation(JSplitPane.VERTICAL_SPLIT);
        jSplitPane4.setOneTouchExpandable(true);
        jSplitPane4.setResizeWeight(0.4);
        jPanelRight.setLayout(borderLayout6);
        jPanelProcessed.setLayout(borderLayout7);
        jPanelConverted.setLayout(borderLayout8);
        jTabbedPaneBottom.setTabPlacement(JTabbedPane.BOTTOM);
        jPanelLog.setLayout(borderLayout9);
        jPanelDependency.setLayout(borderLayout10);
        jPanelSearch.setLayout(borderLayout11);
        jPanelProcessed.setBorder(new TitledBorder(BorderFactory.
            createLineBorder(SystemColor.desktop, 1), " Processed "));
        jPanelConverted.setBorder(new TitledBorder(BorderFactory.
            createLineBorder(SystemColor.desktop, 1), " Converted "));
        jPanelCenter.setBorder(new TitledBorder(BorderFactory.
            createLineBorder(SystemColor.desktop, 1), " Opened files: "));
        contentPane.add(jToolBarMain, BorderLayout.NORTH);
        contentPane.add(jSplitPane1, BorderLayout.CENTER);
        jSplitPane1.add(jSplitPane2, JSplitPane.LEFT);
        jSplitPane2.add(jPanel1, JSplitPane.LEFT);
        jSplitPane2.add(jPanelBottom, JSplitPane.RIGHT);
        jPanelBottom.add(jTabbedPaneBottom, BorderLayout.CENTER);
        jSplitPane1.add(jPanelRight, JSplitPane.RIGHT);
        jPanelRight.add(jSplitPane4, BorderLayout.CENTER);
        jSplitPane4.add(jPanelProcessed, JSplitPane.TOP);
        jPanelProcessed.add(jScrollPane1, BorderLayout.CENTER);
        jScrollPane1.getViewport().add(jTreeNodeProcessed, null);
        jSplitPane4.add(jPanelConverted, JSplitPane.BOTTOM);
        jPanelConverted.add(jPanelEnvironment, BorderLayout.CENTER);
        contentPane.add(jPanelStatus, BorderLayout.SOUTH);
        jPanelStatus.add(jLabelStatus, BorderLayout.CENTER);
        jPanel1.add(jSplitPane3, BorderLayout.CENTER);
        jSplitPane3.add(jPanelLeft, JSplitPane.LEFT);
        jSplitPane3.add(jPanelCenter, JSplitPane.RIGHT);
        jPanelCenter.add(jTabbedPaneFiles, BorderLayout.CENTER);
        jTabbedPaneBottom.add(jPanelLog, "Log");
        jPanelLog.add(jScrollPaneLog, BorderLayout.CENTER);
        jScrollPaneLog.getViewport().add(jTextAreaLog, null);
        jTabbedPaneBottom.add(jPanelDependency, "Dependency");
        jTabbedPaneBottom.add(jPanelSearch, "Search");
        jPanelDependency.add(jScrollPaneDependency, BorderLayout.NORTH);
        jScrollPaneDependency.getViewport().add(jTreeDependency, null);
        jPanelSearch.add(jScrollPaneSearch, BorderLayout.CENTER);
        jScrollPaneSearch.getViewport().add(jTreeSearch, null);
    }

    //Overridden so we can exit when window is closed
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if(e.getID() == WindowEvent.WINDOW_CLOSING) {
            exit();
        }
    }

    public void setVisible(boolean op) {
        super.setVisible(op);
        jSplitPane1.setDividerLocation((int) (jSplitPane1.getWidth() * 0.8));
        jSplitPane2.setDividerLocation((int) (jSplitPane2.getHeight() * 0.8));
        jSplitPane3.setDividerLocation((int) (jSplitPane3.getWidth() * 0.05));
    }

    void exit() {
        int option = JOptionPane.showConfirmDialog(this, "Confirm exit?",
                     "Confirm", JOptionPane.OK_CANCEL_OPTION,
                     JOptionPane.QUESTION_MESSAGE);
        if(option == JOptionPane.OK_OPTION) {
            System.exit(0);
        }
    }

    void populateMenuAndToolBar() {
        JMenuGroup jMenuFile = new JMenuGroup();
        String[] text = new String[] {"New", "Open"};
        ImageIcon[] icons = new ImageIcon[] {Icons.NEW, Icons.OPEN};
        String[] commands = new String[] {Commands.NEW, Commands.OPEN};
        String[] toolTips = new String[] {"Create a new file", "Open files"};
        int[] keyStrokes = new int[] {KeyEvent.VK_N, KeyEvent.VK_O};
        jToolBarMain.addGroup(icons, commands, toolTips);
        jMenuFile.addGroup(text, icons, commands, keyStrokes);
        jMenuFile.addSeparator();
        text = new String[] {"Reload", "Close", "Delete", "Save", "Export"};
        icons = new ImageIcon[] {Icons.RELOAD, Icons.CLOSE, Icons.DELETE,
                Icons.SAVE, Icons.EXPORT};
        commands = new String[] {Commands.RELOAD, Commands.CLOSE,
                   Commands.DELETE, Commands.SAVE, Commands.EXPORT};
        toolTips = new String[] {"Reload selected file", "Close selected file",
                   "Delete files", "Save current file", "Export current files"};
        keyStrokes = new int[] {KeyEvent.VK_R, KeyEvent.VK_C, KeyEvent.VK_D,
                     KeyEvent.VK_S, KeyEvent.VK_E};
        jToolBarMain.addGroup(icons, commands, toolTips);
        jMenuFile.addGroup(text, icons, commands, keyStrokes);
        jMenuFile.addSeparator();
        jMenuFile.addItem("Exit", null, Commands.EXIT, KeyEvent.VK_Q);
        jMenuBarMain.addGroup("File", jMenuFile);

        jMenuFile = new JMenuGroup();
        text = new String[] {"Dependency", "Search"};
        icons = new ImageIcon[] {Icons.DEPENDENCY, Icons.FIND};
        commands = new String[] {Commands.DEPENDENCY, Commands.FIND};
        toolTips = new String[] {"Shows the file dependency tree",
                   "Search a node tree"};
        keyStrokes = new int[] {KeyEvent.VK_T, KeyEvent.VK_F};
        jToolBarMain.addGroup(icons, commands, toolTips);
        jMenuFile.addGroup(text, icons, commands, keyStrokes);
        jMenuBarMain.addGroup("Tools", jMenuFile);

        jMenuFile = new JMenuGroup();
        text = new String[] {"Process", "Convert"};
        icons = new ImageIcon[] {Icons.PROCESS, Icons.CONVERT};
        commands = new String[] {Commands.PROCESS, Commands.CONVERT};
        toolTips = new String[] {"Expand file tree", "Simulate file evaluation"};
        keyStrokes = new int[] {KeyEvent.VK_Z, KeyEvent.VK_X};
        jToolBarMain.addGroup(icons, commands, toolTips);
        jMenuFile.addGroup(text, icons, commands, keyStrokes);
        jMenuBarMain.addGroup("Run", jMenuFile);

        UIManager.LookAndFeelInfo[] types = UIManager.getInstalledLookAndFeels();
        jMenuFile = new JMenuGroup();
        for(int i = 0; i < types.length; i++) {
            jMenuFile.addItem(types[i].getName(), null, types[i].getClassName(),
                JMenuGroup.NONE);
        }
        jMenuBarMain.addGroup("Look & Feel", jMenuFile);
    }

    void jMenuBarMain_actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        if(command.equals(Commands.EXIT)) {
            exit();
        }
        else {
            processCommand(command);
        }
    }

    void jToolBarMain_actionPerformed(ActionEvent e) {
        String command = e.getActionCommand();
        processCommand(command);
    }

    private void processCommand(String command) {
        if(Commands.NEW.equals(command)) {
            try {
                Node node = nodeFactory.getNode();
                addNode(node, null);
                Debugger.println("Editor", "new", "New node created!");
            }
            catch(Exception exc) {
                exc.printStackTrace();
                JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
        else if(Commands.OPEN.equals(command)) {
            jfc.setDialogTitle("Select the configuration files");
            if(jfc.showOpenDialog(this) == jfc.APPROVE_OPTION) {
                try {
                    File[] files = jfc.getSelectedFiles();
                    for(int i = 0; i < files.length; i++) {
                        if(openedFiles.get(files[i]) != null) {
                            JOptionPane.showMessageDialog(this,
                                "File '" + files[i] + "' already opened.",
                                "Atention", JOptionPane.INFORMATION_MESSAGE);
                        }
                        else {
                            openTab(files[i]);
                        }
                    }
                }
                catch(Exception exc) {
                    exc.printStackTrace();
                    JOptionPane.showMessageDialog(this, exc.getMessage(),
                        "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        else if(Commands.RELOAD.equals(command)) {
            try {
                if(tabNode != null) {
                    Node node = tabNode.design.getNode();
                    Include dec = (Include) node.getValue();
                    DataHolder holder = dec.getHolder();
                    if(holder != null) {
                        NodeReader reader = NodeBuilder.lookupNodeReader(
                                            holder);
                        Node newNode = reader.read(holder);
                        holder.close();
                        tabNode.design.setNode(newNode);
                        Debugger.println("Editor", "reload",
                            holder + "' reloaded.");
                    }
                }
            }
            catch(Exception exc) {
                exc.printStackTrace();
                JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
        else if(Commands.CLOSE.equals(command)) {
            NodeFilePanel panel = (NodeFilePanel) jTabbedPaneFiles.
                                  getSelectedComponent();
            if(panel != null) {
                closeTab(panel);
            }
        }
        else if(Commands.DELETE.equals(command)) {
            jfc.setDialogTitle("Select the files to delete");
            if(jfc.showOpenDialog(this) == jfc.APPROVE_OPTION) {
                File[] files = jfc.getSelectedFiles();
                for(int i = 0; i < files.length; i++) {
                    File file = files[i];
                    int option = JOptionPane.showConfirmDialog(this,
                                 "Confirm '" + file + "' deletion?", "Atention",
                                 JOptionPane.OK_CANCEL_OPTION,
                                 JOptionPane.QUESTION_MESSAGE);
                    if(option == JOptionPane.OK_OPTION) {
                        NodeFilePanel tab = (NodeFilePanel) openedFiles.get(
                                            file);
                        if(tab == null) {
                            if(!file.delete()) {
                                JOptionPane.showMessageDialog(this,
                                    "(1) Delete '" + file
                                    + "' fail. Try system explorer.", "Error",
                                    JOptionPane.ERROR_MESSAGE);
                            }
                            else {
                                Debugger.println("Editor", "delete",
                                    "(1) File '" + file + "' removed.");
                            }
                        }
                        else {
                            if(!file.delete()) {
                                JOptionPane.showMessageDialog(this,
                                    "(2) Delete '" + file
                                    + "' fail. Try system explorer.", "Error",
                                    JOptionPane.ERROR_MESSAGE);
                            }
                            else {
                                closeTab(tab);
                                Debugger.println("Editor", "delete",
                                    "(2) File '" + file + "' removed.");
                            }
                        }
                    }
                }
            }
        }
        else if(Commands.SAVE.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    try {
                        Include dec = (Include) node.getValue();
                        if(dec == null) {
                            jfc.setDialogTitle("Select the file name");
                            if(jfc.showSaveDialog(this)
                                == jfc.APPROVE_OPTION) {
                                File file = jfc.getSelectedFile();
                                if(openedFiles.get(file) != null) {
                                    String message = "Close '" + file
                                        + "' first.";
                                    Debugger.println("Editor", "save", message);
                                    JOptionPane.showMessageDialog(this,
                                        message, "Atention",
                                        JOptionPane.INFORMATION_MESSAGE);
                                }
                                else {
                                    int option = JOptionPane.OK_OPTION;
                                    if(file.exists()) {
                                        option = JOptionPane.showConfirmDialog(this,
                                                 "Override file '" + file
                                                 + "'?", "Atention",
                                                 JOptionPane.OK_CANCEL_OPTION);
                                    }
                                    if(option == JOptionPane.OK_OPTION) {
                                        DataHolder holder = holderFactory.
                                            produce(file);
                                        dec = new Include(holder, null);
                                        node.setValue(dec);
                                        tabNode.file = file;
                                        openedFiles.put(file, tabNode);
                                        setTitle("File: " + file);
                                    }
                                }
                            }
                        }
                        if(dec != null) {
                            DataHolder holder = dec.getHolder();
                            NodeWriter writer = NodeBuilder.lookupNodeWriter(
                                                holder);
                            writer.write(node, holder);
                            holder.close();
                            Debugger.println("Editor", "save",
                                holder + " saved.");
                        }
                    }
                    catch(Exception exc) {
                        exc.printStackTrace();
                        JOptionPane.showMessageDialog(this, exc.getMessage(),
                            "Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        }
        else if(Commands.EXPORT.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    try {
                        Include dec = (Include) node.getValue();
                        //DataHolder holder = (DataHolder)node.getValue();
                        jfc.setDialogTitle("Choose the export file name");
                        if(jfc.showSaveDialog(this) == jfc.APPROVE_OPTION) {
                            File file = jfc.getSelectedFile();
                            if(openedFiles.get(file) != null) {
                                String message = "Close '" + file + "' first.";
                                Debugger.println("Editor", "export", message);
                                JOptionPane.showMessageDialog(this, message,
                                    "Atention", JOptionPane.INFORMATION_MESSAGE);
                            }
                            else {
                                int option = JOptionPane.OK_OPTION;
                                if(file.exists()) {
                                    option = JOptionPane.showConfirmDialog(this,
                                             "Override file '" + file + "'?",
                                             "Atention",
                                             JOptionPane.OK_CANCEL_OPTION);
                                }
                                if(option == JOptionPane.OK_OPTION) {
                                    DataHolder holder = holderFactory.produce(
                                        file);
                                    dec.setHolder(holder);
                                }
                            }
                        }
                        if(dec != null) {
                            DataHolder holder = dec.getHolder();
                            NodeWriter writer = NodeBuilder.lookupNodeWriter(
                                                holder);
                            writer.write(node, holder);
                            holder.close();
                            Debugger.println("Editor", "export",
                                "File '" + holder + "' exported.");
                        }
                    }
                    catch(Exception exc) {
                        exc.printStackTrace();
                        JOptionPane.showMessageDialog(this, exc.getMessage(),
                            "Error", JOptionPane.ERROR_MESSAGE);
                    }
                }
            }
        }
        else if(Commands.PROCESS.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    addNodeProcessed(node, tabNode.file);
                    Debugger.println("Editor", "process", node + " expanded.");
                }
            }
            else {
                JOptionPane.showMessageDialog(this,
                    "First choose the reference configuration.", "Atention",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        }
        else if(Commands.CONVERT.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    addNodeConverted(node, tabNode.file);
                    Debugger.println("Editor", "convert", node + " converted.");
                }
            }
            else {
                JOptionPane.showMessageDialog(this,
                    "First choose the reference configuration.", "Atention",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        }
        else if(Commands.DEPENDENCY.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    addNodeDependency(node, tabNode.file);
                    Debugger.println("Editor", "dependency",
                        node + " expanded.");
                }
            }
            else {
                JOptionPane.showMessageDialog(this,
                    "First choose the reference configuration.", "Atention",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        }
        else if(Commands.FIND.equals(command)) {
            if(tabNode != null) {
                Node node = tabNode.design.getNode();
                if(node != null) {
                    addNodeSearch(tabNode, node);
                    Debugger.println("Editor", "search", node + " finished.");
                }
            }
            else {
                JOptionPane.showMessageDialog(this,
                    "First choose the reference configuration.", "Atention",
                    JOptionPane.INFORMATION_MESSAGE);
            }
        }
        else {
            try {
                UIManager.setLookAndFeel(command);
                SwingUtilities.updateComponentTreeUI(this);
            }
            catch(UnsupportedLookAndFeelException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE);
            }
            catch(IllegalAccessException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE);
            }
            catch(InstantiationException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE);
            }
            catch(ClassNotFoundException ex) {
                JOptionPane.showMessageDialog(this, ex.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE);
            }
        }
    }

    private class NodeFilePanel extends JTabbedPane {
        public File file;
        public JTreeNode tree;
        public JPaneNode design;
        public JTextArea text;
        public JScrollPane source;

        private boolean ignoreReload;

        public NodeFilePanel(File file, JTreeNode tree, JPaneNode design) {
            super(JTabbedPane.BOTTOM);
            this.file = file;
            this.tree = tree;
            this.design = design;
            addTab("Design", design);
            text = new JTextArea();
            text.setFont(new Font("Monospaced", 0, 12));
            source = new JScrollPane(text);
            addTab("Source", source);
            setSelectedComponent(design);
            addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    localStateChanged(e);
                }
            });
            ignoreReload = false;
        }

        private void localStateChanged(ChangeEvent e) {
            if(!ignoreReload) {
                if(getSelectedComponent() == source) {
                    try {
                        Node node = design.getNode();
                        if(node.getValue() != null) {
                            ByteArrayOutputStream bout = new
                                ByteArrayOutputStream();
                            DataHolder out = new DataHolderBinaryStream(bout);
                            Include dec = (Include) node.getValue();
                            DataHolder holder = dec.getHolder();
                            NodeWriter writer = NodeBuilder.lookupNodeWriter(
                                                holder);
                            writer.write(node, out);
                            String result = bout.toString();
                            out.close();
                            bout.close();
                            text.setText(result);
                            text.setCaretPosition(0);
                        }
                    }
                    catch(Exception exc) {
                        JOptionPane.showMessageDialog(this, exc.getMessage(),
                            "Atention", JOptionPane.INFORMATION_MESSAGE);
                        ignoreReload = true;
                        setSelectedComponent(design);
                    }
                }
                else if(getSelectedComponent() == design) {
                    try {
                        Node node = design.getNode();
                        if(node.getValue() != null) {
                            ByteArrayInputStream bin = new ByteArrayInputStream(
                                text.getText().getBytes());
                            DataHolder in = new DataHolderBinaryStream(bin);
                            Include dec = (Include) node.getValue();
                            DataHolder holder = dec.getHolder();
                            NodeReader reader = NodeBuilder.lookupNodeReader(
                                                holder);
                            Node newNode = reader.read(in);
                            ((Include) newNode.getValue()).setHolder(holder);
                            in.close();
                            bin.close();
                            design.setNode(newNode);
                        }
                    }
                    catch(Exception exc) {
                        JOptionPane.showMessageDialog(this, exc.getMessage(),
                            "Atention", JOptionPane.INFORMATION_MESSAGE);
                        ignoreReload = true;
                        setSelectedComponent(source);
                    }
                }
            }
            else {
                ignoreReload = false;
            }
        }
    }

    private void openTab(File file) throws DataHolderFactoryException,
        ClassNotFoundException, DataHolderException, NodeException {
        DataHolder holder = holderFactory.produce(file);
        NodeReader reader = NodeBuilder.lookupNodeReader(holder);
        Node node = reader.read(holder);
        holder.close();
        NodeFilePanel tab = addNode(node, file);
        openedFiles.put(file, tab);
        Debugger.println("Editor", "open", holder + " opened.");
    }

    private void closeTab(NodeFilePanel tab) {
        jTabbedPaneFiles.remove(tab);
        openedFiles.remove(tab.file);
        if(openedFiles.size() == 0) {
            setTitle("Editor");
        }
    }

    void jTabbedPaneFiles_stateChanged(ChangeEvent e) {
        NodeFilePanel tab = (NodeFilePanel) jTabbedPaneFiles.
                            getSelectedComponent();
        if(tab != null) {
            File file = tab.file;
            setTitle("File: " + (file != null ? "" + file : getTitle()));
            tabNode = tab;
        }
        else {
            tabNode = null;
        }
    }

    private NodeFilePanel addNode(Node node, File file) {
        JTreeNode tree = new JTreeNode();
        tree.setNodeEditor(nodeEditor);
        tree.setNodeRenderer(nodeRenderer);
        JPaneNode scroll = new JPaneNode(tree);
        scroll.setNode(node);
        scroll.setNodeEditor(nodeEditor);
        scroll.setNodeRenderer(nodeRenderer);
        NodeFilePanel panel = new NodeFilePanel(file, tree, scroll);
        jTabbedPaneFiles.addTab(file != null ? file.getName()
            : "Untitled_" + (serial++), panel);
        jTabbedPaneFiles.setSelectedComponent(panel);
        return panel;
    }

    private void addNodeProcessed(Node node, File file) {
        try {
            NodeProcessor processor = NodeBuilder.lookupNodeProcessor(
                                      Environment.class.getName());
            //NodeProcessor processor = new NodeProcessorAll();
            Node processed = processor.process(node);
            jTreeNodeProcessed.setNode(processed);
            jTreeNodeProcessed.expandAll();
            status("Processed '" + file + "'");
        }
        catch(Exception exc) {
            exc.printStackTrace();
            JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }

    private void addNodeConverted(Node node, File file) {
        try {
            NodeConversor conversor = NodeBuilder.lookupNodeConversor(
                                      Environment.class.getName());
            Environment environment = (Environment) conversor.convert(node);
            jPanelEnvironment.setEnvironment(environment);
            status("Converted '" + file + "'");
        }
        catch(Exception exc) {
            exc.printStackTrace();
            JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }

    private class JTreeNodeListener implements JTreeSmartListener {
        private JFrame messageParent;

        public JTreeNodeListener(JFrame messageParent) {
            this.messageParent = messageParent;
        }

        public void singleClick(JTreeSmartEvent e) {
            try {
                DefaultMutableTreeNode last = (DefaultMutableTreeNode) e.path.
                                              getLastPathComponent();
                if(!last.isRoot()) {
                    Object[] path = e.path.getPath();
                    LinkedList holders = new LinkedList();
                    int indexLastHolder = 0;
                    for(int i = 0; i < path.length; i++) {
                        Object obj = ((Node) ((DefaultMutableTreeNode)
                                     path[i]).getUserObject()).getValue();
                        if(obj instanceof Include) {
                            holders.add(((Include) obj).getHolder());
                            indexLastHolder = i;
                        }
                    }
                    Iterator iter = holders.iterator();
                    DataHolder holder = (DataHolder) iter.next();
                    while(iter.hasNext()) {
                        holder = holder.resolve((DataHolder) iter.next());
                    }
                    File file = (File) holder.getReference();
                    if(openedFiles.get(file) == null) {
                        openTab(file);
                    }
                    else {
                        jTabbedPaneFiles.setSelectedComponent((NodeFilePanel)
                            openedFiles.get(file));
                    }
                    if(indexLastHolder < path.length) {
                        NodeFilePanel opened = (NodeFilePanel) openedFiles.
                                               get(file);
                        Object[] select = new Object[path.length
                                          - indexLastHolder - 1];
                        System.arraycopy(path, indexLastHolder + 1, select, 0,
                            select.length);
                        if(select.length > 0) {
                            TreePath selectPath = new TreePath(select);
                            opened.tree.setSelectionPath(selectPath);
                        }
                    }
                }
            }
            catch(Exception exc) {
                exc.printStackTrace();
                Debugger.println("Editor", "Open dependency",
                    "Error:" + exc.getMessage());
            }
        }

        public void doubleClick(JTreeSmartEvent e) {}
    }

    private void addNodeDependency(Node node, File file) {
        try {
            NodeProcessor processor = new NodeProcessorDependency();
            //NodeProcessor processor = new NodeProcessorAll();
            Node processed = processor.process(node);
            jTreeDependency.setNode(processed);
            jTreeDependency.expandAll();
            jTabbedPaneBottom.setSelectedComponent(jPanelDependency);
            status("Dependency tree '" + file + "'");
        }
        catch(Exception exc) {
            exc.printStackTrace();
            JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                JOptionPane.ERROR_MESSAGE);
        }
    }

    private void addNodeSearch(NodeFilePanel tabNode, Node node) {
        int option = JOptionPane.showConfirmDialog(tabNode, matcherPanel,
                     "Search", JOptionPane.OK_CANCEL_OPTION);
        if(option == JOptionPane.OK_OPTION) {
            try {
                NodeMatcher matcher = matcherPanel.getMatcher();
                Node processed = matcher.process(node);
                jTreeSearch.setNode(processed);
                jTreeSearch.expandAll();
                jTabbedPaneBottom.setSelectedComponent(jPanelSearch);
                status("Search tree '" + matcher.getPattern() + "'");
            }
            catch(Exception exc) {
                exc.printStackTrace();
                JOptionPane.showMessageDialog(this, exc.getMessage(), "Error",
                    JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    private void status(String message) {
        jLabelStatus.setText(message);
    }
}
