package jpel.gui.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.File;
import java.util.Iterator;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import jpel.gui.tree.JTreeNodeEditor;
import jpel.gui.util.Icons;
import jpel.gui.util.JTreeClassSelector;
import jpel.gui.util.Utils;
import jpel.language.Abstraction;
import jpel.language.Declaration;
import jpel.language.DeclarationFunction;
import jpel.language.DeclarationModule;
import jpel.language.DeclarationNative;
import jpel.language.DeclarationType;
import jpel.language.EnvironmentBuilder;
import jpel.language.EnvironmentFactory;
import jpel.language.Expression;
import jpel.language.ExpressionBuilder;
import jpel.language.ExpressionId;
import jpel.language.ExpressionList;
import jpel.language.ExpressionParser;
import jpel.language.Include;
import jpel.tree.Node;
import jpel.tree.NodeBuilder;
import jpel.tree.NodeException;
import jpel.tree.NodeFactory;
import jpel.util.dataholder.DataHolder;
import jpel.util.dataholder.DataHolderBuilder;
import jpel.util.dataholder.DataHolderException;
import jpel.util.dataholder.DataHolderFactory;
import jpel.util.dataholder.DataHolderFactoryException;

public class NodeEditorLanguage extends JTreeNodeEditor {
    public static final String INCLUDE = "Include";
    public static final String MODULE = "Module";
    public static final String FUNCTION = "Function";
    public static final String NATIVE = "Native";
    public static final String[] types = new String[] {INCLUDE, MODULE, FUNCTION, NATIVE};

    ExpressionParser expressionParser;
    EnvironmentFactory environmentFactory;
    DataHolderFactory dataFactory;
    NodeFactory nodeFactory;
    JTreeClassSelector classSelector;
    Node root;
    Node newNode;
    Node updateNode;
    boolean changed;

    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JLabel jLabelMiddle = new JLabel();
    JLabel jLabelTop = new JLabel();
    JLabel jLabelBottom = new JLabel();
    JTextField jTextFieldTop = new JTextField();
    JTextField jTextFieldMiddle = new JTextField();
    JScrollPane jScrollPane1 = new JScrollPane();
    JTextArea jTextAreaBottom = new JTextArea();
    JButton jButtonOpen = new JButton();
    JLabel jLabelType = new JLabel();
    DefaultComboBoxModel typesModel = new DefaultComboBoxModel();
    JComboBox jComboBoxType = new JComboBox(typesModel);
    JButton jButtonOpenJar = new JButton();
    JLabel jLabelDescription = new JLabel();
    JScrollPane jScrollPane2 = new JScrollPane();
    JTextArea jTextAreaDescription = new JTextArea();
    JButton jButtonApply = new JButton();

    public NodeEditorLanguage() {
        super();
        try {
            jbInit();

            for(int i = 0; i < types.length; i++) {
                typesModel.addElement(types[i]);
            }
            enableType(FUNCTION);
            enableInterface(FUNCTION);

            expressionParser = ExpressionBuilder.getExpressionParser();
            environmentFactory = EnvironmentBuilder.getEnvironmentFactory();
            dataFactory = DataHolderBuilder.getDataHolderFactory();
            nodeFactory = NodeBuilder.getNodeFactory();
            classSelector = new JTreeClassSelector(true, false, false, true, true,
                            Expression.class);
        }
        catch(Exception e) {
            e.printStackTrace();
        }
    }

    private void jbInit() throws Exception {
        this.setLayout(gridBagLayout1);
        jLabelType.setText("Type:");
        jLabelTop.setText("Name:");
        jLabelMiddle.setText("Args:");
        jLabelBottom.setText("Body:");
        jButtonOpen.setIcon(Icons.OPEN);
        jButtonOpen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonOpen_actionPerformed(e);
            }
        });
        jComboBoxType.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                jComboBoxType_itemStateChanged(e);
            }
        });
        jTextFieldTop.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextFieldTop.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jTextFieldTop_actionPerformed(e);
            }
        });
        jTextFieldTop.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                jTextFieldTop_keyPressed(e);
            }
        });
        jTextFieldMiddle.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextFieldMiddle.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                jTextFieldMiddle_keyPressed(e);
            }
        });
        jTextFieldMiddle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jTextFieldMiddle_actionPerformed(e);
            }
        });
        jTextAreaBottom.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextAreaBottom.setLineWrap(true);
        jTextAreaBottom.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                jTextAreaBottom_keyPressed(e);
            }
        });
        jTextAreaDescription.setFont(new java.awt.Font("Monospaced", 0, 12));
        jTextAreaDescription.setLineWrap(true);
        jTextAreaDescription.setTabSize(4);
        jTextAreaDescription.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                jTextAreaDescription_keyPressed(e);
            }
        });
        jButtonOpenJar.setIcon(Icons.OPEN);
        jButtonOpenJar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonOpenJar_actionPerformed(e);
            }
        });
        jLabelDescription.setText("Description:");
        jButtonApply.setEnabled(false);
        jButtonApply.setText("Apply");
        jButtonApply.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                jButtonApply_actionPerformed(e);
            }
        });
        this.add(jLabelMiddle, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(5, 5,
            0, 5), 0, 0));
        this.add(jLabelTop, new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5,
            0, 5), 0, 0));
        this.add(jLabelBottom, new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0
            , GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
            new Insets(5, 5, 0, 5), 0, 0));
        this.add(jTextFieldTop, new GridBagConstraints(1, 2, 2, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
            new Insets(5, 0, 0, 5), 0, 0));
        this.add(jTextFieldMiddle, new GridBagConstraints(1, 3, 2, 1, 1.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
            new Insets(5, 0, 0, 5), 2, 0));
        this.add(jScrollPane1, new GridBagConstraints(1, 4, 2, 1, 1.0, 0.25
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5,
            0, 5, 5), 0, 0));
        this.add(jButtonOpen, new GridBagConstraints(3, 2, 1, 1, 0.0, 0.0
            , GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
            new Insets(5, 5, 0, 5), 0, 0));
        jScrollPane1.getViewport().add(jTextAreaBottom, null);
        this.add(jLabelType, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(0, 5,
            0, 5), 0, 0));
        this.add(jComboBoxType, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5, 0,
            0, 5), 0, 0));
        this.add(jButtonOpenJar, new GridBagConstraints(3, 4, 1, 3, 0.0, 0.0
            , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets(5, 5, 0, 5), 0, 0));
        this.add(jLabelDescription, new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0
            , GridBagConstraints.NORTHEAST, GridBagConstraints.NONE,
            new Insets(5, 5, 0, 5), 0, 0));
        this.add(jScrollPane2, new GridBagConstraints(1, 5, 2, 1, 1.0, 0.5
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5,
            0, 5, 5), 0, 0));
        this.add(jButtonApply, new GridBagConstraints(2, 0, 1, 2, 0.0, 0.0
            , GridBagConstraints.EAST, GridBagConstraints.VERTICAL,
            new Insets(5, 5, 0, 5), 0, 0));
        jScrollPane2.getViewport().add(jTextAreaDescription, null);
    }

    public void setChanged(boolean option) {
        changed = option;
        if(changed) {
            jButtonApply.setEnabled(true);
        }
        else {
            jButtonApply.setEnabled(false);
        }
    }

    public boolean isChanged() {
        return changed;
    }

    public void setNewNodeReference(Node reference) throws NodeException {
        newNode = reference;
        root = reference.getRoot();
        Object obj = newNode.getValue();
        if(newNode.isRoot()) {
            enableType(FUNCTION);
            enableInterface(FUNCTION);
        }
        else {
            if(obj instanceof Declaration) {
                Declaration dec = (Declaration) newNode.getValue();
                DeclarationType type = dec.getType();
                if(type.isModule()) {
                    enableType(FUNCTION);
                    enableInterface(FUNCTION);
                }
                else {
                    throw new NodeException(
                        "You can only add things to modules.");
                }
            }
            else if(obj instanceof DataHolder) {
                throw new NodeException(
                    "You can only add things to modules or to the root node.");
            }
        }
        updateUI();
    }

    public Node getNewNode() throws NodeException {
        try {
            String type = (String) jComboBoxType.getSelectedItem();
            if(!type.equals(INCLUDE)) {
                ExpressionId id = (ExpressionId) expressionParser.
                                  getExpression(getTop());
                if(type.equals(MODULE)) {
                    Declaration e;
                    e = new DeclarationModule(null, id, environmentFactory.empty());
                    e.setDescription(getDescription());
                    return nodeFactory.getNode(e);
                }
                else if(type.equals(FUNCTION)) {
                    ExpressionList formals = (ExpressionList)
                                             expressionParser.getExpression(
                                             getFormals());
                    Expression body = expressionParser.getExpression(getBottom());
                    Declaration e = new DeclarationFunction(null, id,
                                    new Abstraction(formals, body));
                    e.setDescription(getDescription());
                    return nodeFactory.getNode(e);
                }
                else if(type.equals(NATIVE)) {
                    ExpressionList formals = (ExpressionList)
                                             expressionParser.getExpression(
                                             getFormals());
                    ExpressionId body = (ExpressionId) expressionParser.
                                        getExpression(getBottom());
                    Abstraction abs = new Abstraction(formals, body);
                    Declaration e = new DeclarationNative(null, id, abs);
                    e.setDescription(getDescription());
                    return nodeFactory.getNode(e);
                }
                else {
                    throw new NodeException("'" + type
                        + "' is not a valid type.");
                }
            }
            else {
                DataHolder h = dataFactory.produce(getTop());
                ExpressionId name = null;
                if(getMiddle() != null) {
                    name = (ExpressionId) expressionParser.getExpression(
                           getMiddle());
                }
                Include e = new Include(h, name);
                e.setDescription(getDescription());
                return nodeFactory.getNode(e);
            }
        }
        catch(Exception exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
    }

    public void setUpdateNodeReference(Node reference) throws NodeException {
        updateNode = reference;
        root = reference.getRoot();
        setNodeReference(updateNode);
        if(updateNode.isRoot()) {
            jLabelType.setEnabled(false);
            jComboBoxType.setEnabled(false);
            jLabelTop.setEnabled(false);
            jTextFieldTop.setEnabled(false);
            jLabelMiddle.setEnabled(false);
            jTextFieldMiddle.setEnabled(false);
            jButtonOpen.setEnabled(false);
        }
        updateUI();
    }

    public Node getUpdatedNode() throws NodeException {
        if(!isChanged()) { // se nao mudou retorna o n original
            //System.out.println("Node:"+updateNode+" has not been modified.");
            return updateNode;
        }
        if(updateNode.isRoot()) { // se  raiz deve mudar apenas a descrio.
            Include dec = (Include) updateNode.getValue();
            dec.setDescription(getDescription());
            return updateNode;
        }
        try {
            String type = (String) jComboBoxType.getSelectedItem();
            if(!type.equals(INCLUDE)) {
                ExpressionId id = (ExpressionId) expressionParser.
                                  getExpression(getTop());
                if(type.equals(MODULE)) {
                    Declaration e;
                    e = new DeclarationModule(null, id, environmentFactory.empty());
                    e.setDescription(getDescription());
                    updateNode.setValue(e);
                    return updateNode;
                }
                else if(type.equals(FUNCTION)) {
                    ExpressionList formal = (ExpressionList) expressionParser.
                                            getExpression(getFormals());
                    Expression body = expressionParser.getExpression(getBottom());
                    Declaration e = new DeclarationFunction(null, id,
                                    new Abstraction(formal, body));
                    e.setDescription(getDescription());
                    updateNode.setValue(e);
                    return updateNode;
                }
                else if(type.equals(NATIVE)) {
                    ExpressionList formals = (ExpressionList)
                                             expressionParser.getExpression(
                                             getFormals());
                    ExpressionId body = (ExpressionId) expressionParser.
                                        getExpression(getBottom());
                    Abstraction abs = new Abstraction(formals, body);
                    Declaration e = new DeclarationNative(null, id, abs);
                    e.setDescription(getDescription());
                    updateNode.setValue(e);
                    return updateNode;
                }
                else {
                    throw new NodeException("'" + type
                        + "' is not a valid type.");
                }
            }
            else {
                DataHolder holder = dataFactory.produce(getTop());
                ExpressionId name = null;
                if(getMiddle() != null) {
                    name = (ExpressionId) expressionParser.getExpression(
                           getMiddle());
                }
                Include e = new Include(holder, name);
                e.setDescription(getDescription());
                updateNode.setValue(e);
                return updateNode;
            }
        }
        catch(Exception exc) {
            throw new NodeException(exc.getMessage(), exc);
        }
    }

    private void setNodeReference(Node reference) throws NodeException {
        Object value = reference.getValue();
        if(value instanceof Declaration) {
            Declaration declaration = (Declaration) value;
            DeclarationType type = declaration.getType();
            if(type.isFunction()) {
                enableType(FUNCTION);
                enableInterface(FUNCTION);
                DeclarationFunction dec = (DeclarationFunction) declaration;
                putTop(String.valueOf(dec.getName()));
                Abstraction abs = dec.getAbstraction();
                if(abs != null) {
                    putFormals(abs.getArguments());
                    putBottom(String.valueOf(abs.getBody()));
                }
                else {
                    putMiddle("");
                    putBottom("");
                }
                putDescription(dec.getDescription());
            }
            else if(type.isNative()) {
                enableType(NATIVE);
                enableInterface(NATIVE);
                DeclarationNative dec = (DeclarationNative) value;
                putTop(String.valueOf(dec.getName()));
                Abstraction abs = dec.getAbstraction();
                putFormals(abs.getArguments());
                putBottom(String.valueOf(abs.getBody()));
                putDescription(dec.getDescription());
            }
            else if(type.isModule()) {
                enableType(MODULE);
                enableInterface(MODULE);
                DeclarationModule dec = (DeclarationModule) declaration;
                putTop(String.valueOf(dec.getName()));
                putMiddle("");
                putBottom("");
                putDescription(dec.getDescription());
            }
        }
        else if(value instanceof Include) {
            enableType(INCLUDE);
            enableInterface(INCLUDE);
            Include dec = (Include) value;
            putTop(String.valueOf(dec.getHolder().getReference()));
            if(dec.getName() != null) {
                putMiddle(dec.getName().getName());
            }
            putDescription(dec.getDescription());
        }
        setChanged(false);
    }

    private void putTop(String str) {
        jTextFieldTop.setText(str != null ? str : "");
        jTextFieldTop.setCaretPosition(0);
    }

    private String getTop() {
        String str = jTextFieldTop.getText().trim();
        return "".equals(str) ? null : str;
    }

    private void putMiddle(String str) {
        jTextFieldMiddle.setText(str != null ? str : "");
        jTextFieldMiddle.setCaretPosition(0);
    }

    private String getMiddle() {
        String str = jTextFieldMiddle.getText().trim();
        return "".equals(str) ? null : str;
    }

    private void putBottom(String str) {
        jTextAreaBottom.setText(str != null ? str : "");
        jTextAreaBottom.setCaretPosition(0);
    }

    private String getBottom() {
        String str = jTextAreaBottom.getText().trim();
        return "".equals(str) ? null : str;
    }

    private void putDescription(String description) {
        jTextAreaDescription.setText(description != null ? description : "");
        jTextAreaDescription.setCaretPosition(0);
    }

    private String getDescription() {
        String str = jTextAreaDescription.getText().trim();
        return "".equals(str) ? null : str;
    }

    private void putFormals(ExpressionList formal) throws NodeException {
        String str = null;
        jTextFieldMiddle.setText("");
        Iterator nav = formal.iterator();
        while(nav.hasNext()) {
            str = (str == null ? String.valueOf(nav.next())
                  : str + "," + String.valueOf(nav.next()));
        }
        jTextFieldMiddle.setText(str);
    }

    private String getFormals() throws NodeException {
        String f = getMiddle();
        return f != null ? "[" + f + "]" : "[]";
    }

    public JTreeNodeEditor mirror() {
        return new NodeEditorLanguage();
    }

    void jButtonOpen_actionPerformed(ActionEvent e) {
        JFileChooser jfc = Utils.getDefaultFileChooser(null, null);
        if(jfc.showOpenDialog(this) == jfc.APPROVE_OPTION) {
            File file = jfc.getSelectedFile();
            if(root.getValue() == null) {
                jTextFieldTop.setText(String.valueOf(file));
            }
            else {
                try {
                    Include dec = (Include) root.getValue();
                    DataHolder base = dec.getHolder();
                    DataHolder holder = dataFactory.produce(file);
                    DataHolder relative = base.relative(holder);
                    jTextFieldTop.setText(String.valueOf(relative.
                        getReference()));
                }
                catch(DataHolderFactoryException ex) {
                    jTextFieldTop.setText(String.valueOf(file));
                }
                catch(DataHolderException ex) {
                    jTextFieldTop.setText(String.valueOf(file));
                }
            }
            setChanged(true);
        }
    }

    void jButtonOpenJar_actionPerformed(ActionEvent e) {
        try {
            Object[] options = new Object[] {"Ok", "Cancel"};
            int option = JOptionPane.showOptionDialog(
                         this,
                         classSelector,
                         "Class selection",
                         JOptionPane.OK_CANCEL_OPTION,
                         JOptionPane.QUESTION_MESSAGE,
                         null,
                         options,
                         options[0]
                         );
            if(option == 0) {
                String className = classSelector.getSelectedClass();
                if(className != null) {
                    jTextAreaBottom.setText(className);
                    setChanged(true);
                }
            }
        }
        catch(Exception exc) {
            exc.printStackTrace();
        }
    }

    void jComboBoxType_itemStateChanged(ItemEvent e) {
        enableInterface((String) e.getItem());
        setChanged(true);
    }

    public void enableType(String type) {
        typesModel.setSelectedItem(type);
    }

    public void enableInterface(String type) {
        enableAll();
        if(type.equals(FUNCTION)) {
            jButtonOpen.setEnabled(false);
            jButtonOpenJar.setEnabled(false);
            jLabelMiddle.setText("Args:");
            jLabelBottom.setText("Body:");
        }
        else if(type.equals(NATIVE)) {
            jButtonOpen.setEnabled(false);
            jLabelMiddle.setText("Args:");
            jLabelBottom.setText("Class:");
        }
        else if(type.equals(MODULE)) {
            jButtonOpen.setEnabled(false);
            jButtonOpenJar.setEnabled(false);
            jLabelMiddle.setText("Args:");
            jTextFieldMiddle.setEnabled(false);
            jLabelBottom.setText("Body:");
            jTextAreaBottom.setEnabled(false);
        }
        else if(type.equals(INCLUDE)) {
            jButtonOpenJar.setEnabled(false);
            jLabelMiddle.setText("As:");
            jLabelBottom.setEnabled(false);
            jTextAreaBottom.setEnabled(false);
        }
    }

    private void enableAll() {
        jLabelType.setEnabled(true);
        jComboBoxType.setEnabled(true);

        jLabelTop.setEnabled(true);
        jTextFieldTop.setEnabled(true);

        jButtonOpen.setEnabled(true);
        jLabelMiddle.setEnabled(true);
        jTextFieldMiddle.setEnabled(true);

        jLabelBottom.setEnabled(true);
        jTextAreaBottom.setEnabled(true);
        jButtonOpenJar.setEnabled(true);

        jLabelDescription.setEnabled(true);
        jTextAreaDescription.setEnabled(true);
    }

    void jButtonApply_actionPerformed(ActionEvent e) {
        apply();
    }

    void jTextFieldTop_keyPressed(KeyEvent e) {
        setChanged(true);
    }

    void jTextFieldTop_actionPerformed(ActionEvent e) {
        apply();
    }

    void jTextFieldMiddle_keyPressed(KeyEvent e) {
        setChanged(true);
    }

    void jTextFieldMiddle_actionPerformed(ActionEvent e) {
        apply();
    }

    void jTextAreaBottom_keyPressed(KeyEvent e) {
        setChanged(true);
    }

    void jTextAreaDescription_keyPressed(KeyEvent e) {
        setChanged(true);
    }

    private void apply() {
        try {
            getUpdatedNode();
            setChanged(false);
        }
        catch(NodeException exc) {
            JOptionPane.showMessageDialog(this, exc.getMessage(), "Atention",
                JOptionPane.ERROR_MESSAGE);
            setChanged(true);
        }
    }
}
