package jpel.gui.bridge;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.File;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import jpel.gui.tree.JTreeNodeRenderer;
import jpel.language.Declaration;
import jpel.language.DeclarationType;
import jpel.language.ExpressionId;
import jpel.language.Include;
import jpel.tree.Node;

/**
 * Ajusta a viasualizao de uma rvore de ns cujo contedo so declarao
 * da linguagem funcional.
 */
public class NodeRendererLanguage extends JLabel implements JTreeNodeRenderer {

    private Color selection = new Color(207, 207, 255);
    private Color zigzag = new Color(235, 235, 235);

    public NodeRendererLanguage() {
        setOpaque(true);
        setFont(new Font(getFont().getFontName(), Font.BOLD, 11));
    }

    public Component getTreeCellRendererComponent(JTree tree, Object value,
        boolean selected, boolean expanded, boolean leaf, int row,
        boolean hasFocus) {
        if(selected) {
            setBackground(selection);
        }
        else {
            setForeground(tree.getForeground());
            if(row % 2 == 0) {
                setBackground(zigzag);
            }
            else {
                setBackground(tree.getBackground());
            }
        }
        Node node = (Node) ((DefaultMutableTreeNode) value).getUserObject();
        if(node != null) {
            Object data = node.getValue();
            if(data instanceof Declaration) {
                Declaration dec = (Declaration) data;
                DeclarationType type = dec.getType();
                if(type.isFunction()) {
                    setIcon(IconsLanguage.FUNCTION);
                    setText("" + dec.getName());
                }
                else if(type.isNative()) {
                    setIcon(IconsLanguage.NATIVE);
                    setText("" + dec.getName());
                }
                else if(type.isModule()) {
                    setIcon(IconsLanguage.MODULE);
                    setText("" + dec.getName());
                }
            }
            else if(data instanceof Include) {
                setIcon(IconsLanguage.INCLUDE);
                Object reference = (((Include) data).getHolder()).getReference();
                if(node.isRoot() && (reference instanceof File)) {
                    setText(String.valueOf(((File) reference).getName()));
                }
                else {
                    ExpressionId name = ((Include) data).getName();
                    setText(String.valueOf(reference) + (name != null
                        ? " as " + name.getName() : ""));
                }
            }
            else {
                setIcon(IconsLanguage.FUNCTION);
                setText(null);
            }
            if(node.isRoot()) {
                setIcon(IconsLanguage.HOME);
            }
        }
        else {
            setIcon(IconsLanguage.FUNCTION);
            setText(null);
        }
        return this;
    }
}
