package jpel.gui.language;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

import java.awt.Font;
import javax.swing.tree.DefaultMutableTreeNode;

import jpel.gui.util.JTreeSmart;
import jpel.language.BadTypedException;
import jpel.language.Abstraction;
import jpel.language.Environment;
import jpel.language.Expression;
import jpel.language.ExpressionId;
import jpel.language.IdNotFoundException;

/**
 * Fornece uma visualizao de uma arvore de objetos do tipo Environment.
 */
public class JTreeEnvironment extends JTreeSmart {

    private DefaultMutableTreeNode root;
    private Environment environment;
    private boolean showLang;
    private boolean showJvm;

    public JTreeEnvironment() {
        this(new DefaultMutableTreeNode());
        this.putClientProperty("JTree.lineStyle", "Angled");
        setFont(new Font(getFont().getFontName(), Font.PLAIN, 11));
    }

    public JTreeEnvironment(DefaultMutableTreeNode root) {
        super(root);
        this.root = root;
    }

    public boolean isShowLang() {
        return showLang;
    }

    public void setShowLang(boolean showLang) {
        this.showLang = showLang;
    }

    public boolean isShowJvm() {
        return showJvm;
    }

    public void setShowJvm(boolean showJvm) {
        this.showJvm = showJvm;
    }

    public static class EnvironmentEntry {
        public String id;
        public Abstraction abstraction;
        public EnvironmentEntry(String id, Abstraction abstraction) {
            this.id = id;
            this.abstraction = abstraction;
        }

        public String toString() {
            String result = null;
            if(abstraction != null) {
                result = id + " -> " + abstraction;
            }
            else {
                result = id;
            }
            return result;
        }
    }

    public Environment getEnvironment() {
        return environment;
    }

    public void setEnvironment(Environment environment) throws
        IdNotFoundException, BadTypedException {
        root.setUserObject(new EnvironmentEntry("ENV", null));
        root.removeAllChildren();
        Iterator iter = environment.keys();
        while(iter.hasNext()) {
            ExpressionId id = (ExpressionId) iter.next();
            if(!id.isLang() || showLang) {
                if(!id.isJvm() || showJvm) {
                    recursive(root, id, environment);
                }
            }
        }
        updateUI();
        this.environment = environment;
    }

    private void recursive(DefaultMutableTreeNode root, ExpressionId id,
        Environment env) throws IdNotFoundException, BadTypedException {
        Abstraction abstraction = null;
        Environment subEnvironment = null;
        try {
            abstraction = env.lookup(id);
        }
        catch(IdNotFoundException exc) {}
        try {
            subEnvironment = env.lookupSub(id);
        }
        catch(IdNotFoundException exc) {}

        DefaultMutableTreeNode node = new DefaultMutableTreeNode(
                                      new EnvironmentEntry(id.getName(),
                                      abstraction)
                                      );
        root.add(node);
        if(subEnvironment != null) {
            Iterator iter = subEnvironment.keys();
            while(iter.hasNext()) {
                ExpressionId newId = (ExpressionId) iter.next();
                if(!newId.isLang() || showLang) {
                    if(!newId.isJvm() || showJvm) {
                        recursive(node, newId, subEnvironment);
                    }
                }
            }
        }
    }
}
