package jpel.gui.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import jpel.gui.util.Commands;
import jpel.gui.util.Icons;
import jpel.gui.util.JToolBarGroup;
import jpel.tree.Node;
import jpel.tree.NodeException;
import jpel.util.Debugger;

public class JPaneNode extends JPanel implements ActionListener,
    TreeSelectionListener {

    private JTreeNode tree;

    private boolean ignoreChange;
    private TreePath lastPath;
    private JTreeNodeEditor nodeEditorMirror;

    public JPaneNode() {
        this( new JTreeNode() );
    }

    public JPaneNode( JTreeNode tree ) {
        super();
        setLayout( new BorderLayout() );

        JPanel center = new JPanel();
        center.setLayout( new BorderLayout() );
        ImageIcon[] icons = new ImageIcon[] {
                            Icons.EXPAND, Icons.COLAPSE,
                            Icons.INSERT, Icons.REMOVE, Icons.UPDATE,
                            Icons.BACK, Icons.UP, Icons.DOWN
        };
        String[] commands = new String[] {
                            Commands.EXPAND, Commands.COLAPSE,
                            Commands.INSERT, Commands.REMOVE, Commands.UPDATE,
                            Commands.BACK, Commands.UP, Commands.DOWN
        };
        String[] toolTips = new String[] {
                            "Expand selected node", "Colapse selected node",
                            "Insert news nodes", "Remove selected nodes",
                            "Update selected nodes",
                            "Promote nodes to it parents", "Move nodes up",
                            "Move nodes down"
        };
        JToolBarGroup toolbar = new JToolBarGroup();
        toolbar.addGroup( icons, commands, toolTips );
        toolbar.addActionListener( this );
        center.add( toolbar, BorderLayout.NORTH );
        JScrollPane pane = new JScrollPane( tree );
        center.add( pane, BorderLayout.CENTER );

        JTreeNodeEditor editor = tree.getNodeEditor();
        if( editor != null ) {
            center.remove( toolbar );
            toolbar.setOrientation( SwingConstants.VERTICAL );
            center.add( toolbar, BorderLayout.EAST );
            JSplitPane split = new JSplitPane();
            split.setOneTouchExpandable( true );
            split.setContinuousLayout( true );
            split.setOrientation( SwingConstants.VERTICAL );
            split.setLeftComponent( center );
            split.setResizeWeight( 0.5 );
            nodeEditorMirror = editor.mirror();
            split.setRightComponent( nodeEditorMirror );
            add( split, BorderLayout.CENTER );
        }
        else {
            add( center, BorderLayout.CENTER );
        }
        tree.addTreeSelectionListener( this );
        this.tree = tree;
        this.ignoreChange = false;
        this.lastPath = null;
    }

    public Node getNode() {
        return tree.getNode();
    }

    public void setNode( Node node ) {
        tree.setNode( node );
        tree.expandAll();
    }

    public JTreeNodeEditor getNodeEditor() {
        return tree.getNodeEditor();
    }

    public void setNodeEditor( JTreeNodeEditor nodeEditor ) {
        tree.setNodeEditor( nodeEditor );
    }

    public JTreeNodeRenderer getNodeRenderer() {
        return tree.getNodeRenderer();
    }

    public void setNodeRenderer( JTreeNodeRenderer nodeRenderer ) {
        tree.setNodeRenderer( nodeRenderer );
    }

    public void valueChanged( TreeSelectionEvent e ) {
        if( nodeEditorMirror != null ) {
            if( !ignoreChange ) {
                TreePath path = e.getNewLeadSelectionPath();
                if( path != null ) {
                    try {
                        DefaultMutableTreeNode node = ( DefaultMutableTreeNode )
                            path.getLastPathComponent();
                        if( lastPath != null ) {
                            DefaultMutableTreeNode last = (
                                DefaultMutableTreeNode ) lastPath.
                                getLastPathComponent();
                            Node data = nodeEditorMirror.getUpdatedNode();
                            last.setUserObject( data );
                        }
                        nodeEditorMirror.setUpdateNodeReference( ( Node ) node.
                            getUserObject() );
                        lastPath = path;
                    }
                    catch( NodeException exc ) {
                        ignoreChange = true;
                        JOptionPane.showMessageDialog( this, exc.getMessage(),
                            "Error", JOptionPane.ERROR_MESSAGE );
                        tree.setSelectionPath( e.getOldLeadSelectionPath() );
                    }
                }
            }
            else {
                Debugger.println( "PaneNode", "selection",
                    "Undo selection, update error." );
            }
            ignoreChange = false;
        }
    }

    public void actionPerformed( ActionEvent e ) {
        String command = e.getActionCommand();

        if( Commands.EXPAND.equals( command ) ) {
            tree.expandCurrent();
        }
        else if( Commands.COLAPSE.equals( command ) ) {
            tree.colapseCurrent();
        }
        else if( Commands.INSERT.equals( command ) ) {
            try {
                tree.insert();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        else if( Commands.UPDATE.equals( command ) ) {
            try {
                tree.update();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        else if( Commands.REMOVE.equals( command ) ) {
            try {
                tree.remove();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        else if( Commands.BACK.equals( command ) ) {
            try {
                tree.promote();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        else if( Commands.UP.equals( command ) ) {
            try {
                tree.moveUp();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
        else if( Commands.DOWN.equals( command ) ) {
            try {
                tree.moveDown();
            }
            catch( Exception exc ) {
                JOptionPane.showMessageDialog( this, exc.getMessage(),
                    "Atention", JOptionPane.INFORMATION_MESSAGE );
            }
        }
    }
}
