package jpel.gui.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import java.awt.Container;
import javax.swing.JInternalFrame;
import javax.swing.JOptionPane;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import jpel.gui.util.JTreeSmart;
import jpel.gui.util.JTreeSmartEvent;
import jpel.gui.util.JTreeSmartListener;
import jpel.tree.Node;
import jpel.tree.NodeException;

/**
 * Fornece uma visualizao de uma arvore de objetos do tipo Node.
 */
public class JTreeNode extends JTreeSmart implements JTreeSmartListener {

    private DefaultTreeModel model;
    private JTreeNodeEditor nodeEditor;

    public JTreeNode() {
        this( new DefaultMutableTreeNode() );
    }

    private JTreeNode( DefaultMutableTreeNode root ) {
        this( new DefaultTreeModel( root ) );
    }

    private JTreeNode( DefaultTreeModel model ) {
        super( model );
        this.model = model;
        this.putClientProperty( "JTree.lineStyle", "Angled" );
        addJTreeSmartListener( this );
    }

    public Node getNode() {
        return( Node ) ( ( DefaultMutableTreeNode ) model.getRoot() ).
            getUserObject();
    }

    public void setNode( Node node ) {
        DefaultMutableTreeNode root = new DefaultMutableTreeNode( node );
        Iterator iter = node.children();
        while( iter.hasNext() ) {
            recursive( root, ( Node ) iter.next() );
        }
        model.setRoot( root );
        setSelectionPath( new TreePath( root ) );
    }

    private void recursive( DefaultMutableTreeNode parent, Node child ) {
        DefaultMutableTreeNode node = new DefaultMutableTreeNode( child );
        parent.add( node );
        Iterator iter = child.children();
        while( iter.hasNext() ) {
            recursive( node, ( Node ) iter.next() );
        }
    }

    public JTreeNodeEditor getNodeEditor() {
        return nodeEditor;
    }

    public void setNodeEditor( JTreeNodeEditor nodeEditor ) {
        this.nodeEditor = nodeEditor;
    }

    public JTreeNodeRenderer getNodeRenderer() {
        return( JTreeNodeRenderer )this.getCellRenderer();
    }

    public void setNodeRenderer( JTreeNodeRenderer nodeRenderer ) {
        setCellRenderer( nodeRenderer );
    }

    public void singleClick( JTreeSmartEvent e ) {
    }

    public void doubleClick( JTreeSmartEvent e ) {
        try {
            update();
        }
        catch( NodeException exc ) {
            JOptionPane.showMessageDialog( this, exc.getMessage(), "Error",
                JOptionPane.ERROR_MESSAGE );
        }
    }

    public void insert() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    // modelo de dados
                    Node nodeTarget = ( Node ) treeTarget.getUserObject();
                    nodeEditor.setNewNodeReference( nodeTarget );
                    Object[] options = new Object[] {"Ok", "Cancel"};
                    int option = JOptionPane.showOptionDialog( getShowParent(),
                                 nodeEditor, "Node editor - New node",
                                 JOptionPane.DEFAULT_OPTION,
                                 JOptionPane.PLAIN_MESSAGE, null, options,
                                 options[0] );
                    if( option == 0 ) {
                        //modelo de dados.
                        Node newNode = nodeEditor.getNewNode();
                        nodeTarget.insert( newNode );
                        // interface grtica
                        DefaultMutableTreeNode treeNode = new
                            DefaultMutableTreeNode( newNode );
                        treeTarget.add( treeNode );
                        list.add( path.pathByAddingChild( treeNode ) );
                    }
                }
            }
            updateUI();
            paths = new TreePath[list.size()];
            list.toArray( paths );
            setSelectionPaths( paths );
        }
    }

    public void update() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null && nodeEditor != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    if( !treeTarget.isRoot() ) {
                        // modelo de dados
                        Node nodeTarget = ( Node ) treeTarget.getUserObject();
                        nodeEditor.setUpdateNodeReference( nodeTarget );
                        Object[] options = new Object[] {"Ok", "Cancel"};
                        int option = JOptionPane.showOptionDialog(
                                     getShowParent(), nodeEditor,
                                     "Node editor - Update node",
                                     JOptionPane.DEFAULT_OPTION,
                                     JOptionPane.PLAIN_MESSAGE, null, options,
                                     options[0] );
                        if( option == 0 ) {
                            //modelo de dados.
                            Node newNode = nodeEditor.getUpdatedNode();
                            // interfaces grafica
                            // reseleciona os caminhos.
                        }
                    }
                }
            }
            updateUI();
            setSelectionPaths( paths );
        }
    }

    public void remove() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    if( !treeTarget.isRoot() ) {
                        Node nodeTarget = ( Node ) treeTarget.getUserObject();
                        int option = JOptionPane.showConfirmDialog(
                                     getShowParent(),
                                     "Confirm " + nodeTarget.getValue()
                                     + " deletion?", "Atention",
                                     JOptionPane.OK_CANCEL_OPTION );
                        if( option == JOptionPane.OK_OPTION ) {
                            // modelo de dados
                            nodeTarget.remove();
                            // interface grfica
                            list.add( path.getParentPath() );
                            treeTarget.removeFromParent();
                        }
                    }
                    else {
                        list.add( path );
                    }
                }
            }
            updateUI();
            paths = new TreePath[list.size()];
            list.toArray( paths );
            setSelectionPaths( paths );
        }
    }

    public void moveUp() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    if( !treeTarget.isRoot() ) {
                        DefaultMutableTreeNode treeParent = (
                            DefaultMutableTreeNode ) path.getParentPath().
                            getLastPathComponent();
                        //modelo de dados
                        Node nodeTarget = ( Node ) treeTarget.getUserObject();
                        Node nodeParent = nodeTarget.getParent();
                        nodeTarget.moveUp();
                        int newIndex = nodeTarget.indexOf();
                        //interface grfica
                        treeTarget.removeFromParent();
                        treeParent.insert( treeTarget, newIndex );
                        list.add( new TreePath( treeTarget.getPath() ) );
                    }
                    else {
                        list.add( path );
                    }
                }
            }
            updateUI();
            paths = new TreePath[list.size()];
            list.toArray( paths );
            setSelectionPaths( paths );
        }
    }

    public void moveDown() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    if( !treeTarget.isRoot() ) {
                        DefaultMutableTreeNode treeParent = (
                            DefaultMutableTreeNode ) path.getParentPath().
                            getLastPathComponent();
                        //modelo de dados
                        Node nodeTarget = ( Node ) treeTarget.getUserObject();
                        Node nodeParent = nodeTarget.getParent();
                        nodeTarget.moveDown();
                        int newIndex = nodeTarget.indexOf();
                        //interface grfica
                        treeTarget.removeFromParent();
                        treeParent.insert( treeTarget, newIndex );
                        list.add( new TreePath( treeTarget.getPath() ) );
                    }
                    else {
                        list.add( path );
                    }
                }
            }
            updateUI();
            paths = new TreePath[list.size()];
            list.toArray( paths );
            setSelectionPaths( paths );
        }
    }

    public void promote() throws NodeException {
        TreePath[] paths = getSelectionPaths();
        if( paths != null ) {
            List list = new LinkedList();
            for( int i = 0; i < paths.length; i++ ) {
                TreePath path = paths[i];
                if( path != null ) {
                    DefaultMutableTreeNode treeTarget = (
                        DefaultMutableTreeNode ) path.getLastPathComponent();
                    if( !treeTarget.isRoot() ) {
                        DefaultMutableTreeNode treeParent = (
                            DefaultMutableTreeNode ) path.getParentPath().
                            getLastPathComponent();
                        if( !treeParent.isRoot() ) {
                            // modelo de dados
                            Node nodeTarget = ( Node ) treeTarget.getUserObject();
                            Node nodeParent = nodeTarget.getParent();
                            nodeTarget.promote();
                            int newIndex = nodeTarget.indexOf();
                            // interface grfica
                            treeTarget.removeFromParent();
                            ( ( DefaultMutableTreeNode ) treeParent.getParent() ).
                                insert( treeTarget, newIndex );
                        }
                    }
                    list.add( new TreePath( treeTarget.getPath() ) );
                }
            }
            updateUI();
            paths = new TreePath[list.size()];
            list.toArray( paths );
            setSelectionPaths( paths );
        }
    }

    public Container getShowParent() {
        Container parent = getParent();
        if( parent != null ) {
            Container tmp;
            while( ( tmp = parent.getParent() ) != null
                           && !( tmp instanceof JInternalFrame ) ) {
                parent = tmp;
            }
        }
        else {
            parent = this;
        }
        //System.out.println("PARENT is "+parent);
        return parent;
    }
}
