package jpel.gui.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import javax.swing.JPanel;

import jpel.tree.Node;
import jpel.tree.NodeException;

/**
 * Representa um editor de ns. Quem desejar editar um n usando a JTreeNode, deve
 * ajustar o seu editor de ns para uma sub-classe deste NodeEditor.
 */
public abstract class JTreeNodeEditor extends JPanel {

    /**
     * Ajusta o n referncia para incluso.
     * @param reference N referencia.
     * @throws NodeException Quando h problemas no ajuste do n.
     */
    public abstract void setNewNodeReference( Node reference ) throws
        NodeException;

    /**
     * Retorna um n novo para incluso em uma rvore. Usa informaes do n
     * referncia para ajudar na criao do n.
     * @return Um n novo.
     * @throws NodeException Quando h problemas na criao do n.
     */
    public abstract Node getNewNode() throws NodeException;

    /**
         * Indica se o contedo do editor foi modificado ou no. Este mtodo auxilia
         * na atualizao dos ns, apenas ns que foram modificado devem ser atualizado.
         * @return true, caso o contedo tenha sido modificado, false, caso contrrio.
     */
    public abstract boolean isChanged();

    /**
     * Ajusta o n referncia para atualizao.
     * @param reference N referencia.
     * @throws NodeException Quando h problemas no ajuste do n.
     */
    public abstract void setUpdateNodeReference( Node reference ) throws
        NodeException;

    /**
     * Retorna o n referencia com seus dados modificados. O n original s 
     * modificado quando os dados do editor tiverem sido modificados,
     * economizando assim processamento para atualizao dos ns.
     * @return O n referencia modificado.
     * @throws NodeException Quando h problemas na atualizao do n.
     */
    public abstract Node getUpdatedNode() throws NodeException;

    /**
     * Cria um editor do mesmo tipo.
     * @return Um editor novo do mesmo tipo que foi implementado, e null quando
     * no  possvel criar a cpia.
     */
    public abstract JTreeNodeEditor mirror();
}
