package jpel.gui.util;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Vector;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

public class JMenuGroup extends JMenu implements ActionListener {

    /**
     * Constante que usada para definir o menu sem KeyStroke.
     */
    public static final int NONE = -1;

    private transient Vector actionListeners;

    public JMenuGroup() {
        super();
    }

    public JMenuGroup( String name ) {
        super( name );
    }

    public JMenuGroup( String name, boolean option ) {
        super( name, option );
    }

    public JMenuGroup( Action action ) {
        super( action );
    }

    public void addGroup( String[] texts, ImageIcon[] icons, String[] commands,
        int[] keys ) {
        for( int i = 0; i < texts.length; i++ ) {
            addItem( texts[i], icons[i], commands[i], keys[i] );
        }
    }

    public void addItem( String text, ImageIcon icon, String command, int key ) {
        JMenuItem item = new JMenuItem( text, icon );
        item.setActionCommand( command );
        if( key != NONE ) {
            item.setAccelerator( KeyStroke.getKeyStroke( key,
                KeyEvent.CTRL_MASK, false ) );
        }
        item.addActionListener( this );
        add( item );
    }

    public void actionPerformed( ActionEvent e ) {
        fireActionPerformed( e );
    }

    public synchronized void removeActionListener( ActionListener l ) {
        if( actionListeners != null && actionListeners.contains( l ) ) {
            Vector v = ( Vector ) actionListeners.clone();
            v.removeElement( l );
            actionListeners = v;
        }
    }

    public synchronized void addActionListener( ActionListener l ) {
        Vector v = actionListeners == null ? new Vector( 2 )
                   : ( Vector ) actionListeners.clone();
        if( !v.contains( l ) ) {
            v.addElement( l );
            actionListeners = v;
        }
    }

    protected void fireActionPerformed( ActionEvent e ) {
        if( actionListeners != null ) {
            Vector listeners = actionListeners;
            int count = listeners.size();
            for( int i = 0; i < count; i++ ) {
                ( ( ActionListener ) listeners.elementAt( i ) ).actionPerformed(
                    e );
            }
        }
    }
}
