package jpel.gui.util;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.File;
import java.lang.reflect.Modifier;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.tree.DefaultMutableTreeNode;

import jpel.util.Debugger;

public class JTreeClassSelector extends JPanel implements JTreeSmartListener {

    JScrollPane jScrollPaneTree = new JScrollPane();
    DefaultMutableTreeNode root = new DefaultMutableTreeNode( "Classes" );
    JTreeSmart tree = new JTreeSmart( root );
    GridBagLayout gridBagLayout1 = new GridBagLayout();
    JButton jButtonOpenJar = new JButton();

    private boolean showPublic;
    private boolean showAbstract;
    private boolean showInterface;
    private boolean showFinal;
    private boolean showStatic;

    private static String CLASS = ".class";
    private Class reference;

    private String selectedClass;

    public JTreeClassSelector() {
        this( true, true, true, true, true, null );
    }

    public JTreeClassSelector(
        boolean showPublic, boolean showAbstract,
        boolean showInterface, boolean showFinal,
        boolean showStatic
        ) {
        this( showPublic, showAbstract, showInterface, showFinal, showStatic, null );
    }

    public JTreeClassSelector( Class reference ) {
        this( true, true, true, true, true, reference );
    }

    public JTreeClassSelector(
        boolean showPublic, boolean showAbstract,
        boolean showInterface, boolean showFinal,
        boolean showStatic, Class reference
        ) {
        super();
        setShowPublic( showPublic );
        setShowAbstract( showAbstract );
        setShowInterface( showInterface );
        setShowFinal( showFinal );
        setShowStatic( showStatic );
        setReference( reference );
        try {
            jbInit();
        }
        catch( Exception e ) {
            e.printStackTrace();
        }
        tree.addJTreeSmartListener( this );
    }

    public boolean isShowPublic() {
        return showPublic;
    }

    public void setShowPublic( boolean showPublic ) {
        this.showPublic = showPublic;
    }

    public void setShowAbstract( boolean showAbstract ) {
        this.showAbstract = showAbstract;
    }

    public boolean isShowAbstract() {
        return showAbstract;
    }

    public void setShowInterface( boolean showInterface ) {
        this.showInterface = showInterface;
    }

    public boolean isShowInterface() {
        return showInterface;
    }

    public void setShowFinal( boolean showFinal ) {
        this.showFinal = showFinal;
    }

    public boolean isShowFinal() {
        return showFinal;
    }

    public void setShowStatic( boolean showStatic ) {
        this.showStatic = showStatic;
    }

    public boolean isShowStatic() {
        return showStatic;
    }

    public void setReference( Class reference ) {
        this.reference = reference;
    }

    public Class getReference() {
        return reference;
    }

    private void jbInit() throws Exception {
        this.setLayout( gridBagLayout1 );
        this.setMinimumSize( new Dimension( 200, 300 ) );
        jButtonOpenJar.setIcon( Icons.OPEN );
        jButtonOpenJar.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                jButtonOpenJar_actionPerformed( e );
            }
        } );
        this.add( jScrollPaneTree, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0
            , GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
            0, 0, 0 ), 0, 0 ) );
        this.add( jButtonOpenJar, new GridBagConstraints( 1, 0, 1, 1, 0.0, 0.0
            , GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
            new Insets( 5, 5, 0, 5 ), 0, 0 ) );
        jScrollPaneTree.getViewport().add( tree, null );
    }

    void jButtonOpenJar_actionPerformed( ActionEvent e ) {
        openJar();
    }

    private void openJar() {
        try {
            JFileChooser jfc = Utils.getDefaultFileChooser( null, null );
            jfc.setCurrentDirectory( new java.io.File( "lib" ) );
            if( jfc.showOpenDialog( this ) == jfc.APPROVE_OPTION ) {
                File file = jfc.getSelectedFile();
                JarFile jf = new JarFile( file );
                Enumeration en = jf.entries();
                URLClassLoader loader = URLClassLoader.newInstance( new URL[] {
                                        file.toURL()} );
                root.removeAllChildren();
                while( en.hasMoreElements() ) {
                    JarEntry entry = ( JarEntry ) en.nextElement();
                    String name = entry.getName();
                    if( name.endsWith( CLASS ) ) {
                        String className = name.replace( '/', '.' );
                        className = className.replace( '\\', '.' );
                        className = className.substring( 0,
                                    className.length() - CLASS.length() );
                        try {
                            Class c = Class.forName( className, true, loader );
                            int modifier = c.getModifiers();
                            boolean show = true;
                            if( Modifier.isPublic( modifier ) ) {
                                show = show && isShowPublic();
                            }
                            if( Modifier.isAbstract( modifier ) ) {
                                show = show && isShowAbstract();
                            }
                            if( Modifier.isInterface( modifier ) ) {
                                show = show && isShowInterface();
                            }
                            if( Modifier.isFinal( modifier ) ) {
                                show = show && isShowFinal();
                            }
                            if( Modifier.isStatic( modifier ) ) {
                                show = show && isShowStatic();
                            }
                            if( reference != null ) {
                                show = show && reference.isAssignableFrom( c );
                            }
                            if( show ) {
                                //System.out.println("EXPRESSION::"+c.getName());
                                addClass( root, className );
                            }
                        }
                        catch( NoClassDefFoundError exc ) {
                            Debugger.println( exc );
                            Debugger.println(
                                "ClassSelector",
                                "load",
                                "Could not load class '" + className + "': "
                                + exc.getMessage()
                                );
                        }
                    }
                }
            }
        }
        catch( Exception exc ) {
            Debugger.println( exc );
        }
        tree.expandAll();
        tree.updateUI();
    }

    private void addClass( DefaultMutableTreeNode node, String name ) {
        int pos = name.indexOf( '.' );
        if( pos >= 0 ) {
            String strModule = name.substring( 0, pos );
            name = name.substring( pos + 1 );
            DefaultMutableTreeNode module = null;
            Enumeration enum = node.children();
            while( module == null && enum.hasMoreElements() ) {
                DefaultMutableTreeNode tmp = ( DefaultMutableTreeNode ) enum.
                                             nextElement();
                if( strModule.equals( String.valueOf( tmp.getUserObject() ) ) ) {
                    module = tmp;
                }
            }
            if( module != null ) {
                addClass( module, name );
            }
            else {
                module = new DefaultMutableTreeNode( strModule );
                node.add( module );
                addClass( module, name );
            }
        }
        else {
            node.add( new DefaultMutableTreeNode( name ) );
        }
    }

    public String getSelectedClass() {
        return selectedClass;
    }

    public void singleClick( JTreeSmartEvent e ) {
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) e.path.
                                      getLastPathComponent();
        if( !node.isRoot() ) {
            Object[] objs = node.getUserObjectPath();
            String str = "";
            for( int i = 1; i < objs.length; i++ ) {
                str += ( i == 1 ? "" : "." ) + objs[i];
            }
            selectedClass = str;
        }
        else {
            selectedClass = null;
        }
    }

    public void doubleClick( JTreeSmartEvent e ) {}
}
