package jpel.gui.util;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Enumeration;
import java.util.Vector;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

public class JTreeSmart extends JTree {

    private transient Vector JTreeSmartListeners;

    public JTreeSmart( TreeNode root ) {
        this( new DefaultTreeModel( root ) );
    }

    public JTreeSmart( DefaultTreeModel model ) {
        super( model );
        addMouseListener( new ClickListener() );
    }

    private class ClickListener extends MouseAdapter {
        public void mousePressed( MouseEvent e ) {
            int selRow = getRowForLocation( e.getX(), e.getY() );
            TreePath selPath = getPathForLocation( e.getX(), e.getY() );
            if( selRow != -1 ) {
                if( e.getClickCount() == 1 ) {
                    fireSingleClick( new JTreeSmartEvent( selRow, selPath ) );
                }
                else if( e.getClickCount() == 2 ) {
                    fireDoubleClick( new JTreeSmartEvent( selRow, selPath ) );
                }
            }
        }
    }

    public void expandCurrent() {
        TreePath path = getSelectionPath();
        if( path != null ) {
            command( path, true );
        }
    }

    public void expandAll() {
        TreeNode root = ( TreeNode ) getModel().getRoot();
        command( new TreePath( root ), true );
    }

    public void colapseCurrent() {
        TreePath path = getSelectionPath();
        if( path != null ) {
            command( path, false );
        }
    }

    public void colapseAll() {
        TreeNode root = ( TreeNode ) getModel().getRoot();
        command( new TreePath( root ), false );
    }

    private void command( TreePath parent, boolean expand ) {
        TreeNode node = ( TreeNode ) parent.getLastPathComponent();
        if( node.getChildCount() >= 0 ) {
            for( Enumeration e = node.children(); e.hasMoreElements(); ) {
                TreeNode n = ( TreeNode ) e.nextElement();
                TreePath path = parent.pathByAddingChild( n );
                command( path, expand );
            }
        }
        if( expand ) {
            expandPath( parent );
        }
        else {
            collapsePath( parent );
        }
    }

    public synchronized void removeJTreeSmartListener( JTreeSmartListener l ) {
        if( JTreeSmartListeners != null && JTreeSmartListeners.contains( l ) ) {
            Vector v = ( Vector ) JTreeSmartListeners.clone();
            v.removeElement( l );
            JTreeSmartListeners = v;
        }
    }

    public synchronized void addJTreeSmartListener( JTreeSmartListener l ) {
        Vector v = JTreeSmartListeners == null ? new Vector( 2 )
                   : ( Vector ) JTreeSmartListeners.clone();
        if( !v.contains( l ) ) {
            v.addElement( l );
            JTreeSmartListeners = v;
        }
    }

    protected void fireSingleClick( JTreeSmartEvent e ) {
        if( JTreeSmartListeners != null ) {
            Vector listeners = JTreeSmartListeners;
            int count = listeners.size();
            for( int i = 0; i < count; i++ ) {
                ( ( JTreeSmartListener ) listeners.elementAt( i ) ).singleClick(
                    e );
            }
        }
    }

    protected void fireDoubleClick( JTreeSmartEvent e ) {
        if( JTreeSmartListeners != null ) {
            Vector listeners = JTreeSmartListeners;
            int count = listeners.size();
            for( int i = 0; i < count; i++ ) {
                ( ( JTreeSmartListener ) listeners.elementAt( i ) ).doubleClick(
                    e );
            }
        }
    }
}
