package jpel.language;

import java.util.Iterator;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Define uma abstrao, isto , um procedimento.
 */
public class Abstraction extends AbstractExpression {
    private ExpressionList arguments;
    private Expression body;

    public Abstraction(ExpressionList arguments, Expression body) {
        super(ExpressionType.UNKNOWN);
        setArguments(arguments);
        setBody(body);
        //System.out.println("ABS:{"+arguments.toString("(",",",")")+"}"+body);
    }

    public ExpressionList getArguments() {
        return arguments;
    }

    public void setArguments(ExpressionList arguments) {
        this.arguments = arguments;
    }

    public Expression getBody() {
        return body;
    }

    public void setBody(Expression body) {
        this.body = body;
    }

    public boolean hasRelative() {
        boolean result = false;
        ExpressionList free = new ExpressionListImpl();
        body.freeVariable(free);
        if(!free.isEmpty()) {
            Iterator iter = free.iterator();
            while(!result && iter.hasNext()) {
                result = ((ExpressionId) iter.next()).isRelative();
            }
        }
        return result;
    }

    public boolean isTerminal() {
        return arguments.isEmpty();
    }

    public void freeVariable(ExpressionList list) {
        ExpressionList a = new ExpressionListImpl();
        arguments.freeVariable(a);
        ExpressionList b = new ExpressionListImpl();
        body.freeVariable(b);
        b.remove(a);
        b.freeVariable(list);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        ExpressionList newArgs = (ExpressionList) arguments.createClone();
        Iterator iter = novo.keys();
        while(iter.hasNext()) { // remove as variveis j resolvidas
            newArgs.remove((Expression) iter.next());
        }
        return new Abstraction(newArgs, body.rebuild(novo));
    }

    public Expression eval(Environment environment) throws ExecutionException {
        Expression result = null;
        if(isTerminal()) {
            result = body.eval(environment);
        }
        else {
            result = createClone();
        }
        return result;
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof Abstraction) {
            Abstraction e = (Abstraction) expression;
            result = arguments.equivalent(e.arguments);
            result = result && body.equivalent(e.body);
        }
        return result;
    }

    public String toString() {
        String result = "";
        if(arguments != null && arguments.length() > 0) {
            result = arguments.toString("(", ",", ")") + ":";
        }
        return result + body;
    }
}