package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Representa operacoes que possuem dois operandos.
 */
public abstract class BinaryExpression extends AbstractNamedExpression {

    private Expression leftExpression;
    private Expression rightExpression;

    public BinaryExpression(String binaryOperator, ExpressionType type,
        Expression leftExpression, Expression rightExpression) {
        super(type, binaryOperator);
        setLeftExpression(leftExpression);
        setRightExpression(rightExpression);
    }

    public Expression getLeftExpression() {
        return leftExpression;
    }

    public void setLeftExpression(Expression newLeftExpression) {
        leftExpression = newLeftExpression;
    }

    public void setRightExpression(Expression newRightExpression) {
        rightExpression = newRightExpression;
    }

    public Expression getRightExpression() {
        return rightExpression;
    }

    public void freeVariable(ExpressionList list) {
        getLeftExpression().freeVariable(list);
        getRightExpression().freeVariable(list);
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof BinaryExpression) {
            BinaryExpression e = (BinaryExpression) expression;
            result = getName().equals(e.getName());
            result = result
                     && getLeftExpression().equivalent(e.getLeftExpression())
                     && getRightExpression().equivalent(e.getRightExpression());
        }
        return result;
    }

    public String toString() {
        return "(" + getLeftExpression() + ")" + getName() + "("
            + getRightExpression() + ")";
    }
}
