package jpel.language;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Indica que toda declarao deve ter um nome no formato ExpressionId.
 */
public interface Declaration {

    /**
     * Ajusta uma descricao p/a declarao.
     * @param description A descrio da declarao.
     */
    void setDescription(String description);

    /**
     * Retorna a descrio associada a declarao.
     * @return A descrio da declarao.
     */
    String getDescription();

    /**
     * Ajusta o tipo da declarao.
     * @param type O tipo.
     */
    void setType(DeclarationType type);

    /**
     * Retorna o tipo da declarao.
     * @return O tipo.
     */
    DeclarationType getType();

    /**
     * Ajusta o mdulo ao qual pertence a declarao.
     * @param module Id do mdulo ao qual a declarao pertence.
     */
    void setModule(ExpressionId module);

    /**
     * Retorna o mdulo ao qual uma declarao est associada.
     * @return O id do mdulo cuja declarao  parte.
     */
    ExpressionId getModule();

    /**
     * Ajusta o nome da declarao.
     * @param name Nome da declarao.
     */
    void setName(ExpressionId name);

    /**
     * Toda declarao tem um nome, este mtodo retorna o identificador da declarao.
     * @return Identificador.
     */
    ExpressionId getName();

    /**
     * Executa a declarao no ambiente fornecido.
     * @param environment O ambiente onde a declarao deve ser executada.
     */
    void execute(Environment environment) throws DeclarationException;

    /**
     * Reconstroi uma declaraao usando um mapeamento.
     * @param map O mapeamento da subsitituiao.
     * @return Uma declarao com as variveis substitudas.
     */
    Declaration rebuild(MapReplace map);
}
