package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Classe que permite a cirao de mdulos dentro da linguagem.
 */

public class DeclarationModule extends AbstractDeclaration {

    private Environment subEnvironment;

    public DeclarationModule(ExpressionId module, ExpressionId name, Environment subEnvironment) {
        super(DeclarationType.MODULE, module, name);
        setSubEnvironment(subEnvironment);
    }

    public void setSubEnvironment(Environment subEnvironment) {
        this.subEnvironment = subEnvironment;
    }

    public Environment getSubEnvironment() {
        return subEnvironment;
    }

    public void execute(Environment environment) throws DeclarationException {
        if(getModule() != null) {
            try {
                environment = (Environment) environment.lookupSub(
                              getModule());
            }
            catch(IdNotFoundException ex) {
                throw new DeclarationException(this, ex.getMessage(), ex);
            }
        }
        if(getSubEnvironment() != null) {
            try {
                environment.lookupSub(getName());
            }
            catch(IdNotFoundException ex) {
                // cria de denovo se no j estiver criado.
                environment.bind(getName(), getSubEnvironment());
            }
        }
    }

    public Declaration rebuild(MapReplace map) {
        Declaration dec = new DeclarationModule(getModule(), getName(),
                          getSubEnvironment().createClone());
        dec.setDescription(getDescription());
        return dec;
    }

    public String toString() {
        return "Module(" + super.toString() + ":" + subEnvironment + ")";
    }
}
