package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Iterator;

/**
 * Classe que permite a adio de funes nativas ao ambiente de execuo.
 */
public class DeclarationNative extends DeclarationFunction {

    public DeclarationNative(ExpressionId module, ExpressionId name,
        Abstraction abstraction) throws DeclarationException {
        super(DeclarationType.NATIVE, module, name, abstraction);
        if((abstraction.getBody() != null)
            && !(abstraction.getBody()instanceof ExpressionId)) {
            throw new DeclarationException(this,
                "The abstraction body must be a ExpressionId with implementation classe name.");
        }
    }

    public void execute(Environment env) throws DeclarationException {
        try {
            ExpressionList arguments = getAbstraction().getArguments();
            Expression bodyId = getAbstraction().getBody();
            String strClass = bodyId.toString();
            Class body = Class.forName(strClass);
            Class[] types = new Class[(int) arguments.length()];
            for(int i = 0; i < types.length; i++) {
                types[i] = Expression.class;
            }
            Class[] idTypes = new Class[types.length + 1];
            idTypes[0] = String.class;
            System.arraycopy(types, 0, idTypes, 1, types.length);
            Constructor constructor = null;
            boolean hasId = false;
            try {
                constructor = body.getConstructor(idTypes);
                hasId = true;
            }
            catch(NoSuchMethodException exc) {
                constructor = body.getConstructor(types);
            }
            Object[] parameters = null;
            if(hasId) {
                parameters = new Object[idTypes.length];
                parameters[0] = getName().getName();
            }
            else {
                parameters = new Object[types.length];
            }
            int index = (hasId ? 1 : 0);
            Iterator nav = arguments.iterator();
            while(nav.hasNext()) {
                parameters[index++] = nav.next();
            }
            DeclarationFunction dec = new DeclarationFunction(
                                      getModule(),
                                      getName(),
                                      new Abstraction(arguments,
                                      (Expression) constructor.
                                      newInstance(parameters)));
            dec.execute(env);
        }
        catch(ClassNotFoundException exc) {
            throw new DeclarationException(this, exc.getMessage(), exc);
        }
        catch(NoSuchMethodException exc) {
            throw new DeclarationException(this, exc.getMessage(), exc);
        }
        catch(InvocationTargetException exc) {
            throw new DeclarationException(this, exc.getMessage(), exc);
        }
        catch(IllegalAccessException exc) {
            throw new DeclarationException(this, exc.getMessage(), exc);
        }
        catch(InstantiationException exc) {
            throw new DeclarationException(this, exc.getMessage(), exc);
        }
    }

    public Declaration rebuild(MapReplace map) {
        return this;
    }

    public String toString() {
        return getType().toString() + "(" + super.toString() + ")";
    }
}
