package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.Serializable;

/**
 * Classe usada para fornecer a linguagem a noo de tipos. Por enquanto,
 * a validao de tipo apenas est presente na avaliao de expresses.
 */
public class DeclarationType implements Serializable {

    // realy unknown
    private static final int UNKNOWN_CODE = 0;
    // basic types
    private static final int FUNCTION_CODE = UNKNOWN_CODE + 1;
    private static final int NATIVE_CODE = FUNCTION_CODE + 1;
    private static final int MODULE_CODE = NATIVE_CODE + 1;

    /**
     * representa um tipo ainda no conhecido. Usado nas expresses
     * ifThenElse e Switch, onde o resultado da operao no tem tipo
     * conhecido.
     */
    public static final DeclarationType UNKNOWN = new DeclarationType(
                                                  UNKNOWN_CODE);
    /**
     * Representa o tipo funo.
     */
    public static final DeclarationType FUNCTION = new DeclarationType(
        FUNCTION_CODE);
    /**
     * Representa o tipo nativo.
     */
    public static final DeclarationType NATIVE = new DeclarationType(
                                                 NATIVE_CODE);
    /**
     * Representa o tipo mdulo.
     */
    public static final DeclarationType MODULE = new DeclarationType(
                                                 MODULE_CODE);
    private int type;

    private DeclarationType(int type) {
        this.type = type;
    }

    public int getType() {
        return type;
    }

    public boolean isUnknown() {
        return check(UNKNOWN_CODE);
    }

    public boolean isFunction() {
        return check(FUNCTION_CODE);
    }

    public boolean isNative() {
        return check(NATIVE_CODE);
    }

    public boolean isModule() {
        return check(MODULE_CODE);
    }

    private boolean check(int t) {
        return type == t;
    }

    public boolean equivalent(Object obj) {
        boolean result = false;
        if(obj instanceof DeclarationType) {
            result = (type == ((DeclarationType) obj).type);
        }
        return result;
    }

    public String toString() {
        String result = "Not found!";
        switch(getType()) {
            case UNKNOWN_CODE:
                result = "Unkown";
                break;
            case FUNCTION_CODE:
                result = "Function";
                break;
            case NATIVE_CODE:
                result = "Native";
                break;
            case MODULE_CODE:
                result = "Module";
                break;
        }
        return result;
    }
}
