package jpel.language;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

public interface Environment {

    /**
     * Cria uma relao entre um identificador e sua abstrao.
     * @param id Nome da abstrao.
     * @param abstraction Abstrao.
     */
    void bind(ExpressionId id, Abstraction abstraction);

    /**
     * Cria uma relao entre um identificador e seu submdulo.
     * @param id Nome da abstrao.
     * @param subEnvironment O sub-ambiente.
     */
    void bind(ExpressionId id, Environment subEnvironment);

    /**
     * Retorna a abstrao associada ao identificador.
     * @param id Identificador da abstrao.
     * @return A abstrao associada.
     * @throws IdNotFoundException Quando a abstrao com o nome dado no 
     * encontrado.
     */
    Abstraction lookup(ExpressionId id) throws IdNotFoundException;

    /**
     * Retorna o sub-ambiente associado ao identificador.
     * @param id Identificador da abstrao.
     * @return O corpo da abstrao associada.
     * @throws IdNotFoundException Quando a abstrao com o nome dado no 
     * encontrado.
     */
    Environment lookupSub(ExpressionId id) throws IdNotFoundException;

    /**
         * Retorna uma listagem de todos os identificadores disponveis no ambiente.
     * @return Um iterator sobre todos o identificadores do ambiente.
     */
    Iterator keys();

    /**
     * Incrementa o contexto do ambiente de execuo, criando um novo elemento
     * na pilha de bindings.
     */
    void addContext();

    /**
     * Decrementa a pilha de execuo.
     */
    void removeContext();

    /**
     * Resolve substitui todas os identificadores relativos aos mdulos ("."Id)
     * prefixando o nome do mdulo ao identificador.
     */
    void resolveRelatives();

    /**
     * Retorna uma cpia do ambiente.
     * @return A cpia.
     */
    Environment createClone();
}