package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import jpel.util.Debugger;

/**
 * Classe utilitria para a criao de dinmica dos elementos da linguagem.
 */
public class EnvironmentBuilder {

    private static final String JVM_PARAMETER = "environmentFactory";
    private static final String COMMENT = "#";
    private static final String TYPE_MARK = "::";
    private static final String TYPE_TOKEN = ",";

    private static HashMap loaders;

    static {
        loaders = new HashMap();
        try {
            InputStream in;
            BufferedReader bin;
            String input;

            in = EnvironmentBuilder.class.getResourceAsStream("loaders.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindEnvironmentLoader(types, str);
                }
                else {
                    bindEnvironmentLoader(str);
                }
            }
            bin.close();
            in.close();
        }
        catch(Exception exc) {
            exc.printStackTrace();
            System.exit(0);
        }
    }

    public static EnvironmentFactory getEnvironmentFactory() throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        String classType = System.getProperty(JVM_PARAMETER,
                           "jpel.language.EnvironmentFactoryImpl");
        return getEnvironmentFactory(classType);
    }

    public static EnvironmentFactory getEnvironmentFactory(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        EnvironmentFactory factory = (EnvironmentFactory) Class.forName(
                                     classType).newInstance();
        Debugger.println("EnvironmentBuilder", "factory",
            "Class>" + factory.getClass().getName());
        return factory;
    }

    public static void bindEnvironmentLoader(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        EnvironmentLoader loader = (EnvironmentLoader) Class.forName(
                                   classType).newInstance();
        bindEnvironmentLoader(loader);
    }

    public static void bindEnvironmentLoader(EnvironmentLoader loader) {
        bindEnvironmentLoader(loader.getTypes(), loader);
    }

    public static void bindEnvironmentLoader(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        EnvironmentLoader loader = (EnvironmentLoader) Class.forName(
                                   classType).newInstance();
        bindEnvironmentLoader(types, loader);
    }

    public static void bindEnvironmentLoader(String[] types,
        EnvironmentLoader loader) {
        for(int i = 0; i < types.length; i++) {
            loaders.put(types[i], loader);
            Debugger.println("EnvironmentBuilder", "bindLoader",
                "Type> " + types[i] + ", Class>" + loader.getClass().getName());
        }
    }

    public static EnvironmentLoader lookupEnvironmentLoader(String type) throws
        ClassNotFoundException {
        EnvironmentLoader loader = (EnvironmentLoader) loaders.get(type);
        if(loader == null) {
            throw new ClassNotFoundException(
                "There is not a Loader bound to '" + type + "'.");
        }
        Debugger.println("EnvironmentBuilder", "lookupLoader",
            "Type> " + type + ", Class>" + loader.getClass().getName());
        return loader;
    }

    public static Iterator loaderTypes() {
        return loaders.keySet().iterator();
    }

    public static void main(String[] args) {
        try {
            Object obj = EnvironmentBuilder.getEnvironmentFactory();
            if(obj != null) {
                System.out.println("Factory is a instance of '"
                    + obj.getClass().getName() + "'.");
            }
            else {
                System.out.println("Factory is null.");
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
