package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;
import java.util.Properties;

import jpel.util.Debugger;

/**
 * Implementao padro da fbrica de ambientes.
 */
public class EnvironmentFactoryImpl implements EnvironmentFactory {

    private static EnvironmentFactory factory;
    private static Environment lang;

    public static final String COMMENT = "//";
    public static final String FORMAL = "::";
    public static final String BODY = "##";

    private ExpressionParser parser;

    static {
        factory = new EnvironmentFactoryImpl();
        try {
            lang = factory.produce();
        }
        catch(EnvironmentFactoryException ex) {
            ex.printStackTrace();
        }
    }

    public EnvironmentFactoryImpl() {}

    public EnvironmentFactoryImpl(ExpressionParser parser) {
        this.parser = parser;
    }

    public ExpressionParser getParser() {
        return parser;
    }

    public void setParser(ExpressionParser parser) {
        this.parser = parser;
    }

    public boolean isLang(ExpressionId id) {
        boolean result = false;
        try {
            lang.lookup(id);
            result = true;
        }
        catch(IdNotFoundException ex) {}
        return result;
    }

    public Environment empty() throws EnvironmentFactoryException {
        return new EnvironmentImpl();
    }

    public Environment produce() throws EnvironmentFactoryException {
        try {
            Environment env = empty();
            populate(env);
            populateSystem(env);
            return env;
        }
        catch(DeclarationException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
    }

    protected void populate(Environment env) throws DeclarationException,
        EnvironmentFactoryException {
        InputStream in = null;
        InputStreamReader inReader = null;
        BufferedReader reader = null;

        DeclarationNative dec = null;
        ExpressionId module = null;
        ExpressionId name = null;
        ExpressionList formals = null;
        ExpressionId body = null;
        try {
            if(parser == null) {
                parser = ExpressionBuilder.getExpressionParser();
            }
            in = getClass().getResourceAsStream("lang.lib");
            inReader = new InputStreamReader(in);
            reader = new BufferedReader(inReader);
            int posFormals, posBody;
            String input, strName, strFormals, strBody;
            while((input = reader.readLine()) != null) {
                if(input.startsWith(COMMENT)) {
                    continue;
                }
                if("".equals(input = input.trim())) {
                    continue;
                }
                posFormals = input.indexOf(FORMAL);
                posBody = input.indexOf(BODY, posFormals + FORMAL.length());
                strName = input.substring(0, posFormals).trim();
                strFormals = input.substring(posFormals + FORMAL.length() + 1,
                             posBody).trim();
                strBody = input.substring(posBody + BODY.length() + 1).trim();
                Debugger.println("EFI", "preload",
                    "Name='" + strName + "', Formals=" + strFormals + ", Body="
                    + strBody);
                name = (ExpressionId) parser.getExpression(strName);
                formals = (ExpressionList) parser.getExpression(strFormals);
                body = (ExpressionId) parser.getExpression(strBody);
                Abstraction abstraction = new Abstraction(formals, body);
                dec = new DeclarationNative(module, name, abstraction);
                name.setLang(true);
                dec.execute(env);
                try {
                    Debugger.println("EFI", "loaded",
                        "Loaded '" + name + "' -> " + env.lookup(name));
                }
                catch(Exception e) {}
            }
        }
        catch(ClassNotFoundException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        catch(IllegalAccessException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        catch(InstantiationException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        catch(IOException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        catch(ExpressionParserException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        catch(DeclarationException exc) {
            throw new EnvironmentFactoryException(exc.getMessage(), exc);
        }
        finally {
            if(reader != null) {
                try {
                    reader.close();
                }
                catch(IOException exc) {}
            }
            if(inReader != null) {
                try {
                    inReader.close();
                }
                catch(IOException exc) {}
            }
            if(in != null) {
                try {
                    in.close();
                }
                catch(IOException exc) {}
            }
        }
    }

    protected void populateSystem(Environment env) throws DeclarationException,
        EnvironmentFactoryException {
        Properties prop = System.getProperties();
        Enumeration enum = prop.propertyNames();

        ExpressionId module;
        ExpressionId name;
        ExpressionString value;
        while(enum.hasMoreElements()) {
            String tmp = (String) enum.nextElement();

            name = new ExpressionId(tmp);
            value = new ExpressionString(prop.getProperty(tmp));

            String str = name.getName();
            int pos = str.indexOf(ExpressionId.SEPARATOR);
            while(pos >= 0) {
                String sub = str.substring(0, pos);
                ExpressionId subId = new ExpressionId(sub);
                Environment subEnv = null;
                boolean create = false;
                try {
                    subEnv = env.lookupSub(subId);
                    create = (subEnv == null);
                }
                catch(IdNotFoundException ex) {
                    create = true;
                }
                if(create) {
                    DeclarationModule dec = new DeclarationModule(
                                            subId.getModule(),
                                            subId.getLocal(),
                                            empty()
                                            );
                    dec.getName().setJvm(true);
                    dec.execute(env);
                }
                pos = str.indexOf(ExpressionId.SEPARATOR, pos + 1);
            }
            Abstraction abs = new Abstraction(new ExpressionListImpl(), value);
            DeclarationFunction dec = new DeclarationFunction(name.getModule(),
                                      name.getLocal(), abs);
            dec.getName().setJvm(true);
            dec.execute(env);
            Debugger.println("EFI", "system", "Loaded " + name + "=" + value);
        }
    }
}
