package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Representa o tipo booleano da linguagem.
 */
public class ExpressionBoolean extends ExpressionObject implements
    StringableExpression, ComparableExpression {

    /**
     * Constante que representa o booleano "verdadeiro".
     */
    public static final ExpressionBoolean TRUE = new
                                                 ExpressionBoolean(Boolean.TRUE);
    /**
     * Constante que representa o booleano "falso".
     */
    public static final ExpressionBoolean FALSE = new
                                                  ExpressionBoolean(Boolean.
                                                  FALSE);

    private ExpressionBoolean(Boolean value) {
        super(ExpressionType.BOOLEAN, value);
    }

    public static ExpressionBoolean getBoolean(boolean bool) {
        ExpressionBoolean result;
        if(bool) {
            result = TRUE;
        }
        else {
            result = FALSE;
        }
        return result;
    }

    public boolean getBoolean() {
        return((Boolean) getData()).booleanValue();
    }

    public String asString() {
        return String.valueOf(getData());
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        if(!expression.getType().isBoolean()) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression);
        }
        ExpressionBoolean exp = (ExpressionBoolean) expression;
        int result = 0;
        if(getBoolean() == exp.getBoolean()) {
            result = 0;
        }
        else if(getBoolean() && !exp.getBoolean()) {
            result = 1;
        }
        else {
            result = -1;
        }
        return result;
    }

    public String toString() {
        return asString();
    }
}
