package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Possibilita a passagem de parametros para a execuo de uma funo.
 */
public class ExpressionCall extends AbstractExpression {

    private ExpressionList arguments;
    private Expression expression;

    public ExpressionCall(Expression expression, ExpressionList arguments) {
        super(ExpressionType.UNKNOWN);
        setExpression(expression);
        setArguments(arguments);
        //System.out.println("CALL:{"+expression+"}"+arguments.toString("(",",",")"));
    }

    public void setExpression(Expression expression) {
        this.expression = expression;
    }

    public Expression getExpression() {
        return expression;
    }

    public void setArguments(ExpressionList newArguments) {
        arguments = newArguments;
    }

    public ExpressionList getArguments() {
        return arguments;
    }

    public void freeVariable(ExpressionList list) {
        expression.freeVariable(list);
        arguments.freeVariable(list);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionCall(expression.rebuild(novo),
            (ExpressionList) arguments.rebuild(novo));
    }

    public Expression eval(Environment environment) throws ExecutionException {
        try {
            environment.addContext();
            Expression result = null;

            Abstraction abs = null;
            if(expression instanceof ExpressionId) {
                abs = environment.lookup((ExpressionId) expression);
            }
            else if(expression instanceof Abstraction) {
                abs = (Abstraction) expression;
            }
            else {
                result = expression.eval(environment);
                if(result instanceof Abstraction) {
                    abs = (Abstraction) result;
                }
            }
            if(abs != null) {
                abs = elabore(arguments, abs);
                result = abs.eval(environment);
            }

            return result;
        }
        finally {
            environment.removeContext();
        }
    }

    private Abstraction elabore(ExpressionList arguments,
        Abstraction abs) throws ExecutionException {
        Iterator names = abs.getArguments().iterator();
        Iterator values = arguments.iterator();
        MapReplace replace = new MapReplaceImpl();
        while(values.hasNext()) {
            ExpressionId name = (ExpressionId) names.next();
            Expression value = (Expression) values.next();
            replace.put(name, value);
        }
        return(Abstraction) abs.rebuild(replace);
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionCall) {
            ExpressionCall e = (ExpressionCall) expression;
            result = expression.equivalent(e.expression);
            result = result && arguments.equivalent(e.arguments);
        }
        return result;
    }

    public String toString() {
        if(getSource() != null) {
            return String.valueOf(getSource());
        }
        String str = expression + " (";
        Iterator i = arguments.iterator();
        int index = 0;
        while(i.hasNext()) {
            str += (index == 0 ? i.next() : "," + i.next());
            index++;
        }
        return str + ")";
    }
}
