package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Expresso que representa um identificador.
 */
public class ExpressionId extends AbstractNamedExpression implements
    StringableExpression, ComparableExpression {

    /**
     * Separador usado para definio hierarquica de identificadores. Case se deseje fazer
     * uma referncia a uma varivel do mesmo pacote usa-se este identificador no comeo. Ex:
     * .MAX
     */
    public static final String SEPARATOR = ".";

    /**
         * Indica se o identificador se refere a um componente default da linguagem.
     */
    private boolean lang = false;

    /**
     * Indica se o identificador se refere a um parametros presente na JVM.
     */
    private boolean jvm = false;

    public ExpressionId(String id) {
        super(ExpressionType.ID, id);
    }

    public void freeVariable(ExpressionList list) {
        if(!list.contains(this)) {
            list.append(this);
        }
    }

    public Expression rebuild(MapReplace map) {
        Expression e = map.get(this);
        return e != null ? e : this;
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Abstraction abs = environment.lookup(this);
        return abs.eval(environment);
    }

    public boolean equivalent(Expression expression) {
        return getName().equals(((ExpressionId) expression).getName());
    }

    public Expression createClone() {
        return new ExpressionId(getName());
    }

    public String asString() {
        return toString();
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        if(!(expression instanceof ExpressionId)) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression);
        }
        return getName().compareTo(((ExpressionId) expression).getName());
    }

    public boolean hasModule() {
        return getName().indexOf(SEPARATOR) >= 0;
    }

    public ExpressionId getModule() {
        ExpressionId result = null;
        if(hasModule()) {
            String str = getName();
            result = new ExpressionId(str.substring(0,
                     str.lastIndexOf(SEPARATOR)));
        }
        return result;
    }

    public ExpressionId getLocal() {
        ExpressionId result = null;
        if(hasModule()) {
            String str = getName();
            result = new ExpressionId(str.substring(str.lastIndexOf(
                     SEPARATOR) + 1));
        }
        else {
            result = this;
        }
        return result;
    }

    public boolean isRelative() {
        return getName().startsWith(SEPARATOR);
    }

    public void setLang(boolean lang) {
        this.lang = lang;
    }

    public boolean isLang() {
        return lang;
    }

    public void setJvm(boolean jvm) {
        this.jvm = jvm;
    }

    public boolean isJvm() {
        return jvm;
    }

    public boolean equals(Object obj) {
        boolean result = false;
        if(obj instanceof ExpressionId) {
            result = equivalent((ExpressionId) obj);
        }
        return result;
    }

    public int hashCode() {
        return getName().hashCode();
    }

    public String toString() {
        return getName() + (getData() != null ? ":" + String.valueOf(getData())
            : "");
    }

    public static void main(String[] args) {
        try {
            MapReplace map = new MapReplaceImpl();
            map.put(new ExpressionId("x"), new ExpressionId("y"));
            ExpressionId id = new ExpressionId("x");
            System.out.println("rebuild(" + id + ")=" + id.rebuild(map));
            System.out.println("old=" + id);
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
