package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Representa o operador condicional da linguagem.
 * <p>
 * Syntax:
 * <br> if(condition) then thenExpression1 else elseExpression
 * <br> Exemplo:
 * <br> if ( x < 0 ) then "Numero negativo!" else "Numero positivo!".
 */
public class ExpressionIfThenElse extends AbstractExpression {

    private Expression condition;
    private Expression thenExpression;
    private Expression elseExpression;

    public ExpressionIfThenElse(Expression condition,
        Expression thenExpression, Expression elseExpression) {
        super(ExpressionType.UNKNOWN);
        setCondition(condition);
        setThenExpression(thenExpression);
        setElseExpression(elseExpression);
    }

    public Expression getCondition() {
        return condition;
    }

    public void setCondition(Expression newCondition) {
        condition = newCondition;
    }

    public void setThenExpression(Expression newThenExpression) {
        thenExpression = newThenExpression;
    }

    public Expression getThenExpression() {
        return thenExpression;
    }

    public void setElseExpression(Expression newElseExpression) {
        elseExpression = newElseExpression;
    }

    public Expression getElseExpression() {
        return elseExpression;
    }

    public void freeVariable(ExpressionList list) {
        getCondition().freeVariable(list);
        getThenExpression().freeVariable(list);
        getElseExpression().freeVariable(list);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionIfThenElse(getCondition().rebuild(novo),
            getThenExpression().rebuild(novo),
            getElseExpression().rebuild(novo));
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        if(((ExpressionBoolean) getCondition().eval(environment)).
            getBoolean()) {
            return getThenExpression().eval(environment);
        }
        else {
            return getElseExpression().eval(environment);
        }
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionIfThenElse) {
            ExpressionIfThenElse e = (ExpressionIfThenElse) expression;
            result = condition.equivalent(e.condition)
                     && thenExpression.equivalent(e.thenExpression)
                     && elseExpression.equivalent(e.elseExpression);
        }
        return result;
    }

    public String toString() {
        if(getSource() != null) {
            return String.valueOf(getSource());
        }
        return "if " + getCondition() + " then " + getThenExpression()
            + " else " + getElseExpression();
    }
}
