package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
     * Incorpora o tipo List  linguagem funcional, definindo as operaes permitidas
 * sobre este tipo de dados.
 */
public interface ExpressionList extends Expression {

    /**
     * Limpa a lista de expressoes.
     * @return A lista de expressoes modificada.
     */
    ExpressionList clear();

    /**
     * Adiciona uma expresso  lista.
     * @param expression Expresso a ser adicionada.
     * @return A lista modificada.
     */
    ExpressionList append(Expression expression);

    /**
     * Adiciona uma lista de expresses  lista.
     * @param expressionList Expresses a serem adicionadas.
     * @return A lista modificada.
     */
    ExpressionList append(ExpressionList expressionList);

    /**
     * Remove uma expresso da lista.
     * @param expression Expresso a ser removida.
     * @return A lista modificada.
     */
    ExpressionList remove(Expression expression);

    /**
     * Remove uma lista de expresses da lista.
     * @param expressionList Expresses a serem removidas.
     * @return A lista modificada.
     */
    ExpressionList remove(ExpressionList expressionList);

    /**
     * Indica se uma expresso  parte constituinte da lista.
     * @param expression Expresso que se deseja procurar.
         * @return true, caso a expresso faa parte da lista, false, caso contrrio.
     */
    boolean contains(Expression expression);

    /**
     * Indica se lista de expresses  parte constituinte da lista.
     * @param expressionList Expresses que se deseja procurar.
         * @return true, caso as expresses faam parte da lista, false, caso contrrio.
     */
    boolean contains(ExpressionList expressionList);

    /**
     * Indica se a lista est vazia.
     * @return true, caso a lsita na possua elementos, false caso contrrio.
     */
    boolean isEmpty();

    /**
     * Indica o tamanho da lista, isto , quantas expresses esto presentes na lista.
     * @return Um long indicando a quantidade de elementos.
     */
    long length();

    /**
     * Retorna o elemento cabea da lista.
     * @return Caso a lista no esteja vazia, o primeiro elemento, caso contrrio, null.
     */
    Expression head();

    /**
     * Retorna a cauda da lista.
     * @return Caso a lista possua cauda ela  retornada, caso contrrio, null.
     */
    ExpressionList tail();

    /**
     * Fornece um mecanismo de varrer os elementos de uma lista.
     * @return Um navigator sobre a lista.
     */
    Iterator iterator();

    /**
         * Retorna a representao String da lista com um determinado prefixo e sufixo.
         * <br>O default  prefix = '[' e suffix = ']', caso de deseje delimitara lista
     * com outros caracteres utiliza-se este mtodo.
     * @param prefix Prefixo a ser aplicado a lista de expresses separadas por
     * separator.
     * @param separator Separador utilizado para representar a lista.
     * @param suffix Sufixo a ser aplicato a lista de expresses separadas por
     * vrgulas.
         * @return A representao String da lista com o prefixo e sufixo fornecido.
     */
    String toString(String prefix, String separator, String suffix);
}