package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Implementao padro da lista de expresses.
 */
public class ExpressionListImpl extends AbstractExpression implements
    ExpressionList, ComparableExpression {

    private Expression head;
    private ExpressionListImpl tail;

    public ExpressionListImpl() {
        this(ExpressionType.LIST);
    }

    public ExpressionListImpl(ExpressionType type) {
        super(type);
        clear();
    }

    public Object getData(Class type) {
        Object[] result = new Object[(int) length()];
        ExpressionList aux = this;
        for(int i = 0; aux.head() != null; i++) {
            result[i] = (aux.head()).getData();
            aux = aux.tail();
        }
        return result;
    }

    public ExpressionList clear() {
        head = null;
        tail = null;
        return this;
    }

    public ExpressionList append(Expression expression) {
        if(head == null) {
            head = expression;
            tail = new ExpressionListImpl();
        }
        else {
            tail.append(expression);
        }
        return this;
    }

    public ExpressionList append(ExpressionList expressionList) {
        Iterator i = expressionList.iterator();
        while(i.hasNext()) {
            append((Expression) i.next());
        }
        return this;
    }

    public ExpressionList remove(Expression expression) {
        if(head != null) {
            if(head.equivalent(expression)) {
                head = tail.head;
                tail = tail.tail;
            }
            else {
                tail.remove(expression);
            }
        }
        return this;
    }

    public ExpressionList remove(ExpressionList expressionList) {
        Iterator i = expressionList.iterator();
        while(i.hasNext()) {
            remove((Expression) i.next());
        }
        return this;
    }

    public boolean contains(Expression expression) {
        boolean result = false;
        if(head != null) {
            if(head.equivalent(expression)) {
                result = true;
            }
            else {
                result = tail.contains(expression);
            }
        }
        return result;
    }

    public boolean contains(ExpressionList expressionList) {
        boolean result = true;
        Iterator i = expressionList.iterator();
        while(result && i.hasNext()) {
            result = contains((Expression) i.next());
        }
        return result;
    }

    public long length() {
        long result = 0;
        if(!isEmpty()) {
            result = 1 + tail.length();
        }
        return result;
    }

    public boolean isEmpty() {
        return head == null;
    }

    public Expression head() {
        return head;
    }

    public ExpressionList tail() {
        return tail;
    }

    public Iterator iterator() {
        return new ExpressionListIterator(this);
    }

    public void freeVariable(ExpressionList list) {
        if(!isEmpty()) {
            head.freeVariable(list);
            tail.freeVariable(list);
        }
    }

    public Expression rebuild(MapReplace map) {
        ExpressionList result = new ExpressionListImpl();
        if(!isEmpty()) {
            MapReplace novo = map.mirror();
            result.append(head.rebuild(novo));
            result.append((ExpressionList) tail.rebuild(novo));
        }
        return result;
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        ExpressionList result = new ExpressionListImpl();
        if(!isEmpty()) {
            result.append(head.eval(environment));
            result.append((ExpressionList) tail.eval(environment));
        }
        return result;
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionList) {
            ExpressionList el = (ExpressionList) expression;
            if(!isEmpty() && !el.isEmpty()) { // listas no vazias
                result = head.equivalent(el.head())
                         && tail.equivalent(el.tail());
            }
            else { // as duas lista so vazias
                result = (head == el.head()) && (tail == el.tail());
            }
        }
        // se as lista tem tamanhos distintos deve retornar false
        return result;
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        int result = 0;
        if(!(expression instanceof ExpressionList)) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression
                + ". Only lists can be compared to lists.");
        }
        ExpressionList el = (ExpressionList) expression;
        if(isEmpty() && !el.isEmpty()) { // listas no vazias
            result = -1;
        }
        else if(!isEmpty() && el.isEmpty()) {
            result = 1;
        }
        else if(!isEmpty() && !el.isEmpty()) {
            result = ((ComparableExpression) head).compareTo((
                     ComparableExpression) el.head());
            if(result == 0) {
                result = tail.compareTo((ComparableExpression) el.tail());
            }
        }
        //se as listas so vazias ento o resultado e 0
        return result;
    }

    public String toString() {
        return toString("[", ",", "]");
    }

    public String toString(String prefix, String separator, String suffix) {
        String result = prefix;
        Iterator i = iterator();
        int index = 0;
        while(i.hasNext()) {
            result += (index == 0 ? String.valueOf(i.next())
                : separator + String.valueOf(i.next()));
            index++;
        }
        return result + suffix;
    }

    public static void main(String[] args) {
        try {
            ExpressionList li = new ExpressionListImpl();
            for(int i = 0; i < args.length * 10; i++) {
                //li.append( new ExpressionString( args[i] ) );
                li.append(ExpressionBoolean.getBoolean(i % 2 == 0));
            }
            System.out.println("LIST>" + li);
            System.out.println("First");
            Iterator e = li.iterator();
            while(e.hasNext()) {
                System.out.println(e.next());
            }
            System.out.println("Second");
            e = li.iterator();
            while(e.hasNext()) {
                System.out.println(e.next());
            }
            System.out.println("Third");
            e = li.iterator();
            while(e.hasNext()) {
                while(e.hasNext()) {
                    System.out.println(e.next());
                }
                System.out.println("+---+");
                li.remove(li.head());
                e = li.iterator();
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
