package jpel.language;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
     * Implementao do tipo lista de inteiros. i.e. [1..], [..1], [1..20], [-12..0]
 */
public class ExpressionListNumber extends AbstractExpression implements
    ExpressionList, ComparableExpression {

    private long minNumber;
    private long maxNumber;

    public ExpressionListNumber() {
        this(ExpressionNumber.MIN_LONG, ExpressionNumber.MAX_LONG);
    }

    public ExpressionListNumber(ExpressionNumber min, ExpressionNumber max) {
        this(min.getLong(), max.getLong());
    }

    public ExpressionListNumber(long min, long max) {
        super(ExpressionType.LIST);
        if(max < min) {
            throw new IllegalArgumentException(
                "Max value must be greater than min value! " + max + " < "
                + min);
        }
        setMinNumber(min);
        setMaxNumber(max);
    }

    public void setMinNumber(long minNumber) {
        this.minNumber = minNumber;
    }

    public long getMinNumber() {
        return minNumber;
    }

    public void setMaxNumber(long maxNumber) {
        this.maxNumber = maxNumber;
    }

    public long getMaxNumber() {
        return maxNumber;
    }

    public Object getData() {
        long[] result = new long[(int) length()];
        for(long i = minNumber; i < maxNumber; i++) {
            result[(int) (i - minNumber)] = i;
        }
        return result;
    }

    public ExpressionList clear() {
        /**@todo: Implement this jpel.language.ExpressionList method*/
        throw new java.lang.UnsupportedOperationException(
            "Method clear() not supported for this type.");
    }

    public ExpressionList append(Expression expression) {
        /**@todo: Implement this jpel.language.ExpressionList method*/
        throw new java.lang.UnsupportedOperationException(
            "Method append() not supported for this type.");
    }

    public ExpressionList append(ExpressionList expressionList) {
        /**@todo: Implement this jpel.language.ExpressionList method*/
        throw new java.lang.UnsupportedOperationException(
            "Method append() not supported for this type.");
    }

    public ExpressionList remove(Expression expression) {
        /**@todo: Implement this jpel.language.ExpressionList method*/
        throw new java.lang.UnsupportedOperationException(
            "Method remove() not supported for this type.");
    }

    public ExpressionList remove(ExpressionList expressionList) {
        /**@todo: Implement this jpel.language.ExpressionList method*/
        throw new java.lang.UnsupportedOperationException(
            "Method remove() not supported for this type.");
    }

    public boolean contains(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionNumber) {
            ExpressionNumber n = (ExpressionNumber) expression;
            result = minNumber
                     <= ((Integer) n.getData(Integer.class)).intValue()
                     && ((Integer) n.getData(Integer.class)).intValue()
                     <= maxNumber;
        }
        return result;
    }

    public boolean contains(ExpressionList expressionList) {
        boolean result = true;
        if(expressionList instanceof ExpressionListNumber) {
            ExpressionListNumber n = (ExpressionListNumber) expressionList;
            result = (minNumber <= n.minNumber) && (n.maxNumber <= maxNumber);
        }
        else {
            Iterator i = expressionList.iterator();
            while(result && i.hasNext()) {
                result = contains((Expression) i.next());
            }
        }
        return result;
    }

    public boolean isEmpty() {
        return(minNumber > maxNumber);
    }

    public long length() {
        return(maxNumber - minNumber);
    }

    public Expression head() {
        return new ExpressionNumber(minNumber);
    }

    public ExpressionList tail() {
        return(minNumber + 1 <= maxNumber)
            ? (ExpressionList)new ExpressionListNumber(minNumber + 1,
            maxNumber)
            : (ExpressionList)new ExpressionListImpl();
    }

    public Iterator iterator() {
        return new ExpressionListIterator(this);
    }

    public void freeVariable(ExpressionList list) { /** do nothing */}

    public Expression rebuild(MapReplace map) {
        return new ExpressionListNumber(minNumber, maxNumber);
    }

    public Expression eval(Environment environment) throws
        IdNotFoundException, BadTypedException {
        return this;
    }

    public boolean equivalent(Expression expression) {
        boolean result = false;
        if(expression instanceof ExpressionListNumber) {
            ExpressionListNumber el = (ExpressionListNumber) expression;
            result = (minNumber == el.minNumber)
                     && (maxNumber == el.maxNumber);
        }
        return result;
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        if(expression instanceof ExpressionList) {
            if(isEmpty() && ((ExpressionList) expression).isEmpty()) {
                return 0;
            }
        }
        if(!(expression instanceof ExpressionListNumber)) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression
                + ". Only ListNumbers can be compared to ListNumbers.");
        }
        return(int) (minNumber
            - ((ExpressionListNumber) expression).minNumber);
    }

    public String toString() {
        return toString("[", "..", "]");
    }

    public String toString(String prefix, String separator, String suffix) {
        return prefix +
            (minNumber == Integer.MIN_VALUE ? "" : String.valueOf(minNumber)) +
            separator +
            (maxNumber == Integer.MAX_VALUE ? "" : String.valueOf(maxNumber)) +
            suffix;
    }

    public static void main(String[] args) {
        ExpressionListNumber e = new ExpressionListNumber(0, 10);
        checkE(e);
        e = new ExpressionListNumber( -10, 0);
        checkE(e);
        e = new ExpressionListNumber( -10, 10);
        checkE(e);
        e = new ExpressionListNumber(Integer.MIN_VALUE, 10);
        checkE(e);
        e = new ExpressionListNumber( -10, Integer.MAX_VALUE);
        checkE(e);
    }

    private static void checkE(ExpressionListNumber e) {
        try {
            System.out.println("toString()=" + e);
            System.out.println("isEmpty()=" + e.isEmpty());
            System.out.println("length()=" + e.length());
            System.out.println("head()=" + e.head());
            System.out.println("tail()=" + e.tail());
            System.out.println("contains(1)="
                + e.contains(new ExpressionNumber(1)));
            System.out.println("contains(-1)="
                + e.contains(new ExpressionNumber( -1)));
        }
        catch(Exception exc) {
        }
    }
}
