package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Representa o tipo numrico da linguagem. Tipos aceitos: byte, short, int, long, float, double.
 */
public class ExpressionNumber extends ExpressionObject implements
    StringableExpression, ComparableExpression {

    // identificadores de tipos.
    public static final int BYTE = 0;
    public static final int SHORT = BYTE + 1;
    public static final int INT = SHORT + 1;
    public static final int LONG = INT + 1;
    public static final int FLOAT = LONG + 1;
    public static final int DOUBLE = FLOAT + 1;
    private int kind;

    //maximos e mnimos
    public static final ExpressionNumber MIN_BYTE = new
        ExpressionNumber(Byte.MIN_VALUE);
    public static final ExpressionNumber MAX_BYTE = new
        ExpressionNumber(Byte.MAX_VALUE);
    public static final ExpressionNumber MIN_SHORT = new
        ExpressionNumber(Short.MIN_VALUE);
    public static final ExpressionNumber MAX_SHORT = new
        ExpressionNumber(Short.MAX_VALUE);
    public static final ExpressionNumber MIN_INT = new
        ExpressionNumber(Integer.MIN_VALUE);
    public static final ExpressionNumber MAX_INT = new
        ExpressionNumber(Integer.MAX_VALUE);
    public static final ExpressionNumber MIN_LONG = new
        ExpressionNumber(Long.MIN_VALUE);
    public static final ExpressionNumber MAX_LONG = new
        ExpressionNumber(Long.MAX_VALUE);
    public static final ExpressionNumber MIN_FLOAT = new
        ExpressionNumber(Float.MIN_VALUE);
    public static final ExpressionNumber MAX_FLOAT = new
        ExpressionNumber(Float.MAX_VALUE);
    public static final ExpressionNumber MIN_DOUBLE = new
        ExpressionNumber(Double.MIN_VALUE);
    public static final ExpressionNumber MAX_DOUBLE = new
        ExpressionNumber(Double.MAX_VALUE);

    public ExpressionNumber(byte value) {
        this(new Byte(value));
    }

    public ExpressionNumber(Byte value) {
        super(ExpressionType.NUMBER, value);
        kind = BYTE;
    }

    public ExpressionNumber(short value) {
        this(new Short(value));
    }

    public ExpressionNumber(Short value) {
        super(ExpressionType.NUMBER, value);
        kind = SHORT;
    }

    public ExpressionNumber(int value) {
        this(new Integer(value));
    }

    public ExpressionNumber(Integer value) {
        super(ExpressionType.NUMBER, value);
        kind = INT;
    }

    public ExpressionNumber(long value) {
        this(new Long(value));
    }

    public ExpressionNumber(Long value) {
        super(ExpressionType.NUMBER, value);
        kind = LONG;
    }

    public ExpressionNumber(float value) {
        this(new Float(value));
    }

    public ExpressionNumber(Float value) {
        super(ExpressionType.NUMBER, value);
        kind = FLOAT;
    }

    public ExpressionNumber(double value) {
        this(new Double(value));
    }

    public ExpressionNumber(Double value) {
        super(ExpressionType.NUMBER, value);
        kind = DOUBLE;
    }

    public ExpressionNumber(String value) { //  usa o menor tipo possivel
        super(ExpressionType.NUMBER, value);
        try {
            this.setData(Byte.valueOf(value));
            kind = BYTE;
        }
        catch(NumberFormatException nfe1) {
            try {
                this.setData(Short.valueOf(value));
                kind = SHORT;
            }
            catch(NumberFormatException nfe2) {
                try {
                    this.setData(Integer.valueOf(value));
                    kind = INT;
                }
                catch(NumberFormatException nfe3) {
                    try {
                        this.setData(Long.valueOf(value));
                        kind = LONG;
                    }
                    catch(NumberFormatException nfe4) {
                        try {
                            this.setData(Float.valueOf(value));
                            kind = FLOAT;
                        }
                        catch(NumberFormatException nfe5) {
                            try {
                                this.setData(Double.valueOf(value));
                                kind = DOUBLE;
                            }
                            catch(NumberFormatException nfe6) {
                                throw new IllegalArgumentException(
                                    "Invalid number string '" + value + "'");
                            }
                        }
                    }
                }
            }
        }
    }

    public Object getData(Class type) {
        if(byte.class == type || Byte.class == type) {
            return new Byte(getByte());
        }
        else if(short.class == type || Short.class == type) {
            return new Short(getShort());
        }
        else if(int.class == type || Integer.class == type) {
            return new Integer(getInt());
        }
        else if(long.class == type || Long.class == type) {
            return new Long(getLong());
        }
        else if(float.class == type || Float.class == type) {
            return new Float(getFloat());
        }
        else if(double.class == type || Double.class == type) {
            return new Double(getDouble());
        }
        else {
            return super.getData(type);
        }
    }

    public boolean isByte() {
        return(getData()instanceof Byte);
    }

    public boolean canBeByte() {
        return isByte() || canBeShort();
    }

    public byte getByte() {
        if(isByte()) {
            return((Byte) getData()).byteValue();
        }
        else if(isShort()) {
            return((Short) getData()).byteValue();
        }
        else if(isInt()) {
            return((Integer) getData()).byteValue();
        }
        else if(isLong()) {
            return((Long) getData()).byteValue();
        }
        else if(isFloat()) {
            return((Float) getData()).byteValue();
        }
        else if(isDouble()) {
            return((Double) getData()).byteValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public boolean isShort() {
        return(getData()instanceof Short) || isByte();
    }

    public boolean canBeShort() {
        return isShort() || canBeInt();
    }

    public short getShort() {
        if(isByte()) {
            return((Byte) getData()).shortValue();
        }
        else if(isShort()) {
            return((Short) getData()).shortValue();
        }
        else if(isInt()) {
            return((Integer) getData()).shortValue();
        }
        else if(isLong()) {
            return((Long) getData()).shortValue();
        }
        else if(isFloat()) {
            return((Float) getData()).shortValue();
        }
        else if(isDouble()) {
            return((Double) getData()).shortValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public boolean isInt() {
        return(getData()instanceof Integer) || isShort();
    }

    public boolean canBeInt() {
        return isInt() || canBeLong();
    }

    public int getInt() {
        if(isByte()) {
            return((Byte) getData()).intValue();
        }
        else if(isShort()) {
            return((Short) getData()).intValue();
        }
        else if(isInt()) {
            return((Integer) getData()).intValue();
        }
        else if(isLong()) {
            return((Long) getData()).intValue();
        }
        else if(isFloat()) {
            return((Float) getData()).intValue();
        }
        else if(isDouble()) {
            return((Double) getData()).intValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public boolean isLong() {
        return(getData()instanceof Long) || isInt();
    }

    public boolean canBeLong() {
        return isLong() || canBeFloat();
    }

    public long getLong() {
        if(isByte()) {
            return((Byte) getData()).longValue();
        }
        else if(isShort()) {
            return((Short) getData()).longValue();
        }
        else if(isInt()) {
            return((Integer) getData()).longValue();
        }
        else if(isLong()) {
            return((Long) getData()).longValue();
        }
        else if(isFloat()) {
            return((Float) getData()).longValue();
        }
        else if(isDouble()) {
            return((Double) getData()).longValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public boolean isFloat() {
        return(getData()instanceof Float) || isLong();
    }

    public boolean canBeFloat() {
        return isFloat() || canBeDouble();
    }

    public float getFloat() {
        if(isByte()) {
            return((Byte) getData()).floatValue();
        }
        else if(isShort()) {
            return((Short) getData()).floatValue();
        }
        else if(isInt()) {
            return((Integer) getData()).floatValue();
        }
        else if(isLong()) {
            return((Long) getData()).floatValue();
        }
        else if(isFloat()) {
            return((Float) getData()).floatValue();
        }
        else if(isDouble()) {
            return((Double) getData()).floatValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public boolean isDouble() {
        return(getData()instanceof Double) || isFloat();
    }

    public boolean canBeDouble() {
        return isDouble();
    }

    public double getDouble() {
        if(isByte()) {
            return((Byte) getData()).doubleValue();
        }
        else if(isShort()) {
            return((Short) getData()).doubleValue();
        }
        else if(isInt()) {
            return((Integer) getData()).doubleValue();
        }
        else if(isLong()) {
            return((Long) getData()).doubleValue();
        }
        else if(isFloat()) {
            return((Float) getData()).doubleValue();
        }
        else if(isDouble()) {
            return((Double) getData()).doubleValue();
        }
        throw new IllegalArgumentException("Data object must be a number.");
    }

    public int bestMatch(ExpressionNumber number) {
        return Math.max(kind, number.kind);
    }

    public String asString() {
        return String.valueOf(getData());
    }

    public int compareTo(ComparableExpression expression) throws
        BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                this +" can not be compared to " + expression);
        }
        ExpressionNumber exp = (ExpressionNumber) expression;
        return(int) (getDouble() - exp.getDouble());
    }

    public String toString() {
        return asString();
    }

    public ExpressionNumber add(ExpressionNumber number) {
        int cast = bestMatch(number);
        if(cast == BYTE) {
            return new ExpressionNumber(getByte() + number.getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber(getShort() + number.getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber(getInt() + number.getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber(getLong() + number.getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber(getFloat() + number.getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber(getDouble() + number.getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }

    public ExpressionNumber sub(ExpressionNumber number) {
        int cast = bestMatch(number);
        if(cast == BYTE) {
            return new ExpressionNumber(getByte() - number.getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber(getShort() - number.getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber(getInt() - number.getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber(getLong() - number.getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber(getFloat() - number.getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber(getDouble() - number.getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }

    public ExpressionNumber mul(ExpressionNumber number) {
        int cast = bestMatch(number);
        if(cast == BYTE) {
            return new ExpressionNumber(getByte() * number.getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber(getShort() * number.getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber(getInt() * number.getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber(getLong() * number.getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber(getFloat() * number.getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber(getDouble() * number.getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }

    public ExpressionNumber div(ExpressionNumber number) {
        int cast = bestMatch(number);
        if(cast == BYTE) {
            return new ExpressionNumber(getByte() / number.getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber(getShort() / number.getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber(getInt() / number.getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber(getLong() / number.getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber(getFloat() / number.getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber(getDouble() / number.getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }

    public ExpressionNumber rem(ExpressionNumber number) {
        int cast = bestMatch(number);
        if(cast == BYTE) {
            return new ExpressionNumber(getByte() % number.getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber(getShort() % number.getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber(getInt() % number.getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber(getLong() % number.getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber(getFloat() % number.getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber(getDouble() % number.getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }

    public ExpressionNumber neg() {
        int cast = kind;
        if(cast == BYTE) {
            return new ExpressionNumber( -getByte());
        }
        else if(cast == SHORT) {
            return new ExpressionNumber( -getShort());
        }
        else if(cast == INT) {
            return new ExpressionNumber( -getInt());
        }
        else if(cast == LONG) {
            return new ExpressionNumber( -getLong());
        }
        else if(cast == FLOAT) {
            return new ExpressionNumber( -getFloat());
        }
        else if(cast == DOUBLE) {
            return new ExpressionNumber( -getDouble());
        }
        throw new IllegalArgumentException("Invalid number type.");
    }
}
