package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.Serializable;

/**
 * Classe usada para fornecer a linguagem a noo de tipos. Por enquanto,
 * a validao de tipo apenas est presente na avaliao de expresses.
 */
public class ExpressionType implements Serializable {

    // realy unknown
    private static final int UNKNOWN_CODE = 0;
    // basic types
    private static final int BOOLEAN_CODE = UNKNOWN_CODE + 1;
    private static final int NUMBER_CODE = BOOLEAN_CODE + 1;
    private static final int CHARACTER_CODE = NUMBER_CODE + 1;
    private static final int STRING_CODE = CHARACTER_CODE + 1;
    private static final int OBJECT_CODE = STRING_CODE + 1;
    private static final int ID_CODE = OBJECT_CODE + 1;
    private static final int LIST_CODE = ID_CODE + 1;

    /**
     * representa um tipo ainda no conhecido. Usado nas expresses
     * ifThenElse e Switch, onde o resultado da operao no tem tipo
     * conhecido.
     */
    public static final ExpressionType UNKNOWN = new ExpressionType(
                                                 UNKNOWN_CODE);
    /**
     * Representa o tipo boolean.
     */
    public static final ExpressionType BOOLEAN = new ExpressionType(
                                                 BOOLEAN_CODE);
    /**
     * Representa o tipo numrico.
     */
    public static final ExpressionType NUMBER = new ExpressionType(NUMBER_CODE);
    /**
     * Representa o tipo caracter.
     */
    public static final ExpressionType CHARACTER = new ExpressionType(
        CHARACTER_CODE);
    /**
     * Representa o tipo String.
     */
    public static final ExpressionType STRING = new ExpressionType(STRING_CODE);
    /**
     * Representa o tipo Object.
     */
    public static final ExpressionType OBJECT = new ExpressionType(OBJECT_CODE);
    /**
     * Indica um identificador.
     */
    public static final ExpressionType ID = new ExpressionType(ID_CODE);
    /**
     * Representa o tipo List.
     */
    public static final ExpressionType LIST = new ExpressionType(LIST_CODE);

    private int type;

    private ExpressionType(int type) {
        this.type = type;
    }

    public int getType() {
        return type;
    }

    public boolean isUnknown() {
        return check(UNKNOWN_CODE);
    }

    public boolean isBoolean() {
        return check(BOOLEAN_CODE);
    }

    public boolean isNumber() {
        return check(NUMBER_CODE);
    }

    public boolean isCharacter() {
        return check(CHARACTER_CODE);
    }

    public boolean isString() {
        return check(STRING_CODE);
    }

    public boolean isObject() {
        return check(OBJECT_CODE);
    }

    public boolean isId() {
        return check(ID_CODE);
    }

    public boolean isList() {
        return check(LIST_CODE) || check(STRING_CODE);
    }

    private boolean check(int t) {
        return type == t;
    }

    public boolean equivalent(Object obj) {
        boolean result = false;
        if(obj instanceof ExpressionType) {
            result = (type == ((ExpressionType) obj).type);
        }
        return result;
    }

    public String toString() {
        String result = "Not found!";
        switch(getType()) {
            case UNKNOWN_CODE:
                result = "Unkown";
                break;
            case BOOLEAN_CODE:
                result = "Boolean";
                break;
            case NUMBER_CODE:
                result = "Number";
                break;
            case CHARACTER_CODE:
                result = "Character";
                break;
            case STRING_CODE:
                result = "String";
                break;
            case OBJECT_CODE:
                result = "Object";
                break;
            case ID_CODE:
                result = "Id";
                break;
            case LIST_CODE:
                result = "List";
                break;
        }
        return result;
    }
}
