package jpel.language;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

/**
 * Durante a execuo de uma expresso, pode-se desejar substituir identificadores
 * ou valores. Para que haja esta substituio deve-se fornecer um mapeamento
 * entre o se deseja substituir e o que deve ser colocado no lugar. A funo desta
 * classe  fornecer este mapeamento.
 */
public interface MapReplace {

    /**
         * Adiciona um substituio que deve acontecer quando o mapeamento for utilizado.
     * @param before Expresses que se deseja substituir.
     * @param after Expresso que deve ser colocada no lugar.
     */
    void put(Expression before, Expression after);

    /**
     * Indica qual o novo valor da expresso dada.
     * @param expression Expresso original.
         * @return A expresso que deve ser utilizada no lugar da expresso original.
     */
    Expression get(Expression expression);

    /**
     * Remove uma expresso do mapeamento.
     * @param remove A expresso  que deve se rremovida do mapeamento.
     */
    void remove(Expression remove);

    /**
     * Lista as expressoes que devem ser substituidas.
     * @return
     */
    Iterator keys();

    /**
     * Cria uma cpia do mapeamento.
     * @return Um mapeamento idntico.
     */
    MapReplace mirror();
}
