package jpel.language.extensions;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.InputStream;

import jpel.language.BinaryExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionBoolean;
import jpel.language.ExpressionList;
import jpel.language.ExpressionString;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;

/**
 * Define um parmetro que  o resultado da execuo de um processo. Isto , o
 * valor do parmetro que utiliza um comando deste tipo  na verdade a captura
 * do output do programa chamado.
 */

public class ExpressionCommand extends BinaryExpression {

    public ExpressionCommand(Expression command, Expression wait) {
        this("exec", command, wait);
    }

    public ExpressionCommand(String id, Expression command, Expression wait) {
        super(id, ExpressionType.STRING, command, wait);
    }

    public void freeVariable(ExpressionList list) {
        getLeftExpression().freeVariable(list);
        getRightExpression().freeVariable(list);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression command = getLeftExpression().eval(environment);
        if(!(command instanceof StringableExpression)) {
            throw new ExecutionException(this,
                "The command can not be converted to a String. Argument given:"
                + command);
        }
        Expression wait = getRightExpression().eval(environment);
        if(!wait.getType().isBoolean()) {
            throw new ExecutionException(this,
                "The information about waiting a process is not boolean. Argument given:"
                + wait);
        }
        String str = ((StringableExpression) command).asString();
        Runtime rt = Runtime.getRuntime();
        try {
            Process process = rt.exec(str);
            if(((ExpressionBoolean) wait).getBoolean()) {
                process.waitFor();
                int exitValue = process.exitValue();
                if(exitValue != 0) {
                    process.destroy();
                    throw new ExecutionException(this,
                        "The command '" + str + "' had a exit value '"
                        + exitValue + "'.");
                }
                else {
                    try {
                        InputStream in = process.getInputStream();
                        int c;
                        StringBuffer out = new StringBuffer();
                        while((c = in.read()) != -1) {
                            out.append((char) c);
                        }
                        process.destroy();
                        return new ExpressionString(out.toString());
                    }
                    catch(IOException exc) {
                        throw new ExecutionException(this,
                            "The command '" + str + "' could not be executed.",
                            exc);
                    }
                }
            }
            else {
                return new ExpressionString("ok");
            }
        }
        catch(IOException exc) {
            throw new ExecutionException(this,
                "The command '" + str + "' could not be executed.", exc);
        }
        catch(InterruptedException exc) {
            throw new ExecutionException(this,
                "Error while waiting for '" + str + "'", exc);
        }
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionCommand(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}