package jpel.language.extensions;

import java.sql.Connection;
import java.sql.DriverManager;

import jpel.language.AbstractExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionObject;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;

/**
 * Realiza uma conexo com um banco de dados.
 */
public class ExpressionConnection extends AbstractExpression {

    private Expression driver;
    private Expression url;
    private Expression user;
    private Expression password;

    public ExpressionConnection(Expression driver, Expression url,
        Expression user, Expression password) {
        this("con", driver, url, user, password);
    }

    public ExpressionConnection(String id, Expression driver, Expression url,
        Expression user, Expression password) {
        super(ExpressionType.OBJECT);
        setDriver(driver);
        setUrl(url);
        setUser(user);
        setPassword(password);
    }

    public void setDriver(Expression driver) {
        this.driver = driver;
    }

    public Expression getDriver() {
        return driver;
    }

    public void setUrl(Expression url) {
        this.url = url;
    }

    public Expression getUrl() {
        return url;
    }

    public void setUser(Expression user) {
        this.user = user;
    }

    public Expression getUser() {
        return user;
    }

    public void setPassword(Expression password) {
        this.password = password;
    }

    public Expression getPassword() {
        return password;
    }

    public void freeVariable(ExpressionList list) {
        driver.freeVariable(list);
        url.freeVariable(list);
        user.freeVariable(list);
        password.freeVariable(list);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionConnection(
            driver.rebuild(novo),
            url.rebuild(novo),
            user.rebuild(novo),
            password.rebuild(novo)
            );
    }

    public Expression eval(Environment env) throws ExecutionException {
        try {
            Expression result = null;
            StringableExpression drv = (StringableExpression)this.driver.
                                       eval(env);
            StringableExpression url = (StringableExpression)this.url.eval(
                                       env);
            StringableExpression usr = (StringableExpression)this.user.
                                       eval(env);
            StringableExpression pwd = (StringableExpression)this.password.
                                       eval(env);
            Class.forName(drv.asString());
            Connection con = DriverManager.getConnection(
                             url.asString(),
                             usr.asString(),
                             pwd.asString()
                             );
            return new ExpressionObject(con);
        }
        catch(Exception exc) {
            throw new ExecutionException(this, exc.getMessage(), exc);
        }
    }

/*
    public boolean equivalent(Expression exp) {
        boolean result = false;
        if(exp instanceof ExpressionConnection) {
            ExpressionConnection ec = (ExpressionConnection) exp;
            result = getDriver().equivalent(ec.getDriver()) &&
                     getUrl().equivalent(ec.getUrl()) &&
                     getUser().equivalent(ec.getUser()) &&
                     getPassword().equivalent(ec.getPassword());
        }
        return result;
    }
*/
}