package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.BinaryExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionNumber;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;

/**
 * Realiza a seleo do n-esimo elemento de uma lista.
 */
public class ExpressionAt extends BinaryExpression {

    public ExpressionAt(Expression left, Expression right) {
        this("at", left, right);
    }

    public ExpressionAt(String identity, Expression left, Expression right) {
        super(identity, ExpressionType.UNKNOWN, left, right);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression index = getLeftExpression().eval(environment);
        if(!index.getType().isNumber()) {
            throw new BadTypedException(this,
                "Left parameter must be a number (the index). Argument given:"
                + index);
        }
        Expression list = getRightExpression().eval(environment);
        if(!list.getType().isList()) {
            throw new BadTypedException(this,
                "Right parameter must be a list (the list). Argument given:"
                + list);
        }
        ExpressionNumber number = (ExpressionNumber) index;
        long i = 0;
        long tmp = number.getLong();
        while(i < tmp && ((ExpressionList) list).tail() != null) {
            list = ((ExpressionList) list).tail();
            i++;
        }
        return((ExpressionList) list).head();
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionAt(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}
