package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.BinaryExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionListImpl;
import jpel.language.ExpressionString;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;

/**
 * Operao de concatenao entre ExpressionLists ou StringableExpression.
 * <br>Alguma regras de tipo dever ser observadas:
 * <br>StringableExpression ++ StringableExpression = ExpressionString
 * <br>List ++ List = List.
 */
public class ExpressionConcatenation extends BinaryExpression {

    public ExpressionConcatenation(Expression left, Expression right) {
        this("++", left, right);
    }

    public ExpressionConcatenation(String identity, Expression left,
        Expression right) {
        super(identity, ExpressionType.LIST, left, right);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression l = getLeftExpression().eval(environment);
        Expression r = getRightExpression().eval(environment);
        if((l instanceof StringableExpression)
            && (r instanceof StringableExpression)) {
            return new ExpressionString(((StringableExpression) l).
                asString() + ((StringableExpression) r).asString());
        }
        else if(l.getType().isList() && r.getType().isList()) {
            ExpressionList vl = new ExpressionListImpl();
            vl.append((ExpressionList) l);
            vl.append((ExpressionList) r);
            return vl;
        }
        else {
            throw new BadTypedException(this,
                "Parameters must be either StringableExpression values or lists.");
        }
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionConcatenation(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}
