package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.BinaryExpression;
import jpel.language.ComparableExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionBoolean;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;

/**
 * Operao de igualdade entre expresses.
 */
public class ExpressionEquals extends BinaryExpression {

    public ExpressionEquals(Expression left,
        Expression right) {
        this("==", left, right);
    }

    public ExpressionEquals(String identity, Expression left,
        Expression right) {
        super(identity, ExpressionType.BOOLEAN, left, right);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression l = getLeftExpression().eval(environment);
        if(!(l instanceof ComparableExpression)) {
            throw new BadTypedException(this,
                "Left parameter must be a ComparableExpression. Argument given:"
                + l);
        }
        Expression r = getRightExpression().eval(environment);
        if(!(r instanceof ComparableExpression)) {
            throw new BadTypedException(this,
                "Left parameter must be a ComparableExpression. Argument given:"
                + r);
        }
        if(!l.getType().equivalent(r.getType())) {
            throw new BadTypedException(this,
                "Parameters must have the same type to be compared. Arguments given: left("
                + l + ") and right(" + r + ")");
        }
        return ExpressionBoolean.getBoolean(((
            ComparableExpression) l).compareTo((
            ComparableExpression) r) == 0);
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionEquals(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}
