package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.BinaryExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionBoolean;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;

/**
 * Operao de or lgico.
 */
public class ExpressionOr extends BinaryExpression {

    public ExpressionOr(Expression left, Expression right) {
        this("|", left, right);
    }

    public ExpressionOr(String identity, Expression left, Expression right) {
        super(identity, ExpressionType.BOOLEAN, left, right);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression l = getLeftExpression().eval(environment);
        if(!l.getType().isBoolean()) {
            throw new BadTypedException(this,
                "Left parameter must be a boolean. Argument given:" + l);
        }
        Expression r = getRightExpression().eval(environment);
        if(!r.getType().isBoolean()) {
            throw new BadTypedException(this,
                "Right parameter must be a boolean. Argument given:" + r);
        }
        return ExpressionBoolean.getBoolean(((ExpressionBoolean) l).getBoolean()
            || ((ExpressionBoolean) r).getBoolean());
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionOr(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}
