package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.BinaryExpression;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionNumber;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;

/**
 * Operao de potenciao entre numeros.
 */
public class ExpressionPow extends BinaryExpression {

    public ExpressionPow(Expression left, Expression right) {
        this("^", left, right);
    }

    public ExpressionPow(String identity, Expression left,
        Expression right) {
        super(identity, ExpressionType.NUMBER, left, right);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression l = getLeftExpression().eval(environment);
        if(!l.getType().isNumber()) {
            throw new BadTypedException(this,
                "Left parameter must be a number. Argument given:" + l);
        }
        Expression r = getRightExpression().eval(environment);
        if(!r.getType().isNumber()) {
            throw new BadTypedException(this,
                "Right parameter must be a number. Argument given:" + r);
        }
        return new ExpressionNumber(Math.pow(((ExpressionNumber) l).
            getDouble(), ((ExpressionNumber) r).getDouble()));
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionPow(getName(),
            getLeftExpression().rebuild(novo),
            getRightExpression().rebuild(novo));
    }
}
