package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;
import jpel.language.UnaryExpression;

/**
 * Operao que retorna a cauda de uma lista.
 */
public class ExpressionTail extends UnaryExpression {

    public ExpressionTail(Expression expression) {
        this("tail", expression);
    }

    public ExpressionTail(String identity, Expression expression) {
        super(identity, ExpressionType.LIST, expression);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression list = getExpression().eval(environment);
        if(!list.getType().isList()) {
            throw new BadTypedException(this,
                "Parameter must be a list. Argument given:" + list);
        }
        return((ExpressionList) list).tail();
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionTail(getName(), getExpression().rebuild(novo));
    }
}
