package jpel.language.operators;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.BadTypedException;
import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionString;
import jpel.language.ExpressionType;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;
import jpel.language.UnaryExpression;

/**
 * Operao que retorna uma String do que foi passado sem carateres vazios
 * no comeo ou no fim. Equivalente a String.trim().
 */
public class ExpressionTrim extends UnaryExpression {

    public ExpressionTrim(Expression expression) {
        this("trim", expression);
    }

    public ExpressionTrim(String identity, Expression expression) {
        super(identity, ExpressionType.STRING, expression);
    }

    public Expression eval(Environment environment) throws
        ExecutionException {
        Expression result = getExpression().eval(environment);
        if(!(result instanceof StringableExpression)) {
            throw new BadTypedException(this,
                "Parameter must be a \"Stringable\" expresssion. Argument given:"
                + result);
        }
        return new ExpressionString(((StringableExpression) result).
            asString().trim());
    }

    public Expression rebuild(MapReplace map) {
        MapReplace novo = map.mirror();
        return new ExpressionTrim(getName(), getExpression().rebuild(novo));
    }
}
