package jpel.remote.language;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.AbstractNamedExpression;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionType;

/**
 * Define o prottipo de  uma expresso remota.
 */
public abstract class ExpressionRemote extends AbstractNamedExpression {

    /**
     * Servidor p/execuo.
     */
    private Expression host;
    /**
     * Porta do servio.
     */
    private Expression port;
    /**
     * Expresso que deve ser avaliada remotamente.
     */
    private Expression expression;

    public ExpressionRemote( String id, ExpressionType type, Expression host,
        Expression port, Expression expression ) {
        super( type,id );
        setHost( host );
        setPort( port );
        setExpression( expression );
    }

    public Expression getHost() {
        return host;
    }

    public void setHost( Expression host ) {
        this.host = host;
    }

    public void setPort( Expression port ) {
        this.port = port;
    }

    public Expression getPort() {
        return port;
    }

    public void setExpression( Expression expression ) {
        this.expression = expression;
    }

    public Expression getExpression() {
        return expression;
    }

    public void freeVariable( ExpressionList list ) {
        host.freeVariable( list );
        port.freeVariable( list );
        // a expresso s deve ser avaliada no lado remoto,
        // desconsideramos as variveis livres.
    }

    public boolean equivalent( Expression expression ) {
        boolean result = false;
        if( expression instanceof ExpressionRemote ) {
            ExpressionRemote e = ( ExpressionRemote ) expression;
            result = getHost().equivalent( e.getHost() ) &&
                     getPort().equivalent( e.getPort() ) &&
                     getExpression().equivalent( e.getExpression() );
        }
        return result;
    }
}
