package jpel.remote.language.rmi;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.net.MalformedURLException;
import java.rmi.Naming;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;

import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionList;
import jpel.language.ExpressionNumber;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;
import jpel.language.ExpressionType;
import jpel.remote.language.ExpressionRemote;

/**
     * Define a execuo de uma expresso remotamente. Para a execuo remota deve-se
 * indicar o nome do servidor (address) e a expresso que deve ser avaliada.
 * Lembrando sempre qua apenas expresses serializveis obtero os resultados
 * corretos.
 */
public class ExpressionRMI extends ExpressionRemote {

    private Expression rmiName;

    public ExpressionRMI( Expression host, Expression port, Expression rmiName,
        Expression expression ) {
        this( "RMI", host, port, rmiName, expression );
    }

    public ExpressionRMI( String id, Expression host, Expression port,
        Expression rmiName, Expression expression ) {
        super( id, ExpressionType.UNKNOWN, host, port, expression );
        setRmiName( rmiName );
    }

    public Expression getRmiName() {
        return rmiName;
    }

    public void setRmiName( Expression rmiName ) {
        this.rmiName = rmiName;
    }

    public void freeVariable( ExpressionList list ) {
        super.freeVariable( list );
        rmiName.freeVariable( list );
    }

    public Expression rebuild( MapReplace map ) {
        MapReplace novo = map.mirror();
        return new ExpressionRMI(
            getName(),
            getHost().rebuild( novo ),
            getPort().rebuild( novo ),
            getRmiName().rebuild( novo ),
            getExpression().rebuild( novo )
            );
    }

    public Expression eval( Environment environment ) throws
        ExecutionException {
        try {
            StringableExpression varHost = ( StringableExpression ) getHost().
                                           eval( environment );
            ExpressionNumber varPort = ( ExpressionNumber ) getPort().eval(
                                       environment );
            StringableExpression varName = ( StringableExpression ) getRmiName().eval( environment );
            String remoteName = "rmi://" + varHost.asString() + ":"
                                + varPort.getInt() + "/" + varName.asString();
            ExpressionServerRMI server = ( ExpressionServerRMI ) Naming.lookup(
                                         remoteName );
            return server.execute( getExpression() );
        }
        catch( RemoteException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( NotBoundException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( MalformedURLException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
    }

    public boolean equivalent( Expression expression ) {
        boolean result = false;
        if( expression instanceof ExpressionRMI ) {
            ExpressionRMI e = ( ExpressionRMI ) expression;
            result = super.equivalent( expression ) &&
                     getRmiName().equivalent( e.getRmiName() );
        }
        return result;
    }
}
