package jpel.remote.language.rmi;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.UnicastRemoteObject;

import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.util.Debugger;

public class ExpressionServerRMIImpl extends UnicastRemoteObject implements
    ExpressionServerRMI {

    private int port;
    private String name;
    private Registry registry;
    private Environment environment;

    private int serialNumber;

    public ExpressionServerRMIImpl( int port, String name,
        Environment environment ) throws RemoteException {
        this.port = port;
        this.name = name;
        this.environment = environment;
        this.serialNumber = 0;
        try {
            registry = LocateRegistry.getRegistry( port );
            registry.rebind( name, this );
            Debugger.println( "RMIServer", "bind", "Recycled" );
        }
        catch( Exception exc ) {
            registry = LocateRegistry.createRegistry( port );
            registry.rebind( name, this );
            Debugger.println( "RMIServer", "bind", "New" );
        }
        Debugger.println( "RMIServer", "bind",
            "Bounded at " + port + ":" + name );
    }

    public void close() {
        try {
            registry.unbind( name );
            Debugger.println( "RMIServer", "close", "Closed!" );
        }
        catch( NotBoundException ex ) {
            ex.printStackTrace();
        }
        catch( RemoteException ex ) {
            ex.printStackTrace();
        }
    }

    public Expression execute( Expression expression ) throws
        ExecutionException, RemoteException {
        serialNumber++;
        String from = null;
        try {
            from = getClientHost();
        }
        catch( ServerNotActiveException exc ) {
            exc.printStackTrace();
        }

        Debugger.println( "RMIServer", "request(" + serialNumber + ")",
            from != null ? from + ": " + String.valueOf( expression )
            : String.valueOf( expression )
            );
        Expression result = expression.eval( environment );
        Debugger.println( "RMIServer", "result(" + serialNumber + ")",
            from != null ? from + ": " + String.valueOf( result )
            : String.valueOf( result )
            );
        return result;
    }
}
