package jpel.remote.language.socket;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.net.ServerSocket;
import java.net.Socket;

import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.util.Debugger;

public class ExpressionServerSocket implements Runnable {

    private int port;
    private ServerSocket server;
    private Environment environment;

    public ExpressionServerSocket( int port,
        Environment environment ) throws IOException {
        this.port = port;
        this.server = new ServerSocket( port );
        this.environment = environment;
        Debugger.println( "SocketServer", "bind", "Bounded at " + port );
        new Thread( this ).start();
    }

    public void close() {
        try {
            server.close();
            Debugger.println( "SocketServer", "close", "Closed!" );
            Socket socket = new Socket( "localhost", port );
            synchronized( this ) {
                Debugger.println( "SocketServer", "close", "wait" );
                try {
                    wait();
                }
                catch( InterruptedException exc ) {}
            }
        }
        catch( IOException ex ) {
            //ex.printStackTrace();
        }
        Debugger.println( "SocketServer", "close", "Finished" );
    }

    public void run() {
        int serialNumber = 0;
        while( true ) {
            try {
                Socket socket = server.accept();
                serialNumber++;
                new ExecuteThread( serialNumber, socket ).start();
            }
            catch( IOException exc ) {
                break;
            }
            finally {
                Debugger.println( "SocketServer", "run", "notifyAll()" );
                synchronized( this ) {
                    notifyAll();
                }
            }
        }
        Debugger.println( "SocketServer", "run", "Finished!" );
    }

    private class ExecuteThread extends Thread {
        private int serialNumber;
        private Socket socket;
        public ExecuteThread( int serialNumber, Socket socket ) {
            this.serialNumber = serialNumber;
            this.socket = socket;
        }

        public void run() {
            ObjectInputStream in = null;
            ObjectOutputStream out = null;
            try {
                try {
                    //open channels
                    in = new ObjectInputStream( socket.getInputStream() );
                    out = new ObjectOutputStream( socket.getOutputStream() );
                    String from = socket.getInetAddress().getHostAddress();
                    //receive
                    Expression expression = ( Expression ) in.readObject();
                    Debugger.println( "SocketThread",
                        "request(" + serialNumber + ")",
                        from != null ? from + ": " + String.valueOf( expression )
                        : String.valueOf( expression )
                        );
                    //execute
                    Expression result = null;
                    synchronized( environment ) {
                        result = expression.eval( environment );
                    }
                    Debugger.println( "SocketThread",
                        "result(" + serialNumber + ")",
                        from != null ? from + ": " + String.valueOf( result )
                        : String.valueOf( result )
                        );
                    //send
                    out.writeObject( result );
                    out.flush();
                }
                catch( ExecutionException exc ) {
                    if( out != null ) {
                        out.writeObject( exc );
                    }
                }
                catch( ClassNotFoundException exc ) {
                    if( out != null ) {
                        out.writeObject( exc );
                    }
                }
                catch( StreamCorruptedException exc ) {
                    if( out != null ) {
                        out.writeObject( exc );
                    }
                }
                catch( OptionalDataException exc ) {
                    if( out != null ) {
                        out.writeObject( exc );
                    }
                }
                catch( IOException exc ) {
                    if( out != null ) {
                        out.writeObject( exc );
                    }
                }
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            finally {
                try {
                    socket.shutdownInput();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
                try {
                    socket.shutdownOutput();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
                try {
                    socket.close();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
            }
        }
    }
}
