package jpel.remote.language.socket;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OptionalDataException;
import java.io.StreamCorruptedException;
import java.net.Socket;
import java.net.UnknownHostException;

import jpel.language.Environment;
import jpel.language.ExecutionException;
import jpel.language.Expression;
import jpel.language.ExpressionNumber;
import jpel.language.MapReplace;
import jpel.language.StringableExpression;
import jpel.language.ExpressionType;
import jpel.remote.language.ExpressionRemote;

/**
     * Define a execuo de uma expresso remotamente. Para a execuo remota deve-se
 * indicar o nome do servidor (address) e a expresso que deve ser avaliada.
 * Lembrando sempre qua apenas expresses serializveis obtero os resultados
 * corretos.
 */
public class ExpressionSocket extends ExpressionRemote {

    public ExpressionSocket( Expression host, Expression port,
        Expression expression ) {
        this( "Socket", host, port, expression );
    }

    public ExpressionSocket( String id, Expression host, Expression port,
        Expression expression ) {
        super( id, ExpressionType.UNKNOWN, host, port, expression );
    }

    public Expression rebuild( MapReplace map ) {
        MapReplace novo = map.mirror();
        return new ExpressionSocket(
            getName(),
            getHost().rebuild( novo ),
            getPort().rebuild( novo ),
            getExpression().rebuild( novo )
            );
    }

    public Expression eval( Environment environment ) throws
        ExecutionException {
        try {
            StringableExpression varHost = ( StringableExpression ) getHost().
                                           eval( environment );
            ExpressionNumber varPort = ( ExpressionNumber ) getPort().eval(
                                       environment );
            //open
            Socket socket = new Socket( varHost.asString(), varPort.getInt() );
            //send
            ObjectOutputStream out = new ObjectOutputStream( socket.
                                     getOutputStream() );
            out.writeObject( getExpression() );
            out.flush();
            //receive
            ObjectInputStream in = new ObjectInputStream( socket.getInputStream() );
            Expression result = ( Expression ) in.readObject();
            //finalize
            socket.shutdownInput();
            socket.shutdownOutput();
            socket.close();
            return result;
        }
        catch( ClassNotFoundException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( StreamCorruptedException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( OptionalDataException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( UnknownHostException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
        catch( IOException exc ) {
            throw new ExecutionException( this, exc.getMessage(), exc );
        }
    }
}
