package jpel.remote.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

import jpel.resolver.Configuration;
import jpel.resolver.ConfigurationException;
import jpel.util.Debugger;

public class ConfigurationClient implements Configuration {

    private String host;
    private int port;

    private Socket socket = null; // channel
    private ObjectOutputStream out = null; // send
    private ObjectInputStream in = null; // receive

    public ConfigurationClient( String host, int port ) {
        setHost( host );
        setPort( port );
    }

    public void setHost( String host ) {
        this.host = host;
    }

    public String getHost() {
        return host;
    }

    public void setPort( int port ) {
        this.port = port;
    }

    public int getPort() {
        return port;
    }

    public void setSocket( Socket socket ) {
        this.socket = socket;
    }

    public Socket getSocket() {
        return socket;
    }

    public void setOut( ObjectOutputStream out ) {
        this.out = out;
    }

    public ObjectOutputStream getOut() {
        return out;
    }

    public void setIn( ObjectInputStream in ) {
        this.in = in;
    }

    public ObjectInputStream getIn() {
        return in;
    }

    protected void openChannel() throws IOException {
        socket = new Socket( getHost(), getPort() );
        out = new ObjectOutputStream( socket.getOutputStream() );
        in = new ObjectInputStream( socket.getInputStream() );
    }

    protected void closeChannel() {
        if( out != null ) {
            try {
                out.close();
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            finally {
                out = null;
            }
        }
        if( in != null ) {
            try {
                in.close();
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            finally {
                in = null;
            }
        }
        if( socket != null ) {
            try {
                if( !socket.isClosed() ) {
                    socket.shutdownInput();
                }
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            try {
                if( !socket.isClosed() ) {
                    socket.shutdownOutput();
                }
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            try {
                socket.close();
            }
            catch( IOException exc ) {
                exc.printStackTrace();
            }
            finally {
                socket = null;
            }
        }
    }

    public void load( Object obj ) throws ConfigurationException {
        try {
            openChannel();
            out.writeUTF( ConfigurationServer.LOAD );
            out.writeObject( obj );
            out.flush();
            Object result = in.readObject();
            Debugger.println( "ConfigurationClient", "load", "Result:" + result );
        }
        catch( ClassNotFoundException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        catch( IOException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        finally {
            closeChannel();
        }
    }

    public Object getSource() throws ConfigurationException {
        try {
            openChannel();
            out.writeUTF( ConfigurationServer.SOURCE );
            out.flush();
            Object result = in.readObject();
            Debugger.println( "ConfigurationClient", "source",
                "Result:" + result );
            return result;
        }
        catch( ClassNotFoundException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        catch( IOException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        finally {
            closeChannel();
        }
    }

    public void reload() throws ConfigurationException {
        try {
            openChannel();
            out.writeUTF( ConfigurationServer.RELOAD );
            out.flush();
            Object result = in.readObject();
            Debugger.println( "ConfigurationClient", "reload",
                "Result:" + result );
        }
        catch( ClassNotFoundException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        catch( IOException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        finally {
            closeChannel();
        }
    }

    public Object process( Object source ) throws ConfigurationException {
        try {
            openChannel();
            out.writeUTF( ConfigurationServer.EXECUTE );
            out.writeObject( source );
            out.flush();
            Object result = in.readObject();
            Debugger.println( "ConfigurationClient", "process",
                "process(" + source + ")=" + result );
            return result;
        }
        catch( ClassNotFoundException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        catch( IOException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
        finally {
            closeChannel();
        }
    }
}
