package jpel.remote.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;

import jpel.remote.util.GenericSocketServer;
import jpel.remote.util.GenericSocketServerEvent;
import jpel.remote.util.GenericSocketServerException;
import jpel.remote.util.GenericSocketServerListener;
import jpel.resolver.Configuration;
import jpel.resolver.ConfigurationException;
import jpel.util.Debugger;

public class ConfigurationServer implements Configuration,
    GenericSocketServerListener {

    public static final String LOAD = "LOAD";
    public static final String SOURCE = "SOURCE";
    public static final String RELOAD = "RELOAD";
    public static final String EXECUTE = "EXECUTE";

    private Configuration configuration;

    public ConfigurationServer( int port,
        Configuration configuration ) throws ConfigurationException {
        try {
            GenericSocketServer server = new GenericSocketServer( port );
            server.addGenericConfigurationServerListener( this );
            setConfiguration( configuration );
        }
        catch( GenericSocketServerException exc ) {
            throw new ConfigurationException( exc.getMessage(), exc );
        }
    }

    public void setConfiguration( Configuration configuration ) {
        this.configuration = configuration;
    }

    public Configuration getConfiguration() {
        return configuration;
    }

    public void load( Object obj ) throws ConfigurationException {
        configuration.load( obj );
    }

    public Object getSource() throws ConfigurationException {
        return configuration.getSource();
    }

    public void reload() throws ConfigurationException {
        configuration.reload();
    }

    public Object process( Object command ) throws ConfigurationException {
        return configuration.process( command );
    }

    public void processCommand( GenericSocketServerEvent e ) {
        Debugger.println( "ConfigurationServer",
            e.serialNumber + ". " + e.command + " (" + e.from + ")", "Start" );
        try {
            if( LOAD.equals( e.command ) ) {
                Object obj = e.in.readObject();
                configuration.load( obj );
                String message = "Load '" + String.valueOf( obj ) + "' done.";
                e.out.writeObject( message );
                e.out.flush();
                Debugger.println( "ConfigurationServer",
                    e.serialNumber + ". " + e.command + " (" + e.from + ")",
                    message );
            }
            else if( SOURCE.equals( e.command ) ) {
                Object obj = e.in.readObject();
                Object source = configuration.getSource();
                String message = "Source '" + String.valueOf( obj ) + "' done.";
                e.out.writeObject( source );
                e.out.flush();
                Debugger.println( "ConfigurationServer",
                    e.serialNumber + ". " + e.command + " (" + e.from + ")",
                    message );
            }
            else if( RELOAD.equals( e.command ) ) {
                configuration.reload();
                String message = "Reload done.";
                e.out.writeObject( message );
                e.out.flush();
                Debugger.println( "ConfigurationServer",
                    e.serialNumber + ". " + e.command + " (" + e.from + ")",
                    message );
            }
            else if( EXECUTE.equals( e.command ) ) {
                Object input = e.in.readObject();
                Object output = configuration.process( input );
                e.out.writeObject( output );
                e.out.flush();
                Debugger.println( "ConfigurationServer",
                    e.serialNumber + ". " + e.command + " (" + e.from + ")",
                    input + " -> " + output );
            }
        }
        catch( Exception exc ) {
            try {
                e.out.writeObject( exc );
            }
            catch( IOException ioe ) {
                ioe.printStackTrace();
            }
        }
        Debugger.println( "ConfigurationServer",
            e.serialNumber + ". " + e.command + " (" + e.from + ")", "Finished" );
    }
}
