package jpel.remote.util;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Vector;

import jpel.util.Debugger;

public class GenericSocketServer implements Runnable {

    private int port;
    private ServerSocket server;
    transient private Vector genericConfigurationServerListeners;

    public GenericSocketServer( int port ) throws GenericSocketServerException {
        try {
            setPort( port );
            setServer( new ServerSocket( port ) );
            Debugger.println( "GenericConfigurationServer", "Bound",
                "Bounded at " + port );
            new Thread( this ).start();
        }
        catch( IOException exc ) {
            throw new GenericSocketServerException( exc.getMessage(), exc );
        }
    }

    public void setPort( int port ) {
        this.port = port;
    }

    public int getPort() {
        return port;
    }

    public void setServer( ServerSocket server ) {
        this.server = server;
    }

    public ServerSocket getServer() {
        return server;
    }

    public void run() {
        int serialNumber = 0;
        while( true ) {
            try {
                Socket socket = server.accept();
                serialNumber++;
                new ExecuteThread( this, serialNumber, socket ).start();
            }
            catch( IOException exc ) {
                break;
            }
        }
        Debugger.println( "GenericConfigurationServer", "run", "Finished!" );
    }

    public synchronized void removeGenericConfigurationServerListener(
        GenericSocketServerListener l ) {
        if( genericConfigurationServerListeners != null
            && genericConfigurationServerListeners.contains( l ) ) {
            Vector v = ( Vector ) genericConfigurationServerListeners.clone();
            v.removeElement( l );
            genericConfigurationServerListeners = v;
        }
    }

    public synchronized void addGenericConfigurationServerListener(
        GenericSocketServerListener l ) {
        Vector v = genericConfigurationServerListeners == null ? new Vector( 2 )
                   : ( Vector ) genericConfigurationServerListeners.clone();
        if( !v.contains( l ) ) {
            v.addElement( l );
            genericConfigurationServerListeners = v;
        }
    }

    private class ExecuteThread extends Thread {
        private GenericSocketServer server;
        private int serialNumber;
        private Socket socket;
        public ExecuteThread( GenericSocketServer server, int serialNumber,
            Socket socket ) {
            this.server = server;
            this.serialNumber = serialNumber;
            this.socket = socket;
        }

        public void run() {
            ObjectInputStream in = null;
            ObjectOutputStream out = null;
            try {
                in = new ObjectInputStream( socket.getInputStream() );
                out = new ObjectOutputStream( socket.getOutputStream() );
                String from = socket.getInetAddress().getHostName();
                GenericSocketServerEvent event;
                event = new GenericSocketServerEvent( server, serialNumber,
                        in.readUTF(), from, in, out );
                fireProcessCommand( event );
            }
            catch( IOException exc ) {
                exc.printStackTrace();
                try {
                    out.writeObject( exc );
                }
                catch( IOException exc1 ) {}
            }
            finally {
                if( in != null ) {
                    try {
                        in.close();
                    }
                    catch( IOException exc ) {}
                    finally {
                        in = null;
                    }
                }
                if( out != null ) {
                    try {
                        out.close();
                    }
                    catch( IOException exc ) {}
                    finally {
                        out = null;
                    }
                }
                try {
                    socket.shutdownInput();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
                try {
                    socket.shutdownOutput();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
                try {
                    socket.close();
                }
                catch( IOException exc ) {
                    exc.printStackTrace();
                }
                finally {
                    socket = null;
                }
            }
        }
    }

    protected void fireProcessCommand( GenericSocketServerEvent e ) {
        if( genericConfigurationServerListeners != null ) {
            Vector listeners = genericConfigurationServerListeners;
            int count = listeners.size();
            for( int i = 0; i < count; i++ ) {
                ( ( GenericSocketServerListener ) listeners.elementAt( i ) ).
                    processCommand( e );
            }
        }
    }
}
