package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Fornece informaes de como deve ser feito o mapeamento entre objetos, seus
 * mtodos e seus parametros.
 */
public class Bind {

    private Object target;
    private String method;
    private Class[] types;
    private String[] commands;
    private Object[] oldValues;
    private Object[] newValues;

    /**
     * Cria um Bind.
     * @param target Objeto onde deve ser chamado o procedimento.
     * @param method Nome do mtodo que deve ser chamado.
     * @param types Tipos dos parmetros dos metodos
     * @param command Chamada da configurao
     * @param oldValue Objeto antigo usado como argumento do mtodo
     * @param newValue Objeto novo usado como argumento do mtodo
     * que devem ser utilizado na chamada de mtodo.
     */
    public Bind(Object target, String method, Class[] types, String[] command,
        Object[] oldValue, Object[] newValue) {
        setTarget(target);
        setMethod(method);
        setTypes(types);
        setCommands(command);
        setNewValues(newValue);
        setOldValues(oldValue);
    }

    public void setTarget(Object target) {
        this.target = target;
    }

    public Object getTarget() {
        return target;
    }

    public void setMethod(String method) {
        this.method = method;
    }

    public String getMethod() {
        return method;
    }

    public void setTypes(Class[] types) {
        this.types = types;
    }

    public Class[] getTypes() {
        return types;
    }

    public void setCommands(String[] commands) {
        this.commands = commands;
    }

    public String[] getCommands() {
        return commands;
    }

    public void setOldValues(Object[] oldValues) {
        this.oldValues = oldValues;
    }

    public Object[] getOldValues() {
        return oldValues;
    }

    public void setNewValues(Object[] newValues) {
        this.newValues = newValues;
    }

    public Object[] getNewValues() {
        return newValues;
    }

    public String toString() {
        StringBuffer str = new StringBuffer();
        str.append("Bind(" + target);
        str.append("," + method);
        if(types != null) {
            str.append("," + array(types));
        }
        if(commands != null) {
            str.append("," + array(commands));
        }
        if(newValues != null) {
            str.append("," + array(newValues));
        }
        if(oldValues != null) {
            str.append("," + array(oldValues));
        }
        return str.toString();
    }

    private String array(Object[] objs) {
        if(objs == null) {
            return "";
        }
        StringBuffer str = new StringBuffer();
        for(int i = 0; i < objs.length; i++) {
            str.append((i == 0 ? "" : ",") + objs[i]);
        }
        return str.toString();
    }
}