package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.beans.Statement;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import jpel.language.Expression;

/**
 * Implementao padro do Binder.
 */

public class BinderImpl implements Binder {

    private Object[] giveParameters(Class[] types, Object[] values) {
        Object[] parameters = new Object[types.length];
        for(int i = 0; i < types.length; i++) {
            Expression value = (Expression) values[i];
            parameters[i] = value.getData(types[i]);
        }
        return parameters;
    }

    public void validate(Bind bind) throws BinderException {
        try {
            Object target = bind.getTarget();
            synchronized(target) {
                Class c = target.getClass();
                String method = bind.getMethod();
                Class[] types = bind.getTypes();
                Method m = c.getMethod(method, types);
            }
        }
        catch(NoSuchMethodException exc) {
            throw new BinderException(exc.getMessage(), exc);
        }
    }

    public void execute(Bind bind) throws BinderException {
        try {
            Object[] parameters = giveParameters(bind.getTypes(),
                                  bind.getNewValues());
            Statement st = new Statement(bind.getTarget(), bind.getMethod(),
                           parameters);
            st.execute();
        }
        catch(IllegalAccessException ex) {
            throw new BinderException(ex.getMessage(), ex);
        }
        catch(InvocationTargetException ex) {
            throw new BinderException(ex.getMessage(), ex);
        }
        catch(NoSuchMethodException ex) {
            throw new BinderException(ex.getMessage(), ex);
        }
        catch(Exception ex) {
            throw new BinderException(ex.getMessage(), ex);
        }
    }
}