package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Define uma configurao de sistema.
 */
public interface Configuration {

    /**
     * Carrega uma configurao.
         * @param source Objeto com os dados, ou meta-dados, para carregar um ambiente.
         * @throws ConfigurationException Quando h problemas na leitura da configurao.
     */
    void load(Object source) throws ConfigurationException;

    /**
     * Retorna o objeto que descreve o conjunto de dados.
     * @return O objeto com as informaes sobre onde os parmetros esto.
     * @throws ConfigurationException Quando no  possvel identificar a fonte
     * do dados.
     */
    Object getSource() throws ConfigurationException;

    /**
     * Recarrega todos os recursos usados pela configurao.
         * @throws ConfigurationException Quando no foi possvel realizar o reload().
     */
    void reload() throws ConfigurationException;

    /**
     * Processa um objeto.
     * @param source Objeto a ser processado.
     * @return Objeto resultado de uma processamento sobre a entrada.
         * @throws ConfigurationException Quando houve erro no processamento do objeto.
     */
    Object process(Object source) throws ConfigurationException;
}