package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.Environment;
import jpel.language.EnvironmentBuilder;
import jpel.language.EnvironmentLoader;
import jpel.language.Expression;
import jpel.language.ExpressionBuilder;
import jpel.language.ExpressionParser;

public abstract class ConfigurationImpl implements Configuration {

    private Object source;
    private EnvironmentLoader environmentLoader;
    private Environment environment;
    private ExpressionParser expressionParser;

    public void setSource(Object source) {
        this.source = source;
    }

    public Object getSource() {
        return source;
    }

    public void setEnvironmentLoader(EnvironmentLoader environmentLoader) {
        this.environmentLoader = environmentLoader;
    }

    public EnvironmentLoader getEnvironmentLoader() {
        return environmentLoader;
    }

    public void setEnvironment(Environment environment) {
        this.environment = environment;
    }

    public Environment getEnvironment() {
        return environment;
    }

    public void setExpressionParser(ExpressionParser expressionParser) {
        this.expressionParser = expressionParser;
    }

    public ExpressionParser getExpressionParser() {
        return expressionParser;
    }

    public void load(Object obj) throws ConfigurationException {
        try {
            setSource(obj);
            setEnvironmentLoader(EnvironmentBuilder.lookupEnvironmentLoader(
                source.getClass().getName()));
            setEnvironment(getEnvironmentLoader().load(getSource()));
            setExpressionParser(ExpressionBuilder.getExpressionParser());
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public void reload() throws ConfigurationException {
        load(getSource());
    }

    public Object process(Object source) throws ConfigurationException {
        try {
            Expression exp = expressionParser.getExpression(String.valueOf(
                             source));
            return exp.eval(getEnvironment());
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }
}
