package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

/**
 * Define uma configurao dinamica. Neste tipo de conmfigurao, sempre que 
 * executado um reload todas as chamadas de parametros so reavaliadas e os
 * objetos que utilizam esses parametros so reajustados de acordo com a nova
 * configurao.
 * IMPORTANTE: Durante o reload da configurao a re-execuo dos rebinds 
 * efetuada na mesma ordem em que foram executados os binds.
 */
public interface DynamicConfiguration extends StaticConfiguration {

    /**
     * Mtodo com muitos parmetros.
     * <p>
     * Associa determinado valor a um objeto utilizando um mtodo. Nos casos de
         * reload dinmico apenas os valores ajustados desta forma sero beneficiados.
         * @param target Objeto alvo. Ex: Em "Client c = new Client();", a varivel c.
     * @param method Mtodo usado para ajustar o valor. Ex: setName
         * @param types Tipos dos argumentos do mtodo. Ex: {String.class,int.class}
         * @param commands Valores dos atributos usados. Ex: {"user.name","user.id"}
         * @throws ConfigurationException Quando no foi possvel realizar a associao.
     */
    void bind(Object target, String method, Class[] types,
        String[] commands) throws ConfigurationException;

    /**
     * Mtodo com um nico parmetro.
     * <p>
     * Associa determinado valor a um objeto utilizando um mtodo. Nos casos de
         * reload dinmico apenas os valores ajustados desta forma sero beneficiados.
         * @param target Objeto alvo. Ex: Em "Client c = new Client();", a varivel c.
     * @param method Mtodo usado para ajustar o valor. Ex: setName
     * @param type Tipos dos argumentos do mtodo. Ex: String.class
     * @param command Valor do atributo usado. Ex: user.name
         * @throws ConfigurationException Quando no foi possvel realizar a associao.
     */
    void bind(Object target, String method, Class type,
        String command) throws ConfigurationException;

    /**
     * Mtodo sem parmetros.
     * <p>
     * Define um mtodo sem parametros que deve ser executado. Sempre que a
     * configurao dinamic for recarregada. Abreviao para bind(target,method,null,null).
     * @param target Objeto alvo.
     * @param method Mtodo a ser chamado.
         * @throws ConfigurationException Quando no foi possvel realizar a associao.
     */
    void bind(Object target, String method) throws ConfigurationException;

    /**
         * Aps a realizao de todos os binds, este mtodo deve ser invocado para a
     * realizao dos ajustes dos objetos.
         * @throws ConfigurationException Quando  erros durante a execuo dos binds.
     */
    void execute() throws ConfigurationException;
}