package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import jpel.util.Debugger;

/**
 * Implementao referncia da DynamicConfiguration.
 */
public class DynamicConfigurationImpl extends StaticConfigurationImpl implements
    DynamicConfiguration {

    private Binder binder;
    private List list;

    public DynamicConfigurationImpl() throws ConfigurationException {
        super();
        try {
            binder = new BinderImpl();
            list = new LinkedList();
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
        Debugger.println("Dynamic", "init", "Finished!");
    }

    public void setBinder(Binder binder) {
        this.binder = binder;
    }

    public Binder getBinder() {
        return binder;
    }

    public void setList(List list) {
        this.list = list;
    }

    public List getList() {
        return list;
    }

    public void load(Object obj) throws ConfigurationException {
        Debugger.println("Dynamic", "load", "Started");
        super.load(obj);
        clear();
        Debugger.println("Dynamic", "load", "Finished");
    }

    protected void clear() {
        list = new LinkedList();
    }

    public void reload() throws ConfigurationException {
        Debugger.println("Dynamic", "reload", "Started");
        List backup = list;
        super.reload();
        list = backup;
        rebind();
        execute();
        Debugger.println("Dynamic", "reload", "Finished");
    }

    protected void rebind() throws ConfigurationException {
        Iterator iter = list.iterator();
        while(iter.hasNext()) {
            Bind b = (Bind) iter.next();
            Debugger.println("Dynamic", "rebind", "Bind :" + b);
            bind(b, false);
            Debugger.println("Dynamic", "rebind", "Rebound to :" + b);
        }
    }

    public void bind(Object target, String method, Class[] types,
        String[] commands) throws ConfigurationException {
        bind(new Bind(target, method, types, commands, null, null), true);
    }

    public void bind(Object target, String method, Class type,
        String command) throws ConfigurationException {
        bind(target, method, new Class[] {
            type}
            , new String[] {
            command});
    }

    public void bind(Object target,
        String method) throws ConfigurationException {
        bind(target, method, new Class[] {}
            , new String[] {});
    }

    public void execute() throws ConfigurationException {
        Iterator iter = list.iterator();
        List undo = new LinkedList();
        try {
            while(iter.hasNext()) {
                Bind b = (Bind) iter.next();
                Debugger.println("Dynamic", "execute", "Bind :" + b);
                binder.execute(b);
                undo.add(b);
                Debugger.println("Dynamic", "execute", "Bind done.");
            }
        }
        catch(BinderException exc) {
            rollback(undo);
            throw new ConfigurationException(exc.getMessage(), exc);
        }
        catch(Exception exc) { // caso apareca outro tipo de erro qualquer
            rollback(undo); // como IllegalArgumentException.
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    private void rollback(List rollback) throws ConfigurationException {
        Iterator iter = rollback.iterator();
        while(iter.hasNext()) {
            Bind b = (Bind) iter.next();
            Debugger.println("Dynamic", "rollback", "Rollback :" + b);
            Object[] beforeValues = b.getOldValues();
            if(beforeValues != null) {
                b.setNewValues(beforeValues);
            }
            b.setOldValues(null);
            try {
                binder.execute(b);
            }
            catch(BinderException exc) {
                throw new ConfigurationException("Could not rollback " + b,
                    exc);
            }
            Debugger.println("Dynamic", "rollback", "Rollback done." + b);
        }
    }

    private void bind(Bind bind,
        boolean append) throws ConfigurationException {
        if(bind.getCommands() != null) {
            String[] commands = bind.getCommands();
            Object[] newValues = new Object[commands.length];
            for(int i = 0; i < commands.length; i++) {
                newValues[i] = process(commands[i]); //retorna objetos Expression
            }
            bind.setOldValues(bind.getNewValues());
            bind.setNewValues(newValues);
        }
        try {
            binder.validate(bind);
        }
        catch(BinderException exc) {
            throw new ConfigurationException("Invalid bind '" + bind + ". "
                + exc.getMessage(), exc);
        }
        if(append) {
            list.add(bind);
        }
    }
}