package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.util.Iterator;

import jpel.language.BadTypedException;
import jpel.language.Expression;
import jpel.language.ExpressionBoolean;
import jpel.language.ExpressionCharacter;
import jpel.language.ExpressionList;
import jpel.language.ExpressionNumber;
import jpel.language.StringableExpression;

/**
 * Implementao padro do interpretador.
 */
public class InterpreterImpl implements Interpreter {

    public InterpreterImpl() {}

    public boolean toBoolean(Expression expression) throws BadTypedException {
        if(!expression.getType().isBoolean()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a boolean value.");
        }
        return((ExpressionBoolean) expression).getBoolean();
    }

    public Boolean toBooleanObject(Expression expression) throws
        BadTypedException {
        return toBoolean(expression) ? Boolean.TRUE : Boolean.FALSE;
    }

    public boolean[] toBooleanArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        boolean[] result = new boolean[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toBoolean(exp);
        }
        return result;
    }

    public Boolean[] toBooleanObjectArray(Expression expression) throws
        BadTypedException {
        boolean[] tmp = toBooleanArray(expression);
        Boolean[] result = new Boolean[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            if(tmp[i]) {
                result[i] = Boolean.TRUE;
            }
            else {
                result[i] = Boolean.FALSE;
            }
        }
        return result;
    }

    public char toChar(Expression expression) throws BadTypedException {
        if(!expression.getType().isCharacter()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a character value.");
        }
        return((ExpressionCharacter) expression).getChar();
    }

    public Character toCharObject(Expression expression) throws
        BadTypedException {
        return new Character(toChar(expression));
    }

    public char[] toCharArray(Expression expression) throws BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        char[] result = new char[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toChar(exp);
        }
        return result;
    }

    public Character[] toCharObjectArray(Expression expression) throws
        BadTypedException {
        char[] tmp = toCharArray(expression);
        Character[] result = new Character[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Character(tmp[i]);
        }
        return result;
    }

    public byte toByte(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getByte();
    }

    public Byte toByteObject(Expression expression) throws BadTypedException {
        return new Byte(toByte(expression));
    }

    public byte[] toByteArray(Expression expression) throws BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        byte[] result = new byte[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toByte(exp);
        }
        return result;
    }

    public Byte[] toByteObjectArray(Expression expression) throws
        BadTypedException {
        byte[] tmp = toByteArray(expression);
        Byte[] result = new Byte[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Byte(tmp[i]);
        }
        return result;
    }

    public short toShort(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getShort();
    }

    public Short toShortObject(Expression expression) throws
        BadTypedException {
        return new Short(toShort(expression));
    }

    public short[] toShortArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        short[] result = new short[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toShort(exp);
        }
        return result;
    }

    public Short[] toShortObjectArray(Expression expression) throws
        BadTypedException {
        short[] tmp = toShortArray(expression);
        Short[] result = new Short[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Short(tmp[i]);
        }
        return result;
    }

    public int toInt(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getInt();
    }

    public Integer toIntObject(Expression expression) throws
        BadTypedException {
        return new Integer(toInt(expression));
    }

    public int[] toIntArray(Expression expression) throws BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        int[] result = new int[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toInt(exp);
        }
        return result;
    }

    public Integer[] toIntObjectArray(Expression expression) throws
        BadTypedException {
        int[] tmp = toIntArray(expression);
        Integer[] result = new Integer[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Integer(tmp[i]);
        }
        return result;
    }

    public long toLong(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getLong();
    }

    public Long toLongObject(Expression expression) throws BadTypedException {
        return new Long(toLong(expression));
    }

    public long[] toLongArray(Expression expression) throws BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        long[] result = new long[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toLong(exp);
        }
        return result;
    }

    public Long[] toLongObjectArray(Expression expression) throws
        BadTypedException {
        long[] tmp = toLongArray(expression);
        Long[] result = new Long[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Long(tmp[i]);
        }
        return result;
    }

    public float toFloat(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getFloat();
    }

    public Float toFloatObject(Expression expression) throws
        BadTypedException {
        return new Float(toFloat(expression));
    }

    public float[] toFloatArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        float[] result = new float[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toFloat(exp);
        }
        return result;
    }

    public Float[] toFloatObjectArray(Expression expression) throws
        BadTypedException {
        float[] tmp = toFloatArray(expression);
        Float[] result = new Float[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Float(tmp[i]);
        }
        return result;
    }

    public double toDouble(Expression expression) throws BadTypedException {
        if(!expression.getType().isNumber()) {
            throw new BadTypedException(expression,
                expression.getClass().getName() + " is not a number.");
        }
        return((ExpressionNumber) expression).getDouble();
    }

    public Double toDoubleObject(Expression expression) throws
        BadTypedException {
        return new Double(toDouble(expression));
    }

    public double[] toDoubleArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        double[] result = new double[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toDouble(exp);
        }
        return result;
    }

    public Double[] toDoubleObjectArray(Expression expression) throws
        BadTypedException {
        double[] tmp = toDoubleArray(expression);
        Double[] result = new Double[tmp.length];
        for(int i = 0; i < tmp.length; i++) {
            result[i] = new Double(tmp[i]);
        }
        return result;
    }

    public String toString(Expression expression) throws BadTypedException {
        if(!(expression instanceof StringableExpression)) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a String value.");
        }
        return((StringableExpression) expression).asString();
    }

    public String[] toStringArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        String[] result = new String[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toString(exp);
        }
        return result;
    }

    public Object toObject(Expression expression) throws BadTypedException {
        return expression.getData();
    }

    public Object[] toObjectArray(Expression expression) throws
        BadTypedException {
        if(!expression.getType().isList()) {
            throw new BadTypedException(expression,
                expression.getClass().getName()
                + " cannot be converted to a list value.");
        }
        ExpressionList list = (ExpressionList) expression;
        Object[] result = new Object[(int) list.length()];
        Expression exp;
        Iterator nav = list.iterator();
        for(int i = 0; nav.hasNext(); i++) {
            exp = (Expression) nav.next();
            result[i] = toObject(exp);
        }
        return result;
    }
}