package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import jpel.util.Debugger;

/**
 * Define uma politica que notifica os listeners quando recebe uma solicitao.
 */
public class PolicyActivateOnNotify extends PolicyImpl implements Runnable {

    private Thread t;
    private ServerSocket server;
    private int port = -1;

    private List configurations;

    public PolicyActivateOnNotify() {
        configurations = Collections.synchronizedList(new LinkedList());
    }

    public synchronized void setPort(int port) {
        this.port = port;
        notifyAll();
    }

    public int getPort() {
        return port;
    }

    public void addConfiguration(Configuration conf) throws PolicyException {
        configurations.add(conf);
    }

    public void removeConfiguration(Configuration conf) throws
        PolicyException {
        int index = configurations.indexOf(conf);
        configurations.remove(index);
    }

    public void finish() {
        if(server != null) {
            try {
                server.close();
            }
            catch(IOException exc) {
                Debugger.println("ActivateOnNotify", "finish",
                    "Finalizing server '" + port + "'");
            }
            server = null;
        }
        if(t != null) {
            try {
                new Socket(InetAddress.getLocalHost(), port).close();
            }
            catch(IOException exc) {
                Debugger.println("ActivateOnNotify", "finish",
                    "Finalizing thread '" + port + "'");
            }
            t = null;
        }
    }

    public void start() {
        finish();
        if(t == null) {
            t = new Thread(this);
            t.start();
            Debugger.println("ActivateOnNotify", "start", "Thread started.");
        }
    }

    public synchronized void run() {
        boolean ok = true;
        if(port < 0) {
            try {
                wait();
            }
            catch(InterruptedException ex) {
                ex.printStackTrace();
            }
        }
        if(server == null) {
            try {
                server = new ServerSocket(port);
                Debugger.println("ActivateOnNotify", "start",
                    "Server created on port " + port + ".");
            }
            catch(IOException exc) {
                Debugger.println("ActivateOnNotify", "start",
                    "Server could not be started at " + port + ".");
            }
        }
        while(ok) {
            try {
                Socket s = server.accept();
                callAll();
                s.close();
            }
            catch(IOException ex) {
                ex.printStackTrace();
                ok = false;
            }
        }
    }

    public void callAll() {
        Iterator iter = configurations.iterator();
        while(iter.hasNext()) {
            Object item = iter.next();
            firePolicyActivated(new PolicyEvent((Configuration) item));
        }
        Debugger.println("ActivateOnNotify", "call", "Called all reload.");
    }
}