package jpel.resolver;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import jpel.language.Expression;
import jpel.util.Debugger;

/**
 * Implementao referncia static DynamicConfiguration.
 */
public class StaticConfigurationImpl extends ConfigurationImpl implements
    StaticConfiguration {

    private Interpreter interpreter;

    public StaticConfigurationImpl() throws ConfigurationException {
        try {
            interpreter = new InterpreterImpl();
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
        Debugger.println("Static", "init", "Finished!");
    }

    public void setInterpreter(Interpreter interpreter) {
        this.interpreter = interpreter;
    }

    public Interpreter getInterpreter() {
        return interpreter;
    }

// booleans
    public boolean getBoolean(String command) throws ConfigurationException {
        try {
            return interpreter.toBoolean((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Boolean getBooleanObject(String command) throws
        ConfigurationException {
        try {
            return interpreter.toBooleanObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public boolean getBoolean(String command, boolean defaultValue) {
        try {
            return getBoolean(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Boolean getBooleanObject(String command, Boolean defaultValue) {
        try {
            return getBooleanObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public boolean[] getBooleanArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toBooleanArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Boolean[] getBooleanObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toBooleanObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public boolean[] getBooleanArray(String command, boolean[] defaultValue) {
        try {
            return getBooleanArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Boolean[] getBooleanObjectArray(String command,
        Boolean[] defaultValue) {
        try {
            return getBooleanObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// chars
    public char getChar(String command) throws ConfigurationException {
        try {
            return interpreter.toChar((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Character getCharObject(String command) throws
        ConfigurationException {
        try {
            return interpreter.toCharObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public char getChar(String command, char defaultValue) {
        try {
            return getChar(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Character getCharObject(String command, Character defaultValue) {
        try {
            return getCharObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public char[] getCharArray(String command) throws ConfigurationException {
        try {
            return interpreter.toCharArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Character[] getCharObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toCharObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public char[] getCharArray(String command, char[] defaultValue) {
        try {
            return getCharArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Character[] getCharObjectArray(String command,
        Character[] defaultValue) {
        try {
            return getCharObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// bytes
    public byte getByte(String command) throws ConfigurationException {
        try {
            return interpreter.toByte((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Byte getByteObject(String command) throws ConfigurationException {
        try {
            return interpreter.toByteObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public byte getByte(String command, byte defaultValue) {
        try {
            return getByte(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Byte getByteObject(String command, Byte defaultValue) {
        try {
            return getByteObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public byte[] getByteArray(String command) throws ConfigurationException {
        try {
            return interpreter.toByteArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Byte[] getByteObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toByteObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public byte[] getByteArray(String command, byte[] defaultValue) {
        try {
            return getByteArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Byte[] getByteObjectArray(String command, Byte[] defaultValue) {
        try {
            return getByteObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// shorts
    public short getShort(String command) throws ConfigurationException {
        try {
            return interpreter.toShort((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Short getShortObject(String command) throws ConfigurationException {
        try {
            return interpreter.toShortObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public short getShort(String command, short defaultValue) {
        try {
            return getShort(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Short getShortObject(String command, Short defaultValue) {
        try {
            return getShortObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public short[] getShortArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toShortArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Short[] getShortObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toShortObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public short[] getShortArray(String command, short[] defaultValue) {
        try {
            return getShortArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Short[] getShortObjectArray(String command, Short[] defaultValue) {
        try {
            return getShortObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// ints
    public int getInt(String command) throws ConfigurationException {
        try {
            return interpreter.toInt((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Integer getIntObject(String command) throws ConfigurationException {
        try {
            return interpreter.toIntObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public int getInt(String command, int defaultValue) {
        try {
            return getInt(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Integer getIntObject(String command, Integer defaultValue) {
        try {
            return getIntObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public int[] getIntArray(String command) throws ConfigurationException {
        try {
            return interpreter.toIntArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Integer[] getIntObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toIntObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public int[] getIntArray(String command, int[] defaultValue) {
        try {
            return getIntArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Integer[] getIntObjectArray(String command, Integer[] defaultValue) {
        try {
            return getIntObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// longs
    public long getLong(String command) throws ConfigurationException {
        try {
            return interpreter.toLong((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Long getLongObject(String command) throws ConfigurationException {
        try {
            return interpreter.toLongObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public long getLong(String command, long defaultValue) {
        try {
            return getLong(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Long getLongObject(String command, Long defaultValue) {
        try {
            return getLongObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public long[] getLongArray(String command) throws ConfigurationException {
        try {
            return interpreter.toLongArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Long[] getLongObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toLongObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public long[] getLongArray(String command, long[] defaultValue) {
        try {
            return getLongArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Long[] getLongObjectArray(String command, Long[] defaultValue) {
        try {
            return getLongObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// float
    public float getFloat(String command) throws ConfigurationException {
        try {
            return interpreter.toFloat((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Float getFloatObject(String command) throws ConfigurationException {
        try {
            return interpreter.toFloatObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public float getFloat(String command, float defaultValue) {
        try {
            return getFloat(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Float getFloatObject(String command, Float defaultValue) {
        try {
            return getFloatObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public float[] getFloatArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toFloatArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Float[] getFloatObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toFloatObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public float[] getFloatArray(String command, float[] defaultValue) {
        try {
            return getFloatArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Float[] getFloatObjectArray(String command, Float[] defaultValue) {
        try {
            return getFloatObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// double
    public double getDouble(String command) throws ConfigurationException {
        try {
            return interpreter.toDouble((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Double getDoubleObject(String command) throws
        ConfigurationException {
        try {
            return interpreter.toDoubleObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public double getDouble(String command, double defaultValue) {
        try {
            return getDouble(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Double getDoubleObject(String command, Double defaultValue) {
        try {
            return getDoubleObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public double[] getDoubleArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toDoubleArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Double[] getDoubleObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toDoubleObjectArray((Expression) process(
                command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public double[] getDoubleArray(String command, double[] defaultValue) {
        try {
            return getDoubleArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Double[] getDoubleObjectArray(String command, Double[] defaultValue) {
        try {
            return getDoubleObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// Strings
    public String getString(String command) throws ConfigurationException {
        try {
            return interpreter.toString((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public String getString(String command, String defaultValue) {
        try {
            return getString(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public String[] getStringArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toStringArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public String[] getStringArray(String command, String[] defaultValue) {
        try {
            return getStringArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

// objects
    public Object getObject(String command) throws ConfigurationException {
        try {
            return interpreter.toObject((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Object getObject(String command, Object defaultValue) {
        try {
            return getObject(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }

    public Object[] getObjectArray(String command) throws
        ConfigurationException {
        try {
            return interpreter.toObjectArray((Expression) process(command));
        }
        catch(Exception exc) {
            throw new ConfigurationException(exc.getMessage(), exc);
        }
    }

    public Object[] getObjectArray(String command, Object[] defaultValue) {
        try {
            return getObjectArray(command);
        }
        catch(ConfigurationException exc) {
            return defaultValue;
        }
    }
}