package jpel.tree;

/**
 * <p>Title: Configuration in Java</p>
 * <p>Description: Ferramentas e tcnicas para ajuste de sistemas feitos em Java.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: CIN - UFPE</p>
 * @author Thiago Santos (tlvls@cin.ufpe.br)
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import jpel.util.Debugger;
import jpel.util.dataholder.DataHolder;

public class NodeBuilder {

    private static final String JVM_PARAMETER = "nodeFactory";
    private static final String COMMENT = "#";
    private static final String TYPE_MARK = "::";
    private static final String TYPE_TOKEN = ",";

    private static HashMap readers;
    private static HashMap writers;
    private static HashMap processors;
    private static HashMap conversors;
    private static HashMap matchers;

    static {
        readers = new HashMap();
        writers = new HashMap();
        processors = new HashMap();
        conversors = new HashMap();
        matchers = new HashMap();
        try {
            InputStream in;
            BufferedReader bin;
            String input;

            in = NodeBuilder.class.getResourceAsStream("readers.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindNodeReader(types, str);
                }
                else {
                    bindNodeReader(str);
                }
            }
            bin.close();
            in.close();

            in = NodeBuilder.class.getResourceAsStream("writers.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindNodeWriter(types, str);
                }
                else {
                    bindNodeWriter(str);
                }
            }
            bin.close();
            in.close();

            in = NodeBuilder.class.getResourceAsStream("processors.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindNodeProcessor(types, str);
                }
                else {
                    bindNodeProcessor(str);
                }
            }
            bin.close();
            in.close();

            in = NodeBuilder.class.getResourceAsStream("conversors.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindNodeConversor(types, str);
                }
                else {
                    bindNodeConversor(str);
                }
            }
            bin.close();
            in.close();

            in = NodeBuilder.class.getResourceAsStream("matchers.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindNodeMatcher(types, str);
                }
                else {
                    bindNodeMatcher(str);
                }
            }
            bin.close();
            in.close();
        }
        catch(Exception exc) {
            exc.printStackTrace();
            System.exit(0);
        }
    }

    public static NodeFactory getNodeFactory() throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        String classType = System.getProperty(JVM_PARAMETER,
                           "jpel.tree.NodeFactoryImpl");
        return getNodeFactory(classType);
    }

    public static NodeFactory getNodeFactory(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeFactory factory = (NodeFactory) Class.forName(classType).
                              newInstance();
        Debugger.println("NodeBuilder", "factory",
            "Class>" + factory.getClass().getName());
        return factory;
    }

    public static void bindNodeReader(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeReader reader = (NodeReader) Class.forName(classType).
                            newInstance();
        bindNodeReader(reader);
    }

    public static void bindNodeReader(NodeReader reader) {
        bindNodeReader(reader.getTypes(), reader);
    }

    public static void bindNodeReader(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        NodeReader reader = (NodeReader) Class.forName(classType).
                            newInstance();
        bindNodeReader(types, reader);
    }

    public static void bindNodeReader(String[] types, NodeReader reader) {
        for(int i = 0; i < types.length; i++) {
            readers.put(types[i], reader);
            Debugger.println("NodeBuilder", "bindReader",
                "Type> " + types[i] + ", Class>" + reader.getClass().getName());
        }
    }

    public static NodeReader lookupNodeReader(DataHolder holder) throws
        ClassNotFoundException {
        String type = null;
        String name = null;
        if(holder.getType() == DataHolder.FILE) {
            File file = (File) holder.getReference();
            name = file.getName();
        }
        else if(holder.getType() == DataHolder.URL) {
            URL url = (URL) holder.getReference();
            name = url.getFile();
        }
        else {
            throw new ClassNotFoundException("There is not a reader to a '"
                + holder.getClass().getName() + "'");
        }
        int pos = name.lastIndexOf('.');
        if(pos < 0) {
            throw new ClassNotFoundException(
                "Could not determine the holder type. holder=" + holder);
        }
        type = name.substring(pos + 1);
        return lookupNodeReader(type);
    }

    public static NodeReader lookupNodeReader(String type) throws
        ClassNotFoundException {
        NodeReader reader = (NodeReader) readers.get(type);
        if(reader == null) {
            throw new ClassNotFoundException(
                "There is not a Reader bound to '" + type + "'.");
        }
        Debugger.println("NodeBuilder", "lookupReader",
            "Type> " + type + ", Class>" + reader.getClass().getName());
        return reader;
    }

    public static Iterator readerTypes() {
        return readers.keySet().iterator();
    }

    public static void bindNodeWriter(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeWriter writer = (NodeWriter) Class.forName(classType).
                            newInstance();
        bindNodeWriter(writer);
    }

    public static void bindNodeWriter(NodeWriter writer) {
        bindNodeWriter(writer.getTypes(), writer);
    }

    public static void bindNodeWriter(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        NodeWriter writer = (NodeWriter) Class.forName(classType).
                            newInstance();
        bindNodeWriter(types, writer);
    }

    public static void bindNodeWriter(String[] types, NodeWriter writer) {
        for(int i = 0; i < types.length; i++) {
            writers.put(types[i], writer);
            Debugger.println("NodeBuilder", "bindWriter",
                "Type> " + types[i] + ", Class>" + writer.getClass().getName());
        }
    }

    public static NodeWriter lookupNodeWriter(DataHolder holder) throws
        ClassNotFoundException {
        String type = null;
        String name = null;
        if(holder.getType() == DataHolder.FILE) {
            File file = (File) holder.getReference();
            name = file.getName();
        }
        else if(holder.getType() == DataHolder.URL) {
            URL url = (URL) holder.getReference();
            name = url.getFile();
        }
        else {
            throw new ClassNotFoundException("There is not a writer to a '"
                + holder.getClass().getName() + "'");
        }
        int pos = name.lastIndexOf('.');
        if(pos < 0) {
            throw new ClassNotFoundException(
                "Could not determine the holder type. holder=" + holder);
        }
        type = name.substring(pos + 1);
        return lookupNodeWriter(type);
    }

    public static NodeWriter lookupNodeWriter(String type) throws
        ClassNotFoundException {
        NodeWriter writer = (NodeWriter) writers.get(type);
        if(writer == null) {
            throw new ClassNotFoundException(
                "There is not a Writer bound to '" + type + "'.");
        }
        Debugger.println("NodeBuilder", "lookupWriter",
            "Type> " + type + ", Class>" + writer.getClass().getName());
        return writer;
    }

    public static Iterator writerTypes() {
        return writers.keySet().iterator();
    }

    public static void bindNodeProcessor(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeProcessor processor = (NodeProcessor) Class.forName(classType).
                                  newInstance();
        bindNodeProcessor(processor);
    }

    public static void bindNodeProcessor(NodeProcessor processor) {
        bindNodeProcessor(processor.getTypes(), processor);
    }

    public static void bindNodeProcessor(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        NodeProcessor processor = (NodeProcessor) Class.forName(classType).
                                  newInstance();
        bindNodeProcessor(types, processor);
    }

    public static void bindNodeProcessor(String[] types,
        NodeProcessor processor) {
        for(int i = 0; i < types.length; i++) {
            processors.put(types[i], processor);
            Debugger.println("NodeBuilder", "bindProcessor",
                "Type> " + types[i] + ", Class>" + processor.getClass().getName());
        }
    }

    public static NodeProcessor lookupNodeProcessor(String type) throws
        ClassNotFoundException {
        NodeProcessor processor = (NodeProcessor) processors.get(type);
        if(processor == null) {
            throw new ClassNotFoundException(
                "There is not a Processor bound to '" + type + "'.");
        }
        Debugger.println("NodeBuilder", "lookupProcessor",
            "Type> " + type + ", Class>" + processor.getClass().getName());
        return processor;
    }

    public static Iterator processorTypes() {
        return processors.keySet().iterator();
    }

    public static void bindNodeConversor(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeConversor conversor = (NodeConversor) Class.forName(classType).
                                  newInstance();
        bindNodeConversor(conversor);
    }

    public static void bindNodeConversor(NodeConversor conversor) {
        bindNodeConversor(conversor.getTypes(), conversor);
    }

    public static void bindNodeConversor(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        NodeConversor conversor = (NodeConversor) Class.forName(classType).
                                  newInstance();
        bindNodeConversor(types, conversor);
    }

    public static void bindNodeConversor(String[] types,
        NodeConversor conversor) {
        for(int i = 0; i < types.length; i++) {
            conversors.put(types[i], conversor);
            Debugger.println("NodeBuilder", "bindConversor",
                "Type> " + types[i] + ", Class>" + conversor.getClass().getName());
        }
    }

    public static NodeConversor lookupNodeConversor(String type) throws
        ClassNotFoundException {
        NodeConversor conversor = (NodeConversor) conversors.get(type);
        if(conversor == null) {
            throw new ClassNotFoundException(
                "There is not a Conversor bound to '" + type + "'.");
        }
        Debugger.println("NodeBuilder", "lookupConversor",
            "Type> " + type + ", Class>" + conversor.getClass().getName());
        return conversor;
    }

    public static Iterator conversorTypes() {
        return conversors.keySet().iterator();
    }

    public static void bindNodeMatcher(String classType) throws
        ClassNotFoundException, IllegalAccessException, InstantiationException {
        NodeMatcher matcher = (NodeMatcher) Class.forName(classType).
                              newInstance();
        bindNodeMatcher(matcher);
    }

    public static void bindNodeMatcher(NodeMatcher matcher) {
        bindNodeMatcher(matcher.getTypes(), matcher);
    }

    public static void bindNodeMatcher(String[] types,
        String classType) throws ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        NodeMatcher matcher = (NodeMatcher) Class.forName(classType).
                              newInstance();
        bindNodeMatcher(types, matcher);
    }

    public static void bindNodeMatcher(String[] types,
        NodeMatcher matcher) {
        for(int i = 0; i < types.length; i++) {
            matchers.put(types[i], matcher);
            Debugger.println("NodeBuilder", "bindMatcher",
                "Type> " + types[i] + ", Class>" + matcher.getClass().getName());
        }
    }

    public static NodeMatcher lookupNodeMatcher(String type) throws
        ClassNotFoundException {
        NodeMatcher matcher = (NodeMatcher) matchers.get(type);
        if(matcher == null) {
            throw new ClassNotFoundException(
                "There is not a Matcher bound to '" + type + "'.");
        }
        Debugger.println("NodeBuilder", "lookupMatcher",
            "Type> " + type + ", Class>" + matcher.getClass().getName());
        return matcher;
    }

    public static Iterator matcherTypes() {
        return matchers.keySet().iterator();
    }

    public static void main(String[] args) {
        try {
            NodeFactory obj = NodeBuilder.getNodeFactory();
            if(obj != null) {
                System.out.println("Factory is a instance of '"
                    + obj.getClass().getName() + "'.");
            }
            else {
                System.out.println("Factory is null.");
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}