package jpel.util;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
     * Classe usada para a depurao do software. Escreve mensagens na sada de forma
 * padronizada.
 */
public class Debugger {

    private static final String DEBUGGER_JVM_PARAMETER = "debug";
    private static final String DEBUGGER_SINK_JVM_PARAMETER = "debugSink";

    private static boolean debug;
    private static DebuggerSink sink;
    private static SimpleDateFormat format;

    static {
        debug = new Boolean(System.getProperty(DEBUGGER_JVM_PARAMETER,
                "false")).booleanValue();
        try {
            sink = (DebuggerSink) Class.forName(
                   System.getProperty(
                   DEBUGGER_SINK_JVM_PARAMETER,
                   "jpel.util.DebuggerSinkDefault"
                   )
                   ).newInstance();
        }
        catch(Exception exc) {
            exc.printStackTrace();
        }
        format = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
    };

    /**
     * Habilita a depurao.
     */
    public static void enableDebug() {
        debug = true;
    }

    /**
     * Desabilita a depurao.
     */
    public static void disableDebug() {
        debug = false;
    }

    /**
     * Ajusta o destino do log.
     * @param sink O destino do log.
     */
    public static void setDebuggerSink(DebuggerSink newSink) {
        sink = newSink;
    }

    /**
     * Retorna o destino dos logs.
     * @return O destino dos log. O default  o System.out;
     */
    public static DebuggerSink getDebuggerSink() {
        return sink;
    }

    /**
     * Mtodo usado para impresso padro.
         * @param className Nome da classe que escreveu a mensagem, ou seu nickname.
         * @param header Que trecho da classe est escrevendo uma mensagem, por exemplo
     * o nome do mtodo que est escrevendo no log.
     * @param message Mensagem que dese ser escrita na sada.
     */
    public static void println(String className, String header, String message) {
        //System.out.println("ON_PRINTLN("+className+","+header+","+message+"");
        if(debug) {
            sink.write("[" + format.format(new Date()) + "] [" + className
                + "] [" + header + "] [" + message + "]");
        }
    }

    /**
     * Imprime uma exceo.
     * @param exc Exceo a ser impressa.
     */
    public static void println(Throwable exc) {
        if(debug) {
            sink.write("[" + format.format(new Date()) + "] ["
                + exc.getClass().getName() + "]");
            try {
                StringWriter str = new StringWriter();
                PrintWriter out = new PrintWriter(str);
                exc.printStackTrace(out);
                out.close();
                sink.write(str.toString());
                str.close();
            }
            catch(IOException error) {
                sink.write(error.getMessage());
            }
        }
    }
}
