package jpel.util;

/**
 * Permite formatao de textos para facilitar o processo de escrita
 * e leitura de documento onde existe uma indentao.
 */

public class StringFormater {

    /**
     * Coloca uma string com a mesma formatao de colunas.
     * @param obj Objeto que deve ser escrito.
     * @param gap Nmero de colunas em branco antes de cada linha.
     * @return A String formatada.
     */
    public static String toWrite(Object obj, String gap) {
        return toWrite(String.valueOf(obj), gap);
    }

    /**
     * Mesmo que toWrite(Object obj,gap).
     * @param str String qualquer.
     * @param gap Nmero de colunas.
     * @return String formatada.
     */
    public static String toWrite(String str, String gap) {
        return replace(str, "\n", "\n" + gap);
    }

    /**
     * Realiza a substituio de Strings.
     * @param str String original.
     * @param pattern Padro que deve ser substituido.
     * @param replace Padro que deve ser colocado.
     * @return A String modificada.
     */
    private static String replace(String str, String pattern, String replace) {
        int s = 0;
        int e = 0;
        int pattern_length = pattern.length();
        StringBuffer result = new StringBuffer(str.length());
        while((e = str.indexOf(pattern, s)) >= 0) {
            result.append(str.substring(s, e));
            result.append(replace);
            s = e + pattern_length;
        }
        result.append(str.substring(s));
        return result.toString();
    }

    /**
         * Remove as linhas e espaos em branco antes e depois do texto, normalizando
     * tambm as colunas para o menor possvel.
     * <br>Exemplo:
     * <br> ANTES:
     * <br>
     * <font color="#FF0000">
     * '&nbsp;&nbsp;&nbsp;<br>
     *  &nbsp;&nbsp;&nbsp;Exemplo<br>
     *  &nbsp;&nbsp;de<br>
     *  &nbsp;&nbsp;&nbsp;&nbsp;normalizao.<br>
     *  &nbsp;&nbsp;'
     * </font>
     * <br> DEPOIS:
     * <br>
     * <font color="#0000FF">
     * '&nbsp;Exemplo<br>
     *  de<br>
     *  &nbsp;&nbsp;normalizao.'
     * @param str String que ser "limpa"
     * @return A String formatada.
     */
    public static String toRead(String str) {
        int size = str.length();
        char c = '\u0000';
        int start = 0;
        while(start < size
            && ((c = str.charAt(start)) == ' ' || c == '\n')) {
            start++;
        }
        while(size > 0 && start >= 0 && (c = str.charAt(start)) != '\n') {
            start--;
        }
        if(c == '\n') {
            start++;
        }

        int end = str.length() - 1;
        while(end >= 0 && ((c = str.charAt(end)) == ' ' || c == '\n')) {
            end--;
        }
        while(size > 0 && end < size && (c = str.charAt(end)) != '\n') {
            end++;
        }
        if(c == '\n') {
            end--;
        }
        str = str.substring(Math.max(start, 0), Math.min(end + 1, size));

        int counter = 0;
        int minimo = Integer.MAX_VALUE;
        int index = 0;
        size = str.length();
        boolean read = true;
        while(index < size) {
            c = str.charAt(index++);
            if(c == '\n') {
                read = true;
                if(counter < minimo) {
                    minimo = counter;
                }
                counter = 0;
            }
            else {
                if(read) {
                    if(!Character.isWhitespace(c)) {
                        read = false;
                    }
                    else {
                        counter++;
                    }
                }
            }
        }
        if(counter < minimo) {
            minimo = counter;
        }
        StringBuffer result = new StringBuffer();
        index = minimo;
        while(index < str.length()) {
            c = str.charAt(index);
            result.append(c);
            if(c == '\n') {
                index += minimo + 1;
            }
            else {
                index++;
            }
        }
        return result.toString();
    }

    public static void main(String[] args) {
        String str = toRead("");
        str = toRead("    s");
    }
}
