package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;

/**
 * Implementao de uma fonte de recursos para InputStreams.
 */
public class DataHolderBinaryStream extends AbstractDataHolder {

    private InputStream in;
    private OutputStream out;

    public DataHolderBinaryStream(InputStream in) {
        this(in, null);
    }

    public DataHolderBinaryStream(OutputStream out) {
        this(null, out);
    }

    public DataHolderBinaryStream(InputStream in, OutputStream out) {
        super(DataHolder.BINARYSTREAM);
        setIn(in);
        setOut(out);
    }

    public void setIn(InputStream in) {
        this.in = in;
    }

    public InputStream getIn() {
        return in;
    }

    public void setOut(OutputStream out) {
        this.out = out;
    }

    public OutputStream getOut() {
        return out;
    }

    public void getReference(Object reference) {
        if(reference instanceof InputStream) {
            setIn((InputStream) reference);
        }
        else if(reference instanceof OutputStream) {
            setOut((OutputStream) reference);
        }
        else {
            throw new IllegalArgumentException(
                "Reference must be either an InputStream or an OutputStream object.");
        }
    }

    public Object getReference() {
        if(in != null) {
            return in;
        }
        else {
            return out;
        }
    }

    public DataHolder resolve(DataHolder relative) throws DataHolderException {
        throw new DataHolderException(
            "Resolve not allowed to BinaryStream source.");
    }

    public DataHolder relative(DataHolder absolute) throws
        DataHolderException {
        throw new DataHolderException(
            "Relative not allowed to BinaryStream source.");
    }

    public long lastModified() {
        return DATE_UNKNOWN;
    }

    public InputStream getInputStream() throws DataHolderException {
        setBInputStream(new BufferedInputStream(in));
        return getBInputStream();
    }

    public OutputStream getOutputStream() throws DataHolderException {
        setBOutputStream(new BufferedOutputStream(out));
        return getBOutputStream();
    }

    public Reader getReader() throws DataHolderException {
        setBReader(new BufferedReader(new InputStreamReader(getInputStream())));
        return getBReader();
    }

    public Writer getWriter() throws DataHolderException {
        setBWriter(new BufferedWriter(new OutputStreamWriter(getOutputStream())));
        return getBWriter();
    }

    public String toString() {
        return(in != null ? "InputStream(" + in.getClass().getName() + ")"
            : "") + (out != null
            ? "OutputStream(" + out.getClass().getName() + ")"
            : "");
    }
}
