package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

import jpel.util.Debugger;

/**
 * Classe utilitria para a criao de dinmica dos elementos da linguagem.
 */
public class DataHolderBuilder {

    private static final String JVM_PARAMETER = "dataHolder";
    private static final String COMMENT = "#";
    private static final String TYPE_MARK = "::";
    private static final String TYPE_TOKEN = ",";

    private static HashMap extractors;

    static {
        extractors = new HashMap();
        try {
            InputStream in;
            BufferedReader bin;
            String input;

            in = DataHolderBuilder.class.getResourceAsStream("holders.lib");
            bin = new BufferedReader(new InputStreamReader(in));
            while((input = bin.readLine()) != null) {
                String str = input.trim();
                if(str.startsWith(COMMENT) || str.equals("")) {
                    continue;
                }
                int pos;
                if((pos = str.indexOf(TYPE_MARK)) > 0) {
                    StringTokenizer st = new StringTokenizer(str.substring(0,
                                         pos).trim(), TYPE_TOKEN);
                    str = str.substring(pos + TYPE_MARK.length()).trim();
                    String[] types = new String[st.countTokens()];
                    int index = 0;
                    while(st.hasMoreTokens()) {
                        types[index++] = st.nextToken();
                    }
                    bindDataHolderExtractor(types, str);
                }
                else {
                    bindDataHolderExtractor(str);
                }
            }
            bin.close();
            in.close();
        }
        catch(Exception exc) {
            exc.printStackTrace();
            System.exit(0);
        }
    }

    public static DataHolderFactory getDataHolderFactory() throws
        ClassNotFoundException, IllegalAccessException,
        InstantiationException {
        String classType = System.getProperty(JVM_PARAMETER,
                           "jpel.util.dataholder.DataHolderFactoryImpl");
        return getDataHolderFactory(classType);
    }

    public static DataHolderFactory getDataHolderFactory(String classType) throws
        ClassNotFoundException, IllegalAccessException,
        InstantiationException {
        DataHolderFactory parser = (DataHolderFactory) Class.forName(
                                   classType).newInstance();
        Debugger.println("DataHolderBuilder", "parser",
            "Class>" + parser.getClass().getName());
        return parser;
    }

    public static void bindDataHolderExtractor(String classType) throws
        ClassNotFoundException, IllegalAccessException,
        InstantiationException {
        DataHolderExtractor loader = (DataHolderExtractor) Class.forName(
                                     classType).newInstance();
        bindDataHolderExtractor(loader);
    }

    public static void bindDataHolderExtractor(DataHolderExtractor loader) {
        bindDataHolderExtractor(loader.getTypes(), loader);
    }

    public static void bindDataHolderExtractor(String[] types,
        String classType) throws
        ClassNotFoundException,
        IllegalAccessException, InstantiationException {
        DataHolderExtractor loader = (DataHolderExtractor) Class.forName(
                                     classType).newInstance();
        bindDataHolderExtractor(types, loader);
    }

    public static void bindDataHolderExtractor(String[] types,
        DataHolderExtractor loader) {
        for(int i = 0; i < types.length; i++) {
            extractors.put(types[i], loader);
            Debugger.println("DataHolderBuilder", "bindLoader",
                "Type> " + types[i] + ", Class>"
                + loader.getClass().getName());
        }
    }

    public static DataHolderExtractor lookupDataHolderExtractor(String type) throws
        ClassNotFoundException {
        DataHolderExtractor loader = (DataHolderExtractor) extractors.get(
                                     type);
        if(loader == null) {
            throw new ClassNotFoundException(
                "There is not a Loader bound to '" + type + "'.");
        }
        Debugger.println("DataHolderBuilder", "lookupLoader",
            "Type> " + type + ", Class>"
            + loader.getClass().getName());
        return loader;
    }

    public static Iterator loaderTypes() {
        return extractors.keySet().iterator();
    }

    public static void main(String[] args) {
        try {
            DataHolderFactory obj = DataHolderBuilder.getDataHolderFactory();
            if(obj != null) {
                System.out.println("Factory is a instance of '"
                    + obj.getClass().getName() + "'.");
            }
            else {
                System.out.println("Factory is null.");
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
