package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;

/**
 * Implementao de uma fonte de recursos para Readers.
 */
public class DataHolderCharacterStream extends AbstractDataHolder {

    private Reader in;
    private Writer out;

    public DataHolderCharacterStream(Reader in) {
        this(in, null);
    }

    public DataHolderCharacterStream(Writer out) {
        this(null, out);
    }

    public DataHolderCharacterStream(Reader in, Writer out) {
        super(DataHolder.CHARACTERSTREAM);
        setIn(in);
        setOut(out);
    }

    public void setIn(Reader in) {
        this.in = in;
    }

    public Reader getIn() {
        return in;
    }

    public void setOut(Writer out) {
        this.out = out;
    }

    public Writer getOut() {
        return out;
    }

    public void getReference(Object reference) {
        if(reference instanceof Reader) {
            setIn((Reader) reference);
        }
        else if(reference instanceof Writer) {
            setOut((Writer) reference);
        }
        else {
            throw new IllegalArgumentException(
                "Reference must be either a Reader or a Writer object.");
        }
    }

    public Object getReference() {
        if(in != null) {
            return in;
        }
        else {
            return out;
        }
    }

    public DataHolder resolve(DataHolder relative) throws DataHolderException {
        throw new DataHolderException(
            "Resolve not allowed to CharacterStream source.");
    }

    public DataHolder relative(DataHolder absolute) throws
        DataHolderException {
        throw new DataHolderException(
            "Relative not allowed to CharacterStream source.");
    }

    public long lastModified() {
        return DATE_UNKNOWN;
    }

    public InputStream getInputStream() throws DataHolderException {
        //return new BufferedInputStream(in);
        return null;
    }

    public OutputStream getOutputStream() throws DataHolderException {
        //return new BufferedWriter(out);
        return null;
    }

    public Reader getReader() throws DataHolderException {
        setBReader(new BufferedReader(in));
        return getBReader();
    }

    public Writer getWriter() throws DataHolderException {
        setBWriter(new BufferedWriter(out));
        return getBWriter();
    }

    public String toString() {
        return(in != null ? "Reader(" + in.getClass().getName() + ")"
            : "")
            + (out != null ? "Writer(" + out.getClass().getName() + ")"
            : "");
    }
}
