package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;

/**
 * Implementao de uma fonte de recursos para arquivos.
 */
public class DataHolderFile extends AbstractDataHolder {

    private File file;

    public DataHolderFile(String file) {
        this(new File(file.trim()));
    }

    public DataHolderFile(File file) {
        super(DataHolder.FILE);
        setFile(file);
    }

    public void setFile(File newFile) {
        file = newFile;
    }

    public File getFile() {
        return file;
    }

    public void setReference(Object reference) {
        if(reference instanceof File) {
            setFile((File) reference);
        }
        else {
            throw new IllegalArgumentException(
                "Reference must be a File object.");
        }
    }

    public Object getReference() {
        return getFile();
    }

    public DataHolder resolve(DataHolder relative) throws DataHolderException {
        // se houver import de URL pode ser feito diretamente
        if(relative.getType() == DataHolder.URL) {
            return relative;
        }
        File result = null;
        File parent = file.getParentFile();
        if(parent == null) {
            parent = file.getAbsoluteFile().getParentFile();
        }
        File other = (File) relative.getReference();
        String target = other.toString();
        String token = ".." + File.separator;
        int pos = target.indexOf(token);
        while(pos >= 0 && parent != null) {
            parent = parent.getParentFile();
            target = target.substring(pos + token.length());
            pos = target.indexOf(token);
        }
        if(parent == null) {
            throw new DataHolderException("Relative include '"
                + relative.getReference()
                +
                "' fail, there is no root directory.");
        }
        result = new File(parent, target);
        DataHolder holder = new DataHolderFile(result);
        //holder.setDescription(relative.getDescription());
        return holder;
    }

    public DataHolder relative(DataHolder absolute) throws
        DataHolderException {
        if(absolute.getType() == DataHolder.URL) {
            return absolute;
        }
        File local = getFile();
        File other = (File) absolute.getReference();

        String strLocal = local.toString();
        String strOther = other.toString();
        int posL = 0, posO = 0;
        while((posL = strLocal.indexOf(File.separator, posL)) >= 0 &&
                      (posO = strOther.indexOf(File.separator,
                              posO)) >= 0) {
            if(strLocal.substring(0, posL).equals(strOther.substring(0,
                posO))) {
                strLocal = strLocal.substring(posL + 1);
                strOther = strOther.substring(posO + 1);
                posL = 0;
                posO = 0;
            }
            else {
                break;
            }
        }
        if(!strLocal.equals(local.toString())) {
            posL = 0;
            while((posL = strLocal.indexOf(File.separator, posL)) > 0) {
                strOther = ".." + File.separator + strOther;
                posL++;
            }
        }
        File result = new File(strOther);
        DataHolder holder = new DataHolderFile(result);
        //holder.setDescription(absolute.getDescription());
        return holder;
    }

    public long lastModified() {
        return file.lastModified();
    }

    public InputStream getInputStream() throws DataHolderException {
        try {
            if(!file.exists()) {
                throw new DataHolderException("File '" + file + "' not found!");
            }
            setBInputStream(new BufferedInputStream(new FileInputStream(file)));
            return getBInputStream();
        }
        catch(FileNotFoundException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    public OutputStream getOutputStream() throws DataHolderException {
        try {
//            if( !file.exists() ) {
//                throw new DataHolderException("File '"+file+"' not found!");
//            }
            setBOutputStream(new BufferedOutputStream(new FileOutputStream(
                file)));
            return getBOutputStream();
        }
        catch(FileNotFoundException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    public Reader getReader() throws DataHolderException {
        setBReader(new BufferedReader(new InputStreamReader(getInputStream())));
        return getBReader();
    }

    public Writer getWriter() throws DataHolderException {
        setBWriter(new BufferedWriter(new OutputStreamWriter(getOutputStream())));
        return getBWriter();
    }

    public boolean equals(Object obj) {
        boolean result = false;
        if(obj instanceof DataHolderFile) {
            result = file.equals(((DataHolderFile) obj).file);
        }
        return result;
    }

    public String toString() {
        return "File(" + file + ")";
    }

    public static void main(String[] args) {
        try {
            DataHolder is = new DataHolderFile(new File(args[0].trim()));
            System.out.println("Base file: " + is);
            DataHolder src, aux;
            for(int i = 1; i < args.length; i++) {
                try {
                    src = new DataHolderFile(new File(args[i].trim()));
                    System.out.println(src);
                    aux = is.resolve(src);
                    System.out.println("  Resolve:" + aux);
                    aux = is.relative(src);
                    System.out.println("  Relative:" + aux);
                    //Reader reader = aux.getReader();
                    //reader.close();
                }
                catch(Exception exc) {
                    exc.printStackTrace();
                }
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
