package jpel.util.dataholder;

/**
 * Title:        Configuration in Java
 * Description:  Ferramentas e tcnicas para ajuste de sistemas feitos em Java.
 * Copyright:    Copyright (c) 2002
 * Company:      CIN - UFPE
 * @author
 * @version 1.0
 */

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Implementao de uma fonte de recursos para URLs.
 */
public class DataHolderURL extends AbstractDataHolder {

    private URL url;
    private HttpURLConnection connection;

    public DataHolderURL(String url) throws MalformedURLException {
        this(new URL(url.trim()));
    }

    public DataHolderURL(URL url) {
        super(DataHolder.URL);
        setURL(url);
    }

    public void setURL(URL newURL) {
        url = newURL;
    }

    public URL getURL() {
        return url;
    }

    public void setReference(Object reference) {
        if(reference instanceof URL) {
            setURL((URL) reference);
        }
        else {
            throw new IllegalArgumentException(
                "Reference must be an URL object.");
        }
    }

    public Object getReference() {
        return getURL();
    }

    public DataHolder resolve(DataHolder relative) throws DataHolderException {
        // se houver import de URL pode ser feito diretamente
        if(relative.getType() == DataHolder.URL) {
            return relative;
        }
        URL result = null;
        try {
            String path = relative.getReference().toString();
            path.replace(File.separatorChar, '/');
            result = new URL(url, path);
        }
        catch(MalformedURLException exc) {
            throw new DataHolderException("Could not resolve '" + relative
                + "' to '" + url + "'");
        }
        DataHolder holder = new DataHolderURL(result);
        //holder.setDescription(relative.getDescription());
        return holder;
    }

    public DataHolder relative(DataHolder absolute) throws
        DataHolderException {
        throw new DataHolderException("Relative not allowed to URL source.");
    }

    public long lastModified() {
        if(connection != null) {
            return connection.getLastModified();
        }
        else {
            return DATE_UNKNOWN;
        }
    }

    public InputStream getInputStream() throws DataHolderException {
        try {
            connection = (HttpURLConnection) url.openConnection();
            if(connection.getResponseCode() != HttpURLConnection.HTTP_OK) {
                throw new DataHolderException("URL '" + url + "' not found! "
                    + connection.getResponseMessage());
            }
            setBInputStream(new BufferedInputStream(connection.getInputStream()));
            return getBInputStream();
        }
        catch(IOException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    public OutputStream getOutputStream() throws DataHolderException {
        try {
            connection = (HttpURLConnection) url.openConnection();
            if(connection.getResponseCode() != HttpURLConnection.HTTP_OK) {
                throw new DataHolderException("URL '" + url + "' not found! "
                    + connection.getResponseMessage());
            }
            setBOutputStream(new BufferedOutputStream(connection.
                getOutputStream()));
            return getBOutputStream();
        }
        catch(IOException exc) {
            throw new DataHolderException(exc.getMessage(), exc);
        }
    }

    public Reader getReader() throws DataHolderException {
        close();
        setBReader(new BufferedReader(new InputStreamReader(getInputStream())));
        return getBReader();
    }

    public Writer getWriter() throws DataHolderException {
        close();
        setBWriter(new BufferedWriter(new OutputStreamWriter(getOutputStream())));
        return getBWriter();
    }

    public void close() throws DataHolderException {
        super.close();
        if(connection != null) {
            connection.disconnect();
            connection = null;
        }
    }

    public boolean equals(Object obj) {
        boolean result = false;
        if(obj instanceof DataHolderURL) {
            result = url.equals(((DataHolderURL) obj).url);
        }
        return result;
    }

    public String toString() {
        return "URL(" + url + ")";
    }

    public static void main(String[] args) {
        try {
            DataHolder is = new DataHolderURL(new URL(args[0].trim()));
            is.getReader().close();

            System.out.println(is);
            DataHolder aux;
            for(int i = 1; i < args.length; i++) {
                try {
                    aux = is.resolve(new DataHolderURL(new URL(args[i].trim())));
                    System.out.println(aux);
                    Reader reader = aux.getReader();
                    reader.close();
                }
                catch(Exception exc) {
                    exc.printStackTrace();
                }
            }
        }
        catch(Exception ex) {
            ex.printStackTrace();
        }
    }
}
